// XBaseDefinitions.h:
// -------------------------------------------------------------------
// This file is a part of "XTL" available at:
// http://www.codecommunity.com
//
// XTL is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
// 
// XTL is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with XTL. If not, see <http://www.gnu.org/licenses/>.
//
// coded by: skulk@codecommunity.com
// modified: 2006-09-02 (skulk)
//////////////////////////////////////////////////////////////////////
#ifdef X_WINDOWS
	#pragma once
#else // X_WINDOWS
	#error "Missing definition..."
#endif

#include "XBaseTypes.h"



// #######################################################################################



// ***************************************************************************************
// No operation...
// ***************************************************************************************
#ifdef X_WINDOWS
	#if (_MSC_VER > 1200)
		#define _xnooperation __noop
	#else // _MSC_VER > 1200
		#define _xnooperation void(0)
	#endif
#else // WIN32
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// Calling conventions...
// ***************************************************************************************
#ifdef X_WINDOWS
	#define _xstdcall	__stdcall
	#define _xcdecl		__cdecl
#else // WIN32
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// Attribute specifiers...
// ***************************************************************************************
#ifdef X_WINDOWS
	#define _xnovtable	__declspec(novtable)
	#define _xselectany	__declspec(selectany)
#else // WIN32
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// Pure virtual specifier...
// ***************************************************************************************
#define xpure = 0



// #######################################################################################



// ***************************************************************************************
// "_xinterface" specifier...
// ***************************************************************************************
#define _xinterface struct



// #######################################################################################



// ***************************************************************************************
// "sizeof" for array types...
// ***************************************************************************************
#define	_XCount(__Buffer, __Type)	((xuint32) (sizeof(__Buffer) / sizeof(__Type)))



// #######################################################################################



// ***************************************************************************************
// "sizeof" for string array types...
// ***************************************************************************************
#define _XCountCharT(__Buffer)	_XCount(__Buffer, xchart)
#define _XCountCharA(__Buffer)	_XCount(__Buffer, xchara)
#define _XCountCharW(__Buffer)	_XCount(__Buffer, xcharw)
#define _XCountCharI(__Buffer)	_XCount(__Buffer, xchari)



// #######################################################################################



// ***************************************************************************************
// "xresult" definitions...
//
//   Scheme:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  | S |C|R|           F           |               C               |
//  +---+-+-+-----------------------+-------------------------------+
//
//  S - Severity code (00 = Success, 01 = Information, 10 - Warning, 11 - Error)
//  C - Customer flag
//  R - Reserved
//  F - Facility code
//  C - Code
// ***************************************************************************************
#define _XResultCode(__Result)		((__Result) & 0xFFFF)
#define _XResultFacility(__Result)	(((__Result) >> 16) & 0x1FFF)
#define _XResultSeverity(__Result)	(((__Result) >> 31) & 0x1)



// #######################################################################################



// ***************************************************************************************
// "xresult" severity...
// ***************************************************************************************
#define _XResultSeveritySuccess	0
#define _XResultSeverityWarning	0
#define _XResultSeverityError	1



// #######################################################################################



// ***************************************************************************************
// "xresult" facility base...
// ***************************************************************************************
#define _XResultFacilityBase		0xFF



// #######################################################################################



// ***************************************************************************************
// Create a xresult...
// ***************************************************************************************
#define _XResultCreate(__Severity, __Facility, __Code) \
	( \
		(xresult) (((xulong) (__Severity) << 31) | ((xulong) (__Facility) << 16) | ((xulong) (__Code))) \
	)

#define _XResultCreateSuccess(__Code) \
	( \
		_XResultCreate(_XResultSeveritySuccess, _XResultFacilityBase, __Code) \
	)

#define _XResultCreateWarning(__Code) \
	( \
		_XResultCreate(_XResultSeveritySuccess, _XResultFacilityBase, __Code) \
	)

#define _XResultCreateError(__Code) \
	( \
		_XResultCreate(_XResultSeverityError, _XResultFacilityBase, __Code) \
	)

#ifdef X_WINDOWS
	#define _XResultFromHResult(__HResult)	((xresult) __HResult)
	#define _XResultFromWin32				HRESULT_FROM_WIN32
	#define _XResultLastError				_XResultFromWin32(::GetLastError())
#endif // X_WINDOWS



// #######################################################################################



// ***************************************************************************************
// "xresult" definitions (success)...
// ***************************************************************************************
enum _XResultSuccess
{
	xsOK				= _XResultCreateSuccess(0x0000),
	xsTrue				= _XResultCreateSuccess(0x0001),
	xsFalse				= _XResultCreateSuccess(0x0002)
};



// #######################################################################################



// ***************************************************************************************
// "xresult" definitions (warning)...
// ***************************************************************************************
enum _XResultWarning
{
};



// #######################################################################################



// ***************************************************************************************
// "xresult" definitions (error)...
// ***************************************************************************************
enum _XResultError
{
	xeAbort				= _XResultCreateError(0x0000),
	xeAccessDenied		= _XResultCreateError(0x0001),
	xeException			= _XResultCreateError(0x0002),
	xeFail				= _XResultCreateError(0x0003),
	xeFalse				= _XResultCreateError(0x0004),
	xeNotImplemented	= _XResultCreateError(0x0005),
	xeNoInterface		= _XResultCreateError(0x0006),
	xeInternal			= _XResultCreateError(0x0007),
	xeInvalidArgument	= _XResultCreateError(0x0008),
	xeOutOfMemory		= _XResultCreateError(0x0009),
	xePending			= _XResultCreateError(0x000A),
	xePointer			= _XResultCreateError(0x000B),
	xeTerminate			= _XResultCreateError(0x000C),
	xeUnexpected		= _XResultCreateError(0x000D),
	xeVersion			= _XResultCreateError(0x000E)
};



// #######################################################################################



// ***************************************************************************************
// Interface declaration...
// ***************************************************************************************
#define __XInterfaceDeclare(__Type, __Base) _xinterface _xnovtable __Type : public __Base



// #######################################################################################



// ***************************************************************************************
// Interface declaration...
// ***************************************************************************************
#define _XInterfaceDeclare(__Type) __XInterfaceDeclare(__Type, IXInterface)



// #######################################################################################



// ***************************************************************************************
// Interface method declaration...
// ***************************************************************************************
#define _XInterfaceMethod(__Type, __Method) virtual __Type _xstdcall __Method



// #######################################################################################



// ***************************************************************************************
// Known xguid's...
// ***************************************************************************************
// {00000000-0000-0000-0000-000000000000}
static xcguid IID_XNull = 
{ 0x00000000, 0x0000, 0x0000, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } };



// #######################################################################################



// ***************************************************************************************
// Compile time text conversions...
// ***************************************************************************************
#ifdef X_WINDOWS
	#define __XA(__String)	__String
	#define __XW(__String)	L##__String
	#define __XT(__String)	_T(__String)
	#define __XI(__String)	__XW(__String)
#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// Thread function...
// ***************************************************************************************
typedef xdword (_xstdcall *_XThreadProc)(xvoid* pData);



// #######################################################################################



// ***************************************************************************************
// Property types...
// ***************************************************************************************
enum _XPropertyTypes
{
	XPropertyType_String	= 0x0001,
	XPropertyType_Binary	= 0x0002,
	XPropertyType_Interface	= 0x0003,
	XPropertyType_Type		= 0x0004
};



// #######################################################################################



// ***************************************************************************************
// Create a property ID...
// ***************************************************************************************
#define _XPropertyCreate(__Type, __ID) \
	( \
		(xulong) (((xulong) (__Type) << 16) | ((xulong) (__ID))) \
	)

#define _XPropertyCreateString(__ID) \
	( \
		_XPropertyCreate(XPropertyType_String, __ID) \
	)

#define _XPropertyCreateBinary(__ID) \
	( \
		_XPropertyCreate(XPropertyType_Binary, __ID) \
	)

#define _XPropertyCreateInterface(__ID) \
	( \
		_XPropertyCreate(XPropertyType_Interface, __ID) \
	)

#define _XPropertyCreateType(__ID) \
	( \
		_XPropertyCreate(XPropertyType_Type, __ID) \
	)



// #######################################################################################



// ***************************************************************************************
// Retrieve the property type...
// ***************************************************************************************
#define _XPropertyType(__Property) \
	( \
		((xulong) (__Property & 0xFFFF0000) >> 16) \
	)



// #######################################################################################



// ***************************************************************************************
// Retrieve the property ID...
// ***************************************************************************************
#define _XPropertyID(__Property) \
	( \
		__Property & 0x0000FFFF \
	)



// #######################################################################################



// ***************************************************************************************
// Path separator...
// ***************************************************************************************
#ifdef X_WINDOWS
	#define _XPathSeparator	__XT('\\')
#else // X_WINDOWS
	#define _XPathSeparator	__XT('/')
#endif



// #######################################################################################



// ***************************************************************************************
// Module extension...
// ***************************************************************************************
#ifdef X_WINDOWS
	#define _XModuleExtension	__XT(".dll")
#else // X_WINDOWS
	#define _XModuleExtension	__XT(".so")
#endif



// #######################################################################################



// ***************************************************************************************
// Module extension...
// ***************************************************************************************
#ifdef X_WINDOWS
	#define _XPathMax	_MAX_PATH
#else // X_WINDOWS
	#define _XPathMax	PATH_MAX
#endif



// #######################################################################################



// ***************************************************************************************
// Infinite definition...
// ***************************************************************************************
#define _XInfinite		~0u



// #######################################################################################



// ***************************************************************************************
// Unicode BOM definition...
// ***************************************************************************************
#define _XUnicodeBOM	0xFEFF



// #######################################################################################
