// XBaseFunctions.h:
// -------------------------------------------------------------------
// This file is a part of "XTL" available at:
// http://www.codecommunity.com
//
// XTL is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
// 
// XTL is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with XTL. If not, see <http://www.gnu.org/licenses/>.
//
// coded by: skulk@codecommunity.com
// modified: 2006-09-02 (skulk)
//////////////////////////////////////////////////////////////////////
#ifdef X_WINDOWS
	#pragma once
#else // WIN32
	#error "Missing definition..."
#endif

#include "XBaseDefinitions.h"



// #######################################################################################



// ***************************************************************************************
// Copy memory...
// ***************************************************************************************
inline static xvoid* __XMemoryCopy(xvoid* pTarget, xcvoid* pSource, xcuint32 uiSize)
{
	return memcpy(pTarget, pSource, uiSize);
}

#define _XMemoryCopy __XMemoryCopy



// #######################################################################################



// ***************************************************************************************
// Move memory...
// ***************************************************************************************
inline static xvoid* __XMemoryMove(xvoid* pTarget, xcvoid* pSource, xcuint32 uiSize)
{
	return memmove(pTarget, pSource, uiSize);
}

#define _XMemoryMove __XMemoryMove



// #######################################################################################



// ***************************************************************************************
// Compare memory...
// ***************************************************************************************
inline static xint32 __XMemoryCompare(xcvoid* p1, xcvoid* p2, xcuint32 uiSize)
{
	return memcmp(p1, p2, uiSize);
}

#define _XMemoryCompare __XMemoryCompare



// #######################################################################################



// ***************************************************************************************
// Set memory...
// ***************************************************************************************
inline static xvoid* __XMemorySet(xvoid* p, xcint32 iData, xcuint32 uiSize)
{
	return memset(p, iData, uiSize);
}

#define _XMemorySet __XMemorySet



// #######################################################################################



// ***************************************************************************************
// Zero memory...
// ***************************************************************************************
inline static xvoid* __XMemoryZero(xvoid* p, xcuint32 uiSize)
{
	return _XMemorySet(p, 0, uiSize);
}

#define _XMemoryZero __XMemoryZero



// #######################################################################################

	
	
// Shortcuts...
#define _xalloc(__Size)						_XMemoryAlloc(__Size)
#define _xrealloc(__Buffer, __Size)			_XMemoryRealloc(__Buffer, __Size)
#define _xfree(__Buffer)					_XMemoryFree(__Buffer)
#define _xnew								_XMemoryNew
#define _xdelete							_XMemoryDelete
	
// Mappings...
#define _XMemoryAlloc(__Size)				__XMemoryAlloc(__Size)
#define _XMemoryRealloc(__Buffer, __Size)	__XMemoryRealloc(__Buffer, __Size)
#define _XMemoryFree(__Buffer)				__XMemoryFree(__Buffer)
#define _XMemoryNew							new
#define _XMemoryDelete						delete

// Mappings (tracking)...
#ifdef _DEBUG
	#define _XMemoryTrack					_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF)
	#define _XMemoryDump					_CrtDumpMemoryLeaks()
#else // _DEBUG
	#define _XMemoryTrack					_xnooperation
	#define _XMemoryDump					_xnooperation
#endif



// #######################################################################################



// ***************************************************************************************
// Memory allocation...
// ***************************************************************************************
inline static xvoid* __XMemoryAlloc(xcuint32 uiSize)
{
#if defined _DEBUG && defined X_WINDOWS
	_CrtCheckMemory();
#endif

	xvoid* p = malloc(uiSize);
	if (p)
	{
		_XMemoryZero(p, uiSize);
	}

	return p;
}



// #######################################################################################



// ***************************************************************************************
// Memory reallocation...
// ***************************************************************************************
inline static xvoid* __XMemoryRealloc(xvoid* pBuffer, xcuint32 uiSize)
{
#if defined _DEBUG && defined X_WINDOWS
	_CrtCheckMemory();
#endif

	xvoid* p = realloc(pBuffer, uiSize);
	if (p)
	{
		_XMemoryZero(p, uiSize);
	}

	return p;
}



// #######################################################################################



// ***************************************************************************************
// Memory free...
// ***************************************************************************************
inline static xvoid __XMemoryFree(xvoid* pBuffer)
{
#if defined _DEBUG && defined X_WINDOWS
	_CrtCheckMemory();
#endif

	free(pBuffer);
}



// #######################################################################################



// ***************************************************************************************
// String functions...	
// ***************************************************************************************

#define _xstrlena(__1)						strlen(__1)
#define _xstrstra(__1, __2)					strstr(__1, __2)
#define _xstrchra(__1, __2)					strchr(__1, __2)
#define _xstrrchra(__1, __2)				strrchr(__1, __2)
#define _xstrcmpa(__1, __2)					strcmp(__1, __2)
#define _xstrncmpa(__1, __2, __3)			strncmp(__1, __2, __3)
#define _xstrcolla(__1, __2)				strcoll(__1, __2)
#define _xstricolla(__1, __2)				stricoll(__1, __2)
#define _xstrspna(__1, __2)					strspn(__1, __2)
#define _xstrcspna(__1, __2)				strcspn(__1, __2)
#define _xstrpbrka(__1, __2)				strpbrk(__1, __2)
#define _xstrtoka(__1, __2, __3)			strtok_s(__1, __2, __3)

#ifdef X_WINDOWS
	#define _xstricmpa(__1, __2)			_stricmp(__1, __2)
	#define _xstrcpya(__1, __2, __3)		strcpy_s(__1, __2, __3)
	#define _xstrncpya(__1, __2, __3, __4)	strncpy_s(__1, __2, __3, __4)
	#define _xstrcata(__1, __2, __3)		strcat_s(__1, __2, __3)
	#define _xstrupra(__1, __2)				strupr_s(__1, __2)
	#define _xstrlwra(__1, __2)				strlwr_s(__1, __2)
	#define _xstrreva(__1)					_strrev(__1)
#else // X_WINDOWS
	#define _xstricmpa(__1, __2)			strcasecmp(__1, __2)
	#define _xstrcpya(__1, __2, __3)		strcpy(__1, __3)
	#define _xstrncpya(__1, __2, __3, __4)	strncpy(__1, __3, __4)
	#define _xstrcata(__1, __2, __3)		strcat(__1, __3)
	#define _xstrupra(__1, __2)				strupr(__1, __2)
	#define _xstrlwra(__1, __2)				strlwr(__1, __2)
	#define _xstrreva(__1)					strrev(__1)
#endif

#define _xstrlenw(__1)						wcslen(__1)
#define _xstrstrw(__1, __2)					wcsstr(__1, __2)
#define _xstrchrw(__1, __2)					wcschr(__1, __2)
#define _xstrrchrw(__1, __2)				wcsrchr(__1, __2)
#define _xstrcmpw(__1, __2)					wcscmp(__1, __2)
#define _xstrncmpw(__1, __2, __3)			wcsncmp(__1, __2, __3)
#define _xstrcollw(__1, __2)				wcscoll(__1, __2)
#define _xstricollw(__1, __2)				_wcsicoll(__1, __2)
#define _xstrspnw(__1, __2)					wcsspn(__1, __2)
#define _xstrcspnw(__1, __2)				wcscspn(__1, __2)
#define _xstrpbrkw(__1, __2)				wcspbrk(__1, __2)
#define _xstrtokw(__1, __2, __3)			wcstok_s(__1, __2, __3)

#ifdef X_WINDOWS
	#define _xstricmpw(__1, __2)			_wcsicmp(__1, __2)
	#define _xstrcpyw(__1, __2, __3)		wcscpy_s(__1, __2, __3)
	#define _xstrncpyw(__1, __2, __3, __4)	wcsncpy_s(__1, __2, __3, __4)
	#define _xstrcatw(__1, __2, __3)		wcscat_s(__1, __2, __3)
	#define _xstruprw(__1, __2)				_wcsupr_s(__1, __2)
	#define _xstrlwrw(__1, __2)				_wcslwr_s(__1, __2)
	#define _xstrrevw(__1)					_wcsrev(__1)
#else // X_WINDOWS
	#error "Missing definition..."
#endif

#ifdef X_UNICODE
	#define _xstrlent		_xstrlenw
	#define _xstrstrt		_xstrstrw
	#define _xstrchrt		_xstrchrw
	#define _xstrrchrt		_xstrrchrw
	#define _xstrcmpt		_xstrcmpw
	#define _xstrncmpt		_xstrncmpw
	#define _xstrcollt		_xstrcollw
	#define _xstricollt		_xstricollw
	#define _xstrspnt		_xstrspnw
	#define _xstrcspnt		_xstrcspnw
	#define _xstrpbrkt		_xstrpbrkw
	#define _xstricmpt		_xstricmpw
	#define _xstrcpyt		_xstrcpyw
	#define _xstrncpyt		_xstrncpyw
	#define _xstrrchrt		_xstrrchrw
	#define _xstrcatt		_xstrcatw
	#define _xstruprt		_xstruprw
	#define _xstrlwrt		_xstrlwrw
	#define _xstrrevt		_xstrrevw
	#define _xstrtokt		_xstrtokw
#else // X_UNICODE
	#define _xstrlent		_xstrlena
	#define _xstrstrt		_xstrstra
	#define _xstrchrt		_xstrchra
	#define _xstrrchrt		_xstrrchra
	#define _xstrcmpt		_xstrcmpa
	#define _xstrncmpt		_xstrncmpa
	#define _xstrspnt		_xstrspna
	#define _xstrcspnt		_xstrcspna
	#define _xstrpbrkt		_xstrpbrka
	#define _xstrcollt		_xstrcolla
	#define _xstricollt		_xstricolla
	#define _xstricmpt		_xstricmpa
	#define _xstrcpyt		_xstrcpya
	#define _xstrncpyt		_xstrncpya
	#define _xstrrchrt		_xstrrchra
	#define _xstrcatt		_xstrcata
	#define _xstruprt		_xstrupra
	#define _xstrlwrt		_xstrlwra
	#define _xstrrevt		_xstrreva
	#define _xstrtokt		_xstrtoka
#endif

#ifdef X_WINDOWS
	#define _xstrleni		_xstrlenw
	#define _xstrstri		_xstrstrw
	#define _xstrchri		_xstrchrw
	#define _xstrrchri		_xstrrchrw
	#define _xstrcmpi		_xstrcmpw
	#define _xstrncmpi		_xstrncmpw
	#define _xstrcolli		_xstrcollw
	#define _xstricolli		_xstricollw
	#define _xstrspni		_xstrspnw
	#define _xstrcspni		_xstrcspnw
	#define _xstrpbrki		_xstrpbrkw
	#define _xstricmpi		_xstricmpw
	#define _xstrcpyi		_xstrcpyw
	#define _xstrncpyi		_xstrncpyw
	#define _xstrrchri		_xstrrchrw
	#define _xstrcati		_xstrcatw
	#define _xstrupri		_xstruprw
	#define _xstrlwri		_xstrlwrw
	#define _xstrrevi		_xstrrevw
#else // X_UNICODE
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// Comparison for xguid...
// ***************************************************************************************
inline static xbool __XIsEqualGUID(xcguid& iid1, xcguid& iid2)
{
	return
		(
		((xulong*) &iid1)[0] == ((xulong*) &iid2)[0] &&
		((xulong*) &iid1)[1] == ((xulong*) &iid2)[1] &&
		((xulong*) &iid1)[2] == ((xulong*) &iid2)[2] &&
		((xulong*) &iid1)[3] == ((xulong*) &iid2)[3]
		);
}

#define _XIsEqualGUID __XIsEqualGUID



// #######################################################################################



// ***************************************************************************************
// xresult success checking...
// ***************************************************************************************
inline static xbool __XSucceeded(xcresult xr)
{
	return xr >= 0;
}

inline static xbool __XSucceeded(xresult& xr1, xcresult xr2)
{
	return (_XResultSeverity(xr1) > _XResultSeverity(xr2) ||
		   (_XResultFacility(xr1) > _XResultFacility(xr2) && _XResultSeverity(xr1) >= _XResultSeverity(xr2)) ||
		   (_XResultCode(xr1) > _XResultCode(xr2) && (_XResultSeverity(xr1) >= _XResultSeverity(xr2) && _XResultFacility(xr1) >= _XResultFacility(xr2))))
			   ? __XSucceeded(xr1)
			   : __XSucceeded(xr1 = xr2);
}

#define _XSucceeded __XSucceeded



// #######################################################################################



// ***************************************************************************************
// xresult failure checking...
// ***************************************************************************************
inline static xbool __XFailed(xcresult xr)
{
	return !_XSucceeded(xr);
}

inline static xbool __XFailed(xresult& xr1, xcresult xr2)
{
	return !_XSucceeded(xr1, xr2);
}

#define _XFailed __XFailed



// #######################################################################################



// ***************************************************************************************
// Conversion from string to IID...
// ***************************************************************************************
inline static xresult __XStringToGUID(xcchart* pszGUID, xguid* pGUID)
{
	xulong ulData1 = 0;
	xushort usData2 = 0;
	xushort usData3 = 0;
	xuchart ucData4[8] = {0};
	
	if (_stscanf_s(pszGUID,
				   __XT("%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x"),
				   &ulData1,
				   &usData2,
				   &usData3,
				   &ucData4[0],
				   &ucData4[1],
				   &ucData4[2],
				   &ucData4[3],
				   &ucData4[4],
				   &ucData4[5],
				   &ucData4[6],
				   &ucData4[7]) != 11)
	{
		if (_stscanf_s(pszGUID,
					   __XT("{%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x}"),
					   &ulData1,
					   &usData2,
					   &usData3,
					   &ucData4[0],
					   &ucData4[1],
					   &ucData4[2],
					   &ucData4[3],
					   &ucData4[4],
					   &ucData4[5],
					   &ucData4[6],
					   &ucData4[7]))
		{
			return xeFail;
		}
	}

	pGUID->ulData1 = ulData1;
	pGUID->usData2 = (xword) (usData2 & 0xffff);
	pGUID->usData3 = (xword) (usData3 & 0xffff);

	for (xuint32 ui = 0; ui < 8; ui++)
	{
		pGUID->ucData4[ui] = (xbyte) (ucData4[ui] & 0xff);
	}

	return xsOK;
}

#define _XStringToGUID __XStringToGUID



// #######################################################################################
