// XBaseInterfaces.h:
// -------------------------------------------------------------------
// This file is a part of "XTL" available at:
// http://www.codecommunity.com
//
// XTL is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
// 
// XTL is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with XTL. If not, see <http://www.gnu.org/licenses/>.
//
// coded by: skulk@codecommunity.com
// modified: 2006-09-02 (skulk)
//////////////////////////////////////////////////////////////////////
#ifdef X_WINDOWS
	#pragma once
#endif // WIN32

#include "XBaseDefinitions.h"

// Interface forwards...
_xinterface IXInterface;
_xinterface IXEnum;
_xinterface IXEnumInterface;
_xinterface IXEnumString;
_xinterface IXFunctor;



// #######################################################################################



// ***************************************************************************************
// IXInterface: Interface base...
// ***************************************************************************************

// {0848783d-7f2a-43a5-8bc2-5620bdc2cc53}
static xcguid IID_IXInterface = 
{ 0x0848783d, 0x7f2a, 0x43a5, { 0x8b, 0xc2, 0x56, 0x20, 0xbd, 0xc2, 0xcc, 0x53 } };

_xinterface _xnovtable IXInterface
{
	// Query an interface...
	_XInterfaceMethod(xresult, QueryInterface)(xcguid& iid, xvoid** p) xpure;

	// Increment ref-count...
	_XInterfaceMethod(xulong, AddRef)() xpure;
	// Decrement ref-count...
	_XInterfaceMethod(xulong, Release)() xpure;
};



// #######################################################################################



// ***************************************************************************************
// IXEnum: Interface encapsulating an enumeration...
// ***************************************************************************************

// {CDA55888-DCEA-445f-94D3-52AF42EAD3FF}
static xcguid IID_IXEnum = 
{ 0xcda55888, 0xdcea, 0x445f, { 0x94, 0xd3, 0x52, 0xaf, 0x42, 0xea, 0xd3, 0xff } };

_XInterfaceDeclare(IXEnum)
{
	// Get count of items...
	_XInterfaceMethod(xuint32, GetCount)() xpure;

	// Delete an item...
	_XInterfaceMethod(xresult, Delete)(xcuint32 uiIndex) xpure;
	// Delete all items...
	_XInterfaceMethod(xresult, DeleteAll)() xpure;
};



// #######################################################################################



// ***************************************************************************************
// IXEnumInterface: Interface encapsulating an interface enumeration...
// ***************************************************************************************

// {F87E71F8-2C17-4364-A090-96010AF9C7F3}
static xcguid IID_IXEnumInterface = 
{ 0xf87e71f8, 0x2c17, 0x4364, { 0xa0, 0x90, 0x96, 0x1, 0xa, 0xf9, 0xc7, 0xf3 } };

__XInterfaceDeclare(IXEnumInterface, IXEnum)
{
	// Get an interface...
	_XInterfaceMethod(xresult, Get)(xcuint32 uiIndex, IXInterface** pInterface) xpure;
	// Get an interface...
	_XInterfaceMethod(xresult, Get)(xcuint32 uiIndex, xcguid& iid, IXInterface** pInterface) xpure;

	// Add an interface...
	_XInterfaceMethod(xresult, Add)(IXInterface* pInterface) xpure;
	// Add an enum interface...
	_XInterfaceMethod(xresult, Add)(IXEnumInterface* pEnum) xpure;

	// Insert an interface...
	_XInterfaceMethod(xresult, Insert)(xcuint32 uiIndex, IXInterface* pInterface) xpure;
	// Insert an enum interface...
	_XInterfaceMethod(xresult, Insert)(xcuint32 uiIndex, IXEnumInterface* pEnum) xpure;

	// Delete an interface...
	_XInterfaceMethod(xresult, Delete)(IXInterface* pInterface) xpure;
};



// #######################################################################################



// ***************************************************************************************
// IXEnumString: Interface encapsulating a string enumeration...
// ***************************************************************************************

// {d05ee226-8247-45e2-9f5c-4660e8d907bf}
static xcguid IID_IXEnumString = 
{ 0xd05ee226, 0x8247, 0x45e2, { 0x9f, 0x5c, 0x46, 0x60, 0xe8, 0xd9, 0x07, 0xbf } };

__XInterfaceDeclare(IXEnumString, IXEnum)
{
	// Get a string...
	_XInterfaceMethod(xresult, Get)(xcuint32 uiIndex, xchari** psz) xpure;
	
	// Add a string...
	_XInterfaceMethod(xresult, Add)(xcchari* psz, xcbool bSplit = false) xpure;
	// Add an enum string...
	_XInterfaceMethod(xresult, Add)(IXEnumString* pEnum) xpure;

	// Insert a string...
	_XInterfaceMethod(xresult, Insert)(xcuint32 uiIndex, xcchari* psz, xcbool bSplit = false) xpure;
	// Insert an enum string...
	_XInterfaceMethod(xresult, Insert)(xcuint32 uiIndex, IXEnumString* pEnum) xpure;

	// Delete a string...
	_XInterfaceMethod(xresult, Delete)(xcchari* psz) xpure;
	
	// Convert to a real multi string...
	_XInterfaceMethod(xresult, Convert)(xchari** psz, xuint32* puiSize = NULL) xpure;
};



// #######################################################################################



// ***************************************************************************************
// IXFunctor: Interface encapsulating a functor...
// ***************************************************************************************

// {FBE1CA83-C2C1-4b51-80AB-A63E1E433A43}
static xcguid IID_IXFunctor = 
{ 0xfbe1ca83, 0xc2c1, 0x4b51, { 0x80, 0xab, 0xa6, 0x3e, 0x1e, 0x43, 0x3a, 0x43 } };

_XInterfaceDeclare(IXFunctor)
{
	// Call the functor...
	_XInterfaceMethod(xresult, Call)() xpure;
};



// #######################################################################################
