// XLanguageDefinitions.h:
// -------------------------------------------------------------------
// This file is a part of "XTL" available at:
// http://www.codecommunity.com
//
// XTL is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
// 
// XTL is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with XTL. If not, see <http://www.gnu.org/licenses/>.
//
// coded by: skulk@codecommunity.com
// modified: 2008-01-21 (skulk)
//////////////////////////////////////////////////////////////////////
#include <wchar.h>
#include "XBaseDefinitions.h"



// #######################################################################################



// ***************************************************************************************
// XLanguageMap access...
// ***************************************************************************************
#define XLanguageMapGet(__Name) &XLanguageMapName(__Name)



// #######################################################################################



// ***************************************************************************************
// XLanguageMap names...
// ***************************************************************************************
#define XLanguageMapName(__Name) g_LanguageMap##__Name
#define _XLanguageMapName1(__Name) g_LanguageDataMap##__Name
#define _XLanguageMapName2(__Name) g_pszLanguageMap##__Name



// #######################################################################################



// ***************************************************************************************
// XLanguageData struct...
// ***************************************************************************************
#ifndef XLanguageDataDefined
#define XLanguageDataDefined

typedef struct _XLanguageData
{
	xccharw* pszID;
	xccharw* pszDefault;
} XLanguageData;

#endif // XLanguageDataDefined



// #######################################################################################



// ***************************************************************************************
// XLanguageMap struct...
// ***************************************************************************************
#ifndef XLanguageMapDefined
#define XLanguageMapDefined

typedef struct _XLanguageMap
{
	const XLanguageData* pData;
	xccharw*** psz;
} XLanguageMap;

#endif // XLanguageMapDefined



// #######################################################################################



// ***************************************************************************************
// XLanguageMap pass 4 definitions (for convenience)...
// ***************************************************************************************
#ifdef XLanguageMapPass_4

#undef XLanguageMapBegin
#define XLanguageMapBegin(__Name) \
	extern const XLanguageData _XLanguageMapName1(__Name)[]; \
	extern xccharw** _XLanguageMapName2(__Name)[]; \
	const XLanguageMap XLanguageMapName(__Name) = {_XLanguageMapName1(__Name), _XLanguageMapName2(__Name)};

#undef XLanguageMapEntry
#define XLanguageMapEntry(__ID, __String)

#undef XLanguageMapEnd
#define XLanguageMapEnd



// #######################################################################################



// ***************************************************************************************
// XLanguageMap pass 3 definitions (for dynamic map initialization)...
// ***************************************************************************************
#elif defined XLanguageMapPass_3

#undef XLanguageMapBegin
#define XLanguageMapBegin(__Name) \
	xccharw** _XLanguageMapName2(__Name)[] = \
	{

#undef XLanguageMapEntry
#define XLanguageMapEntry(__ID, __String) \
		&__ID,

#undef XLanguageMapEnd
#define XLanguageMapEnd \
		NULL \
	}; \



// #######################################################################################



// ***************************************************************************************
// XLanguageMap pass 2 definitions (for value initialization)...
// ***************************************************************************************
#elif defined XLanguageMapPass_2

#undef XLanguageMapBegin
#define XLanguageMapBegin(__Name) \
	extern const XLanguageData _XLanguageMapName1(__Name)[]; \
	const XLanguageData* pData = _XLanguageMapName1(__Name);

#undef XLanguageMapEntry
#define XLanguageMapEntry(__ID, __String) \
	xccharw* __ID = pData[__COUNTER__].pszDefault;

#undef XLanguageMapEnd
#define XLanguageMapEnd



// #######################################################################################



// ***************************************************************************************
// XLanguageMap pass 1 definitions (for value definition)...
// ***************************************************************************************
#elif defined XLanguageMapPass_1

#undef XLanguageMapBegin
#define XLanguageMapBegin(__Name) \
	const XLanguageData _XLanguageMapName1(__Name)[] = \
	{

#undef XLanguageMapEntry
#define XLanguageMapEntry(__ID, __String) \
		{L#__ID, L##__String},

#undef XLanguageMapEnd
#define XLanguageMapEnd \
		{NULL, NULL} \
	};



// #######################################################################################



// ***************************************************************************************
// XLanguageMap consumer definitions (these are the ones a consumer needs)...
// ***************************************************************************************
#else

#ifndef XLanguageDefined
#define XLanguageDefined

#undef XLanguageMapBegin
#define XLanguageMapBegin(__Name) \
	extern const XLanguageData _XLanguageMapName1(__Name)[]; \
	extern xccharw** _XLanguageMapName2(__Name)[]; \
	extern const XLanguageMap XLanguageMapName(__Name);

#undef XLanguageMapEntry
#define XLanguageMapEntry(__ID, __String) \
	extern xccharw* __ID;

#undef XLanguageMapEnd
#define XLanguageMapEnd

#endif // XLanguageDefined

#endif



// #######################################################################################
