// XBase.h:
// -------------------------------------------------------------------
// This file is a part of "XTL" available at:
// http://www.codecommunity.com
//
// XTL is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
// 
// XTL is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
// License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with XTL. If not, see <http://www.gnu.org/licenses/>.
//
// coded by: skulk@codecommunity.com
// modified: 2006-09-02 (skulk)
//////////////////////////////////////////////////////////////////////
#ifdef X_WINDOWS
	#pragma once
	#include <crtdbg.h>
	#include <malloc.h>
	#include <new.h>
	
	// Workaround for KB922317...
	#pragma push_macro("_dupenv_s")
	#pragma push_macro("_wdupenv_s")
	#undef _dupenv_s
	#undef _wdupenv_s
		#include <stdlib.h>
	#pragma pop_macro("_dupenv_s")
	#pragma pop_macro("_wdupenv_s")
	
	#include <stdio.h>
	#include <tchar.h>
#else // X_WINDOWS
	#error "Missing definition..."
#endif

#include "XBaseFunctions.h"
#include "XBaseInterfaces.h"



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XReportModes...
// ***************************************************************************************
enum _XReportModes
{
	XReportModeDebug	= _CRTDBG_MODE_DEBUG,
	XReportModeFile		= _CRTDBG_MODE_FILE,
	XReportModeWindow	= _CRTDBG_MODE_WNDW
};

// ***************************************************************************************
// XReportPlatform: Platform traits for "XReportT<>"
// ***************************************************************************************
class XReportPlatform
{
public:
// Operations...
	// Set the reporting mode...
	template <typename t_Traits> inline static xresult SetMode(xcuint32 uiMode)
	{
		return _CrtSetReportMode(t_Traits::GetType(), uiMode) != -1 ? xsOK : xeFail;
	}

	// Set the reporting file (handle)...
	template <typename t_Traits> inline static xvoid* SetFile(xvoid* pFile)
	{
		return _CrtSetReportFile(t_Traits::GetType(), (_HFILE) pFile);
	}

	// Set the reporting file name...
	template <typename t_Traits> inline static xvoid* SetFile(xcchart* pszFileName)
	{
		HANDLE hFile = pszFileName ? ::CreateFile(pszFileName, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL) : INVALID_HANDLE_VALUE;

		_XAssert(hFile != INVALID_HANDLE_VALUE);

		return hFile != INVALID_HANDLE_VALUE ? (xvoid*) _CrtSetReportFile(t_Traits::GetType(), hFile) : _CRTDBG_HFILE_ERROR;
	}

	// Report text...
	template <typename t_Traits> inline static xint32 Report(xcchara* pszFileName, xcint32 iLine, xcchara* pszReport)
	{
		return _CrtDbgReport(t_Traits::GetType(), pszFileName, iLine, XModuleInformation::GetFileNameA(), pszReport);
	}

	// Break into code...
	#define XReportPlatformBreak _CrtDbgBreak

	// Platform traits...
	struct _Traits
	{
		// Warning traits...
		struct _Warning
		{
			inline static xuint32 GetType()
			{
				return _CRT_WARN;
			}
		};

		// Error traits...
		struct _Error
		{
			inline static xuint32 GetType()
			{
				return _CRT_ERROR;
			}
		};

		// Assert traits...
		struct _Assert
		{
			inline static xuint32 GetType()
			{
				return _CRT_ASSERT;
			}
		};
	};
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XReportT: Generic debug reporting...
// ***************************************************************************************
#ifdef _DEBUG

template
<
	typename t_Platform
>
class XReportT
{
public:
// Operations...
	template <typename t_Traits> inline static xresult SetMode(xcuint32 uiMode)
	{
		return t_Platform::SetMode<t_Traits>(uiMode);
	}

	template <typename t_Traits> inline static xvoid* SetFile(xvoid* pFile)
	{
		return t_Platform::SetFile<t_Traits>(pFile);
	}

	template <typename t_Traits> inline static xvoid* SetFile(xcchart* pszFileName)
	{
		return t_Platform::SetFile(pszFileName);
	}
};

#endif // _DEBUG



// #######################################################################################



#ifdef X_WINDOWS

// VC 7.0 only...
#if _MSC_VER >= 1300
	#ifndef _delayimp_h
		extern "C" IMAGE_DOS_HEADER __ImageBase;
	#endif // !_delayimp_h
#endif // _MSC_VER >= 1300

// ***************************************************************************************
// XModuleInformation: Retrieve module information...
// ***************************************************************************************
class XModuleInformation
{
public:
// Operations...
	// Get the module handle...
	inline static HMODULE GetModule()
	{
#if _MSC_VER >= 1300
		return reinterpret_cast<HMODULE>(&__ImageBase);
#else // _MSC_VER >= 1300
		static xcint32 iDummy = 0;
		MEMORY_BASIC_INFORMATION mbi;
		::VirtualQuery(&iDummy, &mbi, sizeof(mbi));

		return reinterpret_cast<HMODULE>(mbi.AllocationBase);
#endif
	}

	// Get the module file name...
	inline static xcchart* GetFileName(HMODULE hModule = NULL)
	{
#ifdef X_UNICODE
		return XModuleInformation::GetFileNameW(hModule);
#else // X_UNICODE
		return XModuleInformation::GetFileNameA(hModule);
#endif
	}

	// Get the module path...
	inline static xcchart* GetPath(HMODULE hModule = NULL)
	{
		if (s_szPath[0] == __XT('\0'))
		{
			xcchart* pszFileName = GetFileName(hModule);
			if (pszFileName)
			{
				_xstrncpyt(s_szPath, _XCountCharT(s_szPath), pszFileName, _xstrlent(pszFileName));

				xchart* psz = _xstrrchrt(s_szPath, _XPathSeparator);

				if (psz && psz++)
				{
					*psz = __XT('\0');
				}
			}
		}

		return s_szPath;
	}

	// Get the module file name...
	inline static xcchara* GetFileNameA(HMODULE hModule = NULL)
	{
		static xchara s_szFileName[_MAX_PATH] = {'\0'};

		::GetModuleFileNameA(hModule ? hModule : GetModule(), s_szFileName, sizeof(s_szFileName) / sizeof(xchara));

		return s_szFileName;
	}

	// Get the module file name...
	inline static xccharw* GetFileNameW(HMODULE hModule = NULL)
	{
		static xcharw s_wszFileName[_MAX_PATH] = {'\0'};

		::GetModuleFileNameW(hModule ? hModule : GetModule(), s_wszFileName, sizeof(s_wszFileName) / sizeof(xcharw));

		return s_wszFileName;
	}

private:
// Internal data...
	static xchart s_szPath[_XPathMax];
};

_xselectany xchart XModuleInformation::s_szPath[_XPathMax] = {__XT('\0')};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



#ifdef _DEBUG

// Set reporting mode...
#define _XWarningSetMode(__Mode) XReportT<XReportPlatform>::SetMode<XReportPlatform::_Traits::_Warning>(__Mode)

// Set reporting file (only valid if reporting mode is "XReportModeFile"...
#define _XWarningSetFile(__File) XReportT<XReportPlatform>::SetFile<XReportPlatform::_Traits::_Warning>(__File)

// ***************************************************************************************
// _XWarning: Reporting warnings...
// ***************************************************************************************
#define _XWarning(__Expression) \
	do \
	{ \
		if (!(__Expression) && XReportPlatform::Report<XReportPlatform::_Traits::_Warning>(__FILE__, __LINE__, #__Expression) == 1) \
		{ \
			XReportPlatformBreak(); \
		} \
	} while (false);
#else // DEBUG
	#define _XWarningSetMode	_xnooperation
	#define _XWarningSetFile	_xnooperation
	#define _XWarning			_xnooperation
#endif



// #######################################################################################



#ifdef _DEBUG

// Set reporting mode...
#define _XErrorSetMode(__Mode) XReportT<XReportPlatform>::SetMode<XReportPlatform::_Traits::_Error>(__Mode)

// Set reporting file (only valid if reporting mode is "XReportModeFile"...
#define _XErrorSetFile(__File) XReportT<XReportPlatform>::SetFile<XReportPlatform::_Traits::_Error>(__File)

// ***************************************************************************************
// _XError: Reporting errors...
// ***************************************************************************************
#define _XError(__Expression) \
	do \
	{ \
		if (!(__Expression) && XReportPlatform::Report<XReportPlatform::_Traits::_Error>(__FILE__, __LINE__, #__Expression) == 1) \
		{ \
			XReportPlatformBreak(); \
		} \
	} while (false);
#else // DEBUG
	#define _XErrorSetMode	_xnooperation
	#define _XErrorSetFile	_xnooperation
	#define _XError			_xnooperation
#endif



// #######################################################################################



#ifdef _DEBUG

// Set reporting mode...
#define _XAssertSetMode(__Mode) XReportT<XReportPlatform>::SetMode<XReportPlatform::_Traits::_Assert>(__Mode)

// Set reporting file (only valid if reporting mode is "XReportModeFile"...
#define _XAssertSetFile(__File) XReportT<XReportPlatform>::SetFile<XReportPlatform::_Traits::_Assert>(__File)

// ***************************************************************************************
// _XAssert: Reporting assertions...
// ***************************************************************************************
#define _XAssert(__Expression) \
	do \
	{ \
		if (!(__Expression) && XReportPlatform::Report<XReportPlatform::_Traits::_Assert>(__FILE__, __LINE__, #__Expression) == 1) \
		{ \
			XReportPlatformBreak(); \
		} \
	} while (false);
#else // DEBUG
	#define _XAssertSetMode	_xnooperation
	#define _XAssertSetFile	_xnooperation
	#define _XAssert		_xnooperation
#endif



// #######################################################################################



// ***************************************************************************************
// XRefCountTypeT: Generic reference counting...	
// ***************************************************************************************
template
<
	typename t_Traits
>
class XRefCountTypeT
{
public:
// Construction/Destruction...
	inline XRefCountTypeT()
	{
		m_Type = t_Traits::Invalid();
	}

	inline XRefCountTypeT(typename t_Traits::Type type)
	{
		xcbool bValid = type != t_Traits::Invalid();

		_XAssert(bValid);

		if (bValid)
		{
			m_Type = type;

			AddRef();
		}
		else
		{
			m_Type = t_Traits::Invalid();
		}
	}

	inline XRefCountTypeT(XRefCountTypeT<t_Traits>& type)
	{
		xcbool bValid = type.m_Type != t_Traits::Invalid();

		_XAssert(bValid);

		if (bValid)
		{
			m_Type = type.Detach();

			AddRef();
		}
		else
		{
			m_Type = t_Traits::Invalid();
		}
	}

	inline ~XRefCountTypeT()
	{
		Release();
	}

public:
// Operations...
	inline typename t_Traits::Type GetSafeType()
	{
		_XAssert(m_Type != t_Traits::Invalid());

		return m_Type;
	}

	inline xvoid Attach(typename t_Traits::Type type)
	{
		xcbool bValid = type != t_Traits::Invalid();

		_XAssert(bValid);

		if (bValid)
		{
			Release();

			m_Type = type;
		}
	}

	inline typename t_Traits::Type Detach()
	{
		t_Traits::Type type = m_Type;

		m_Type = t_Traits::Invalid();
		
		return type;
	}

	inline xulong AddRef()
	{
		xcbool bValid = m_Type != t_Traits::Invalid();

		_XAssert(bValid);

		return bValid ? t_Traits::AddRef(m_Type) : ~0u;
	}

	inline xulong Release()
	{
		typename t_Traits::Type type = t_Traits::Invalid();

		if (m_Type != type)
		{
			xulong ul = t_Traits::Release(m_Type);

			m_Type = type;

			return ul;
		}
		else
		{
			return ~0u;
		}
	}

	inline xbool IsValid() const
	{
		return m_Type != t_Traits::Invalid();
	}

public:
// Operators...
	inline xbool operator!() const
	{
		return !IsValid();
	}

	inline XRefCountTypeT<t_Traits>& operator=(const XRefCountTypeT<t_Traits>& type)
	{
		xcbool bValid = type.m_Type != t_Traits::Invalid();

		_XAssert(bValid);

		if (bValid && this != &type)
		{
			Release();

			m_Type = type.m_Type;

			AddRef();
		}

		return *this;
	}

	inline typename t_Traits::Type operator=(typename t_Traits::Type type)
	{
		xcbool bValid = type != t_Traits::Invalid();

		_XAssert(bValid);

		if (bValid)
		{
			Release();

			m_Type = type;

			AddRef();
		}

		return m_Type;
	}

	inline xbool operator==(typename t_Traits::Type type) const
	{
		return m_Type == type;
	}

	inline operator typename t_Traits::Type()
	{
		return GetSafeType();
	}

	inline const typename t_Traits::Type operator->() const
	{
		return GetSafeType();
	}

	inline typename t_Traits::Type operator->()
	{
		return GetSafeType();
	}

	inline typename t_Traits::Type* operator&()
	{
		_XAssert(m_Type == t_Traits::Invalid());

		return &m_Type;
	}

private:
// Internal data...
	typename t_Traits::Type m_Type;
};



// #######################################################################################



// ***************************************************************************************
// XInterfaceTraitsT: 
// ***************************************************************************************
template
<
	typename t_Type
>
class XInterfaceTraitsT
{
public:
// Operations...
	// Reference counting...
	struct _RefCount
	{
		typedef t_Type* Type;

		inline static Type Invalid()
		{
			return NULL;
		}

		inline static xulong AddRef(Type type)
		{
			return type->AddRef();
		}

		inline static xulong Release(Type type)
		{
			return type->Release();
		}
	};
};



// #######################################################################################



// ***************************************************************************************
// XIPtrT: "XInterface" smart pointer...
// ***************************************************************************************
#define XIPtrTBase XRefCountTypeT<typename XInterfaceTraitsT<typename t_Type>::_RefCount>

template
<
	typename t_Type
>
class XIPtrT : public XIPtrTBase
{
// Base class...
typedef XIPtrTBase _Base;

public:
// Construction...
	inline XIPtrT()
	{
	}

	inline XIPtrT(typename t_Type* pType) : _Base(pType)
	{
	}

public:
// Operators...
	inline XIPtrT<t_Type>& operator=(const XIPtrT<t_Type>& ptr)
	{
		_Base::operator=(ptr);

		return *this;
	}
};



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XHandlePlatform: Platform traits for handle types...
// ***************************************************************************************
class XHandlePlatform
{
public:
// Operations...
	// Platform traits...
	struct _Traits
	{
		// Reference counting...
		struct _RefCount
		{
			typedef HANDLE Type;

			inline static Type Invalid()
			{
				return NULL;
			}

			inline static xulong AddRef(Type type)
			{
				// Nothing...
				return ~0u;
			}

			inline static xulong Release(Type type)
			{
				return ::CloseHandle(type) ? 0 : ~0u;
			}
		};

	};
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif

// Platform type...
typedef XRefCountTypeT<XHandlePlatform::_Traits::_RefCount>	XHandle;



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XHandleFilePlatform: Platform traits for file handle types...
// ***************************************************************************************
class XHandleFilePlatform
{
public:
// Operations...
	// Platform traits...
	struct _Traits
	{
		// Reference counting...
		struct _RefCount
		{
			typedef HANDLE Type;

			inline static Type Invalid()
			{
				return INVALID_HANDLE_VALUE;
			}

			inline static xulong AddRef(Type type)
			{
				return ~0u;
			}

			inline static xulong Release(Type type)
			{
				return ::CloseHandle(type) ? 0 : ~0u;
			}
		};

	};
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif

// Platform type...
typedef XRefCountTypeT<XHandleFilePlatform::_Traits::_RefCount>	XHandleFile;



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XHandleModulePlatform: Platform traits for module handle types...
// ***************************************************************************************
class XHandleModulePlatform
{
public:
// Operations...
	// Platform traits...
	struct _Traits
	{
		// Reference counting...
		struct _RefCount
		{
			typedef HMODULE Type;

			inline static Type Invalid()
			{
				return NULL;
			}

			inline static xulong AddRef(Type type)
			{
				return ~0u;
			}

			inline static xulong Release(Type type)
			{
				return ::FreeLibrary(type) ? 0 : ~0u;
			}
		};
	};
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif

// Platform type...
typedef XRefCountTypeT<XHandleModulePlatform::_Traits::_RefCount> XHandleModule;



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XEventPlatform: Platform traits for "XEventT<>"
// ***************************************************************************************
class XEventPlatform : public XHandle
{
public:
// Construction...
	inline XEventPlatform()
	{
		m_bExist = false;
	}

public:
// Operations...
	// Create...
	inline xresult Create(xcchart* pszName, xcbool bManualReset, xcbool bInitialState, xvoid* pSecurity)
	{
		Attach(::CreateEvent((LPSECURITY_ATTRIBUTES) pSecurity, bManualReset, bInitialState, pszName));
		
		xcbool bValid = IsValid();

		_XAssert(bValid);

		if (bValid)
		{
			m_bExist = ::GetLastError() == ERROR_ALREADY_EXISTS;
		}

		return bValid ? xsOK : _XResultLastError;
	}

	// Check if exists...
	inline xbool Exist() const
	{
		return m_bExist;
	}

	// Set...
	inline xresult Set()
	{
		_XAssert(IsValid());
		if (!IsValid())
			return xeUnexpected;

		return ::SetEvent(GetSafeType()) ? xsOK : _XResultLastError;
	}

	// Reset...
	inline xresult Reset()
	{
		_XAssert(IsValid());
		if (!IsValid())
			return xeUnexpected;

		return ::ResetEvent(GetSafeType()) ? xsOK : _XResultLastError;
	}

	// Wait until the event is signaled...
	inline xresult WaitForSignal(xcuint32 uiMilliseconds)
	{
		xcbool bValid = IsValid();

		_XAssert(bValid);
		if (!bValid)
			return xeUnexpected;

		return ::WaitForSingleObject(GetSafeType(), uiMilliseconds) == WAIT_OBJECT_0 ? xsOK : _XResultLastError;
	}

private:
// Internal data...
	xbool m_bExist;
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XEventT: Generic event...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XEventT : public t_Platform
{
public:
// Construction/Destruction...
	inline XEventT()
	{
	}

	inline ~XEventT()
	{
	}

public:
// Operations...
	// Create...
	inline xresult Create(xcchart* pszName = NULL, xcbool bManualReset = false, xcbool bInitialState = false, xvoid* pSecurity = NULL)
	{
		return t_Platform::Create(pszName, bManualReset, bInitialState, pSecurity);
	}

	// Check if exists...
	inline xbool Exist() const
	{
		return t_Platform::Exist();
	}

	// Set...
	inline xresult Set()
	{
		return t_Platform::Set();
	}

	// Reset...
	inline xresult Reset()
	{
		return t_Platform::Reset();
	}

	inline xresult WaitForSignal(xcuint32 uiMilliseconds = _XInfinite)
	{
		return t_Platform::WaitForSignal(uiMilliseconds);
	}
};

// Platform type...
typedef XEventT<XEventPlatform> XEvent;



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XMutexPlatform: Platform traits for "XMutexT<>"
// ***************************************************************************************
class XMutexPlatform : public XHandle
{
public:
// Construction...
	inline XMutexPlatform()
	{
		m_bExist = false;
	}

public:
// Operations...
	// Create...
	inline xresult Create(xcbool bInitialOwner, xcchart* pszName, xvoid* pSecurity)
	{
		Attach(::CreateMutex((LPSECURITY_ATTRIBUTES) pSecurity, bInitialOwner, pszName));

		xcbool bValid = IsValid();

		_XAssert(bValid);

		if (bValid)
		{
			m_bExist = ::GetLastError() == ERROR_ALREADY_EXISTS;
		}

		return bValid ? xsOK : _XResultLastError;
	}

	// Check if exists...
	inline xbool Exist() const
	{
		return m_bExist;
	}

	// Release...
	inline xresult Release()
	{
		_XAssert(IsValid());
		if (!IsValid())
			return xeUnexpected;

		return ::ReleaseMutex(GetSafeType()) ? xsOK : _XResultLastError;
	}

private:
// Internal data...
	xbool m_bExist;
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XMutexT: Generic mutex...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XMutexT : public t_Platform
{
public:
// Construction/Destruction...
	inline XMutexT()
	{
	}

	inline ~XMutexT()
	{
	}

public:
// Operations...
	// Create...
	inline xresult Create(xcbool bInitialOwner = false, xcchart* pszName = NULL, xvoid* pSecurity = NULL)
	{
		return t_Platform::Create(bInitialOwner, pszName, pSecurity);
	}

	// Check if exists...
	inline xbool Exist() const
	{
		return t_Platform::Exist();
	}

	// Release...
	inline xresult Release()
	{
		return t_Platform::Release();
	}
};

// Platform type...
typedef XMutexT<XMutexPlatform> XMutex;



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XCriticalSectionPlatform: Platform traits for "XCriticalSectionT<>"
// ***************************************************************************************
class XCriticalSectionPlatform
{
public:
// Operations...
	// Create...
	inline xvoid Create()
	{
		::InitializeCriticalSection(&m_cs);
	}

	// Destroy...
	inline xvoid Destroy()
	{
		::DeleteCriticalSection(&m_cs);
	}

	// Lock...
	inline xvoid Lock()
	{
		::EnterCriticalSection(&m_cs);
	}

	// Try lock...
	inline xbool TryLock()
	{
#if(_WIN32_WINNT >= 0x0400)
		return (xbool) !!::TryEnterCriticalSection(&m_cs);
#else // (_WIN32_WINNT < 0x0400)
		return false;
#endif
	}

	// Unlock...
	inline xvoid Unlock()
	{
		::LeaveCriticalSection(&m_cs);
	}

private:
// Internal data...
	CRITICAL_SECTION m_cs;
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XCriticalSectionT: Generic critical section...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XCriticalSectionT : public t_Platform
{
public:
// Construction/Destruction...
	inline XCriticalSectionT()
	{
		t_Platform::Create();
	}

	inline ~XCriticalSectionT()
	{
		t_Platform::Destroy();
	}

public:
// Operations...
	// Lock this object...
	inline xvoid Lock()
	{
		t_Platform::Lock();
	}

	// Try to lock this object...
	inline xbool TryLock()
	{
		return t_Platform::TryLock();
	}

	// Unlock this object...
	inline xvoid Unlock()
	{
		t_Platform::Unlock();
	}
};

// Platform type...
typedef XCriticalSectionT<XCriticalSectionPlatform> XCriticalSection;



// #######################################################################################



// ***************************************************************************************
// XCriticalSectionTraits: Traits for "XCriticalSectionT<>"
// ***************************************************************************************
class XCriticalSectionTraits
{
public:
// Operations...
	// Reference counting...
	struct _RefCount
	{
		typedef XCriticalSection* Type;

		inline static Type Invalid()
		{
			return NULL;
		}

		inline static xulong AddRef(Type type)
		{
			type->Lock();

			return 1;
		}

		inline static xulong Release(Type type)
		{
			type->Unlock();

			return 0;
		}
	};
};

// Trait type...
typedef XRefCountTypeT<XCriticalSectionTraits::_RefCount> XCriticalSectionLock;



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XCriticalSectionCrossProcessPlatform: Platform traits for
//  "XCriticalSectionCrossProcessT<>"
// ***************************************************************************************
class XCriticalSectionCrossProcessPlatform
{
public:
// Operations...
	// Create...
	inline xvoid Create(xcchart* pszID)
	{
		_XAssert(pszID);

		m_dwThreadID = 0;
		m_lLockCount = 0;

		m_Event.Create(pszID);
	}

	// Destroy...
	inline xvoid Destroy()
	{
		_XAssert(m_lLockCount == 0);
	}

	// Lock...
	inline xbool Lock(xcdword dwWait = ~0u)
	{
		if (::InterlockedCompareExchange(&m_lLockCount, 1, 0) == 0)
		{
			m_dwThreadID = ::GetCurrentThreadId();

			return true;
		}

		if (m_dwThreadID == ::GetCurrentThreadId())
		{
			m_lLockCount++;

			return true;
		}

		xcdword dwWaitStart = ::GetTickCount();

		if (::WaitForSingleObject(m_Event, dwWait) == WAIT_OBJECT_0)
		{
			xcdword dwWaitCurrent = ::GetTickCount() - dwWaitStart;

			return dwWaitCurrent < dwWait ? Lock(dwWait - dwWaitCurrent) : false;
		}

		return false;
	}

	// Try lock...
	inline xbool TryLock()
	{
		return Lock(0);
	}

	// Unlock...
	inline xvoid Unlock()
	{
		if (m_dwThreadID != ::GetCurrentThreadId())
		{
			// You called Unlock() on an object more often than Lock()...
			_XAssert(false);

			return;
		}

		if (!--m_lLockCount)
		{
			m_dwThreadID = 0;

			::PulseEvent(m_Event);
		}
	}

private:
// Internal data...
	xdword m_dwThreadID;
	xlong m_lLockCount;
	XEvent m_Event;
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XCriticalSectionCrossProcessT: Generic cross process critical section...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XCriticalSectionCrossProcessT : public t_Platform
{
public:
// Construction/Destruction...
	inline XCriticalSectionCrossProcessT(xcchart* pszID)
	{
		_XAssert(pszID);

		t_Platform::Create(pszID);
	}

	inline ~XCriticalSectionCrossProcessT()
	{
		t_Platform::Destroy();
	}

public:
// Operations...
	// Lock this object...
	inline xvoid Lock()
	{
		t_Platform::Lock();
	}

	// Try to lock this object...
	inline xbool TryLock()
	{
		return t_Platform::TryLock();
	}

	// Unlock this object...
	inline xvoid Unlock()
	{
		t_Platform::Unlock();
	}
};

// Platform type...
typedef XCriticalSectionCrossProcessT<XCriticalSectionCrossProcessPlatform> XCriticalSectionCrossProcess;



// #######################################################################################



// ***************************************************************************************
// XCriticalSectionCrossProcessTraits: Traits for "XCriticalSectionCrossProcessT<>"
// ***************************************************************************************
class XCriticalSectionCrossProcessTraits
{
public:
// Operations...
	// Reference counting...
	struct _RefCount
	{
		typedef XCriticalSectionCrossProcess* Type;

		inline static Type Invalid()
		{
			return NULL;
		}

		inline static xulong AddRef(Type type)
		{
			type->Lock();

			return 1;
		}

		inline static xulong Release(Type type)
		{
			type->Unlock();

			return 0;
		}
	};
};

// Trait type...
typedef XRefCountTypeT<XCriticalSectionCrossProcessTraits::_RefCount> XCriticalSectionCrossProcessLock;



// #######################################################################################



// ***************************************************************************************
// XMemoryTraitsT: Traits for memory...
// ***************************************************************************************
template
<
	typename t_Type
>
class XMemoryTraitsT
{
public:
// Operations...
	// Reference counting...
	struct _RefCount
	{
		typedef t_Type* Type;

		inline static Type Invalid()
		{
			return NULL;
		}

		inline static xulong AddRef(Type type)
		{
			// Nothing...
			return ~0u;
		}

		inline static xulong Release(Type type)
		{
			_xfree(type);

			return 0;
		}
	};
};



// #######################################################################################



// ***************************************************************************************
// XPtrT: Memory smart pointer...
// ***************************************************************************************
template
<
	typename t_Type
>
class XPtrT : public XRefCountTypeT<typename XMemoryTraitsT<typename t_Type>::_RefCount>
{
// Base class...
typedef XRefCountTypeT<typename XMemoryTraitsT<typename t_Type>::_RefCount> _Base;

public:
	// Construction...
	inline XPtrT()
	{
	}

	inline XPtrT(typename t_Type* pType) : _Base(pType)
	{
	}
	
public:
	// Operators...
	inline XPtrT<t_Type>& operator=(const XPtrT<t_Type>& ptr)
	{
		_Base::operator=(ptr);

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// XObjectTraitsT: Traits for objects...
// ***************************************************************************************
template
<
	typename t_Type
>
class XObjectTraitsT
{
public:
	// Operations...
	// Reference counting...
	struct _RefCount
	{
		typedef t_Type* Type;

		inline static Type Invalid()
		{
			return NULL;
		}

		inline static xulong AddRef(Type type)
		{
			// Nothing...
			return ~0u;
		}

		inline static xulong Release(Type type)
		{
			_xdelete type;

			return 0;
		}
	};
};



// #######################################################################################



// ***************************************************************************************
// XOPtrT: Object smart pointer...
// ***************************************************************************************
template
<
	typename t_Type
>
class XOPtrT : public XRefCountTypeT<typename XObjectTraitsT<typename t_Type>::_RefCount>
{
// Base class...
typedef XRefCountTypeT<typename XObjectTraitsT<typename t_Type>::_RefCount> _Base;

public:
	// Construction...
	inline XOPtrT()
	{
	}

	inline XOPtrT(typename t_Type* pType) : _Base(pType)
	{
	}
	
public:
	// Operators...
	inline XOPtrT<t_Type>& operator=(const XOPtrT<t_Type>& ptr)
	{
		_Base::operator=(ptr);

		return *this;
	}
};



// #######################################################################################



// ***************************************************************************************
// Reference counting for interfaces...
// ***************************************************************************************
template
<
	typename t_Base,
	typename t_Interface = IXInterface,
	typename t_Cast = t_Base
>
class _xnovtable XRefCountT : public t_Base
{
protected:
// Construction...
	inline XRefCountT()
	{
		m_lRefCount = 0;
	}

	inline ~XRefCountT()
	{
	}

protected:
// Operations...
	inline typename t_Interface* RefUnknown()
	{
		return static_cast<t_Interface*>(static_cast<t_Cast*>(this));
	}

protected:
// Internal data...
	xlong m_lRefCount;
};



// #######################################################################################



// ***************************************************************************************
// Interface map and entry definition macros for "IXInterface"...
// ***************************************************************************************
#define XInterfaceMapBegin \
public: \
	inline _XInterfaceMethod(xulong, AddRef)() \
	{ \
		return ::InterlockedIncrement(&m_lRefCount); \
	} \
	inline _XInterfaceMethod(xulong, Release)() \
	{ \
		return !!::InterlockedDecrement(&m_lRefCount) ? m_lRefCount : (_xdelete this, 0); \
	} \
	inline _XInterfaceMethod(xresult, QueryInterface)(xcguid& iid, xvoid** p) \
	{ \
		if (!p) \
		{ \
			return xeInvalidArgument; \
		} \
		if (_XIsEqualGUID(iid, IID_IXInterface)) \
		{ \
			*p = RefUnknown(); \
		} \

#define XInterfaceMapEntry(__Interface) \
		else if (_XIsEqualGUID(iid, IID_##__Interface)) \
		{ \
			*p = static_cast<__Interface*>(this); \
		} \

#define XInterfaceMapEntryIf(__Interface, __Clause) \
		else if (_XIsEqualGUID(iid, IID_##__Interface) && (__Clause)) \
		{ \
			*p = static_cast<__Interface*>(this); \
		} \

#define XInterfaceMapEntryIID(__Interface, __IID) \
		else if (_XIsEqualGUID(iid, __IID)) \
		{ \
			*p = static_cast<__Interface*>(this); \
		} \

#define XInterfaceMapEntryRaw(__Interface, __IID, __Cast) \
		else if (_XIsEqualGUID(iid, __IID)) \
		{ \
			*p = static_cast<__Interface*>(__Cast); \
		} \

#define XInterfaceMapEntryRawIf(__Interface, __IID, __Cast, __Clause) \
		else if (_XIsEqualGUID(iid, __IID) && (__Clause)) \
		{ \
			*p = static_cast<__Interface*>(__Cast); \
		} \

#define XInterfaceMapEntryVoid(__IID, __Cast, __RefCommand) \
		else if (_XIsEqualGUID(iid, __IID)) \
		{ \
			*p = __Cast; \
			__RefCommand; \
			return xsOK; \
		} \

#define XInterfaceMapEntryVoidIf(__IID, __Cast, __Clause, __RefCommand) \
		else if (_XIsEqualGUID(iid, __IID) && (__Clause)) \
		{ \
			*p = __Cast; \
			__RefCommand; \
			return xsOK; \
		} \

#define XInterfaceMapEntryFunction(__IID, __Function) \
		else if (_XIsEqualGUID(iid, __IID)) \
		{ \
			return __Function(p); \
		} \

#define XInterfaceMapChain(__ChainClass) \
		else if (xresult xr = __ChainClass::QueryInterface(iid, p) && _XSucceeded(xr)) \
		{ \
			return xr; \
		} \

#define XInterfaceMapEnd \
		else \
		{ \
			*p = NULL; \
			return xeNoInterface; \
		} \
	AddRef(); \
	return xsOK; \
}



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// Interface map and entry definition macros for "IUnknown"...
// ***************************************************************************************
#define XInterfaceMapCOMBegin \
public: \
	inline STDMETHOD_(ULONG, AddRef)() \
	{ \
		return ::InterlockedIncrement(&m_lRefCount); \
	} \
	inline STDMETHOD_(ULONG, Release)() \
	{ \
		return !!::InterlockedDecrement(&m_lRefCount) ? m_lRefCount : (_xdelete this, 0); \
	} \
	inline STDMETHOD(QueryInterface)(REFIID iid, void** p) \
	{ \
		if (!p) \
		{ \
			return xeInvalidArgument; \
		} \
		if (::IsEqualGUID(iid, IID_IUnknown)) \
		{ \
			*p = RefUnknown(); \
		} \

#define XInterfaceMapCOMEntry(__Interface) \
		else if (::IsEqualGUID(iid, IID_##__Interface)) \
		{ \
			*p = static_cast<__Interface*>(this); \
		} \

#define XInterfaceMapCOMEntryIf(__Interface, __Clause) \
		else if (::IsEqualGUID(iid, IID_##__Interface) && (__Clause)) \
		{ \
			*p = static_cast<__Interface*>(this); \
		} \

#define XInterfaceMapCOMEntryIID(__Interface, __IID) \
		else if (::IsEqualGUID(iid, __IID)) \
		{ \
			*p = static_cast<__Interface*>(this); \
		} \

#define XInterfaceMapCOMEntryRaw(__Interface, __IID, __Cast) \
		else if (::IsEqualGUID(iid, __IID)) \
		{ \
			*p = static_cast<__Interface*>(__Cast); \
		} \

#define XInterfaceMapCOMEntryRawIf(__Interface, __IID, __Cast, __Clause) \
		else if (::IsEqualGUID(iid, __IID) && (__Clause)) \
		{ \
			*p = static_cast<__Interface*>(__Cast); \
		} \

#define XInterfaceMapCOMEntryVoid(__IID, __Cast, __RefCommand) \
		else if (::IsEqualGUID(iid, __IID)) \
		{ \
			*p = __Cast; \
			__RefCommand; \
			return S_OK; \
		} \

#define XInterfaceMapCOMEntryVoidIf(__IID, __Cast, __Clause, __RefCommand) \
		else if (::IsEqualGUID(iid, __IID) && (__Clause)) \
		{ \
			*p = __Cast; \
			__RefCommand; \
			return S_OK; \
		} \

#define XInterfaceMapCOMEntryFunction(__IID, __Function) \
		else if (::IsEqualGUID(iid, __IID)) \
		{ \
			return __Function(p); \
		} \

#define XInterfaceMapCOMChain(__ChainClass) \
		else if (HRESULT hr = __ChainClass::QueryInterface(iid, p) && SUCCEEDED(hr)) \
		{ \
			return hr; \
		} \

#define XInterfaceMapCOMEnd \
		else \
		{ \
			*p = NULL; \
			return E_NOINTERFACE; \
		} \
	AddRef(); \
	return S_OK; \
}

#endif // X_WINDOWS



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// "XFile" access...
// ***************************************************************************************
enum _XFileAccess
{
	XFileAccessRead					= GENERIC_READ,
	XFileAccessWrite				= GENERIC_WRITE,
	XFileAccessExecute				= GENERIC_EXECUTE,
};

// ***************************************************************************************
// "XFile" share...
// ***************************************************************************************
enum _XFileShare
{
	XFileShareRead					= FILE_SHARE_READ,
	XFileShareWrite					= FILE_SHARE_WRITE,
	XFileShareDelete				= FILE_SHARE_DELETE,
};

// ***************************************************************************************
// "XFile" disposition...
// ***************************************************************************************
enum _XFileDispositions
{
	XFileDispositionCreate			= CREATE_NEW,
	XFileDispositionCreateAlways	= CREATE_ALWAYS,
	XFileDispositionOpen			= OPEN_EXISTING,
	XFileDispositionOpenAlways		= OPEN_ALWAYS,
	XFileDispositionTruncate		= TRUNCATE_EXISTING
};

// ***************************************************************************************
// "XFile" attributes...
// ***************************************************************************************
enum _XFileAttributes
{
	XFileAttributeArchive			= FILE_ATTRIBUTE_ARCHIVE,
	XFileAttributeCompressed		= FILE_ATTRIBUTE_COMPRESSED,
	XFileAttributeNotContentIndexed	= FILE_ATTRIBUTE_NOT_CONTENT_INDEXED,
	XFileAttributeDevice			= FILE_ATTRIBUTE_DEVICE,
	XFileAttributeDirectory			= FILE_ATTRIBUTE_DIRECTORY,
	XFileAttributeEncrypted			= FILE_ATTRIBUTE_ENCRYPTED,
	XFileAttributeHidden			= FILE_ATTRIBUTE_HIDDEN,
	XFileAttributeNormal			= FILE_ATTRIBUTE_NORMAL,
	XFileAttributeOffline			= FILE_ATTRIBUTE_OFFLINE,
	XFileAttributeReadOnly			= FILE_ATTRIBUTE_READONLY,
	XFileAttributeReparsePoint		= FILE_ATTRIBUTE_REPARSE_POINT,
	XFileAttributeSparseFile		= FILE_ATTRIBUTE_SPARSE_FILE,
	XFileAttributeSystem			= FILE_ATTRIBUTE_SYSTEM,
	XFileAttributeTemporary			= FILE_ATTRIBUTE_TEMPORARY
};

// ***************************************************************************************
// "XFile" flags...
// ***************************************************************************************
enum _XFileFlags
{
	XFileFlagBackupSematics			= FILE_FLAG_BACKUP_SEMANTICS,
	XFileFlagDeleteOnClose			= FILE_FLAG_DELETE_ON_CLOSE,
	XFileFlagFirstPipeInstance		= FILE_FLAG_FIRST_PIPE_INSTANCE,
	XFileFlagNoBuffering			= FILE_FLAG_NO_BUFFERING,
	XFileFlagOpenReparsePoint		= FILE_FLAG_OPEN_REPARSE_POINT,
	XFileFlagOpenNoRecall			= FILE_FLAG_OPEN_NO_RECALL,
	XFileFlagOverlapped				= FILE_FLAG_OVERLAPPED,
	XFileFlagPosixSematics			= FILE_FLAG_POSIX_SEMANTICS,
	XFileFlagRandomAccess			= FILE_FLAG_RANDOM_ACCESS,
	XFileFlagSequentialScan			= FILE_FLAG_SEQUENTIAL_SCAN,
	XFileFlagWriteThrough			= FILE_FLAG_WRITE_THROUGH
};

// ***************************************************************************************
// "XFile" positions...
// ***************************************************************************************
enum _XFilePositions
{
	XFilePositionBegin				= FILE_BEGIN,
	XFilePositionCurrent			= FILE_CURRENT,
	XFilePositionEnd				= FILE_END,
};

// ***************************************************************************************
// XFilePlatform: Platform traits for files...
// ***************************************************************************************
class XFilePlatform : public XHandleFile
{
public:
// Operations...
	inline xresult Create(xcchart* pszName, xcuint32 uiAccess, xcuint32 uiShare, xcuint32 uiDisposition, xcuint32 uiAttributes, xcuint32 uiFlags)
	{
		_XAssert(pszName);
		if (!pszName)
			return xeInvalidArgument;

		HANDLE h = ::CreateFile(pszName, uiAccess, uiShare, NULL, uiDisposition, uiAttributes | uiFlags, NULL);
		if (h == INVALID_HANDLE_VALUE)
			return _XResultLastError;

		Attach(h);

		return xsOK;
	}

	inline xuint64 GetSize()
	{
		LARGE_INTEGER li = {0};
		return ::GetFileSizeEx(GetSafeType(), &li) ? li.QuadPart : 0;
	}

	inline xresult SetPosition(xcint64 iDistance, xcuint32 uiMove, xint64* piPosition = NULL)
	{
		LARGE_INTEGER liDistance = {0, 0};
		
		liDistance.QuadPart = iDistance;

		LARGE_INTEGER liPosition = {0, 0};
		
		if (!::SetFilePointerEx(GetSafeType(), liDistance, piPosition ? &liPosition : NULL, uiMove))
			return _XResultLastError;

		if (piPosition)
		{
			*piPosition = liPosition.QuadPart;
		}

		return xsOK;
	}

	inline xresult Read(xbyte* pBuffer, xcdword dwSize)
	{
		_XAssert(pBuffer);
		if (!pBuffer)
			return xeInvalidArgument;

		xcbool bValid = IsValid();

		_XAssert(bValid);
		if (!bValid)
			return xeUnexpected;
		
		xdword dw = 0;
		if (!::ReadFile(GetSafeType(), pBuffer, dwSize, &dw, NULL))
			return _XResultLastError;

		_XAssert(dwSize == dw);
		if (dwSize != dw)
			return xeUnexpected;

		return xsOK;
	}

	inline xresult Read(xbyte** pBuffer, xcdword dwSize)
	{
		_XAssert(pBuffer);
		if (!pBuffer)
			return xeInvalidArgument;

		XPtrT<xbyte> p((xbyte*) _xalloc(dwSize));
		if (!p)
			return xeOutOfMemory;

		xresult xr = Read(p, dwSize);
		if (_XFailed(xr))
			return xr;

		*pBuffer = p.Detach();

		return xr;
	}

	inline xresult Write(xcbyte* pBuffer, xcdword dwSize)
	{
		_XAssert(pBuffer);
		if (!pBuffer)
			return xeInvalidArgument;

		xcbool bValid = IsValid();

		_XAssert(bValid);
		if (!bValid)
			return xeUnexpected;

		xdword dw = 0;
		if (!::WriteFile(GetSafeType(), pBuffer, dwSize, &dw, NULL))
			return _XResultLastError;

		_XAssert(dwSize == dw);
		if (dwSize != dw)
			return xeUnexpected;
		
		return xsOK;
	}

	inline xresult Close()
	{
		Release();
	}
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XFileT: Implementation of a generic file...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XFileT : public t_Platform
{
public:
// Construction/Destruction...
	inline XFileT()
	{
	}

	inline ~XFileT()
	{
	}

public:
// Operations...
	// Create...
	inline xresult Create(xcchart* pszName, xcuint32 uiAccess, xcuint32 uiShare, xcuint32 uiDisposition, xcuint32 uiAttributes, xcuint32 uiFlags)
	{
		return t_Platform::Create(pszName, uiAccess, uiShare, uiDisposition, uiAttributes, uiFlags);
	}

	// Get size...
	inline xuint64 GetSize()
	{
		return t_Platform::GetSize();
	}

	// Set position...
	inline xresult SetPosition(xcint64 iDistance, xcuint32 uiMove, xint64* piPosition = NULL)
	{
		return t_Platform::SetPosition(iDistance, uiMove, piPosition);
	}

	// Read...
	inline xresult Read(xbyte* pBuffer, xcdword dwSize)
	{
		return t_Platform::Read(pBuffer, dwSize);
	}

	// Read...
	inline xresult Read(xbyte** pBuffer, xcdword dwSize)
	{
		return t_Platform::Read(pBuffer, dwSize);
	}

	// Write...
	inline xresult Write(xcbyte* pBuffer, xcdword dwSize)
	{
		return t_Platform::Write(pBuffer, dwSize);
	}

	// Close...
	inline xresult Close()
	{
		return t_Platform::Close();
	}

	// Overload from "XRefCountTypeT<>" - we need a real pointer here...
	inline XFileT<typename t_Platform>* operator&()
	{
		return this;
	}
};

// Platform type...
typedef XFileT<XFilePlatform> XFile;



// #######################################################################################



// ***************************************************************************************
// XFunctorT: Generic functor...
// ***************************************************************************************
template
<
	typename t_Type
>
class XFunctorT : public XRefCountT<IXFunctor>
{
// Interface map...
XInterfaceMapBegin
	XInterfaceMapEntry(IXFunctor)
XInterfaceMapEnd

public:
// Construction/Destruction...
	inline XFunctorT()
	{
		m_pType = NULL;
		m_pfn = NULL;
	}

	inline explicit XFunctorT(t_Type* pType, xresult (t_Type::*pfn)())
	{
		_XAssert(pType && pfn);
		if (!pType || !pfn)
			return;

		Set(pType, pfn);
	}

	inline ~XFunctorT()
	{
		Detach(m_pType);
	}

public:
// Operations...
	inline xresult Set(t_Type* pType, xresult (t_Type::*pfn)())
	{
		_XAssert(pType && pfn);
		if (!pType || !pfn)
			return xeInvalidArgument;

		if (m_pType)
		{
			Detach(m_pType);
		}

		Attach(pType);

		m_pfn = pfn;

		return xsOK;
	}

public:
// Operators...
	// Call the functor...
	inline _XInterfaceMethod(xresult, Call)()
	{
		_XAssert(m_pType && m_pfn);
		if (!m_pType || !m_pfn)
			return xePointer;

		return (m_pType->*m_pfn)();
	}

private:
// Internal functions...
	xvoid Attach(t_Type* pType)
	{
		m_pType = pType;
	}

	xvoid Attach(IXInterface* pInterface)
	{
		_XAssert(pInterface);
		if (pInterface)
		{
			(m_pType = pInterface)->AddRef();
		}
	}

	xvoid Detach(t_Type* pType)
	{
		m_pType = NULL;
	}

	xvoid Detach(IXInterface* pInterface)
	{
		_XAssert(pInterface);
		if (pInterface)
		{
			pInterface->Release();
		}
	}

private:
// Internal data...
	t_Type* m_pType;
	xresult (t_Type::*m_pfn)();
};



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XThreadPlatform: Platform traits for "XThreadBaseT<>"
// ***************************************************************************************
class XThreadPlatform : public XHandle
{
// Base class...
typedef XHandle _Base;

public:
// Operations...
	// Create the thread...
	inline xresult Create(_XThreadProc pfn, xvoid* pData, xcbool bRun = true, xvoid* pSecurityAttributes = NULL)
	{
		_XAssert(pfn);
		if (!pfn)
			return xeInvalidArgument;

		xdword dwThreadID = 0;
		_Base::Attach(::CreateThread((LPSECURITY_ATTRIBUTES) pSecurityAttributes, 0, (LPTHREAD_START_ROUTINE) pfn, pData, bRun ? 0 : CREATE_SUSPENDED, &dwThreadID));

		return _Base::IsValid() ? xsOK : _XResultLastError;
	}

	// Suspend the thread...
	inline xresult Suspend()
	{
		xcbool bValid = _Base::IsValid();

		_XAssert(bValid);
		if (!bValid)
			return xeUnexpected;

		return _XResultFromWin32(::SuspendThread(_Base::GetSafeType()));
	}

	// Resume the thread...
	inline xresult Resume()
	{
		xcbool bValid = _Base::IsValid();

		_XAssert(bValid);
		if (!bValid)
			return xeUnexpected;

		return _XResultFromWin32(::ResumeThread(_Base::GetSafeType()));
	}

	// Terminate the thread...
	inline xresult Terminate(xcdword dwExitCode)
	{
		xcbool bValid = _Base::IsValid();

		_XAssert(bValid);
		if (!bValid)
			return xeUnexpected;

		return ::TerminateThread(_Base::GetSafeType(), dwExitCode) ? xsOK : _XResultLastError;
	}

	// Get the thread ID...
	inline xdword GetThreadID()
	{
		xcbool bValid = _Base::IsValid();

		_XAssert(bValid);
		if (!bValid)
			return 0;

		return ::GetThreadId(_Base::GetSafeType());
	}

	// Get the thread exit code...
	inline xdword GetExitCode()
	{
		xcbool bValid = _Base::IsValid();

		_XAssert(bValid);
		if (!bValid)
			return 0;

		xdword dwExitCode = 0;
		return ::GetExitCodeThread(_Base::GetSafeType(), &dwExitCode) ? dwExitCode : 0;
	}

	// Check if the thread is running...
	inline xbool IsRunning()
	{
		return _Base::IsValid() && GetExitCode() == STILL_ACTIVE;
	}

	// Wait until thread operation is finished...
	inline xdword WaitForCompletion(xcuint32 uiMilliseconds)
	{
		if (IsRunning())
		{
			::WaitForSingleObject(GetSafeType(), uiMilliseconds);
		}

		return GetExitCode();
	}
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XThreadBaseT: Generic thread...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XThreadBaseT : public t_Platform
{
public:
// Construction/Destruction...
	inline explicit XThreadBaseT()
	{
	}

	inline ~XThreadBaseT()
	{
	}

public:
// Operations...
	// Create the thread...
	inline xresult Create(_XThreadProc pfn, xvoid* pData, xcbool bRun = true, xvoid* pSecurityAttributes = NULL)
	{
		_XAssert(pfn);
		if (!pfn)
			return xeInvalidArgument;

		return t_Platform::Create(pfn, pData, bRun, pSecurityAttributes);
	}

	// Create the thread...
	inline xresult Create(IXFunctor* pFunctor, xcbool bRun = true, xvoid* pSecurityAttributes = NULL)
	{
		_XAssert(pFunctor);
		if (!pFunctor)
			return xeInvalidArgument;

		pFunctor->AddRef();

		xresult xr = Create(_ThreadProc, pFunctor, bRun, pSecurityAttributes);
		if (_XFailed(xr))
		{
			// If the thread cannot be created we need to release the functor here...
			pFunctor->Release();
		}

		return xr;
	}

	// Suspend the thread...
	inline xresult Suspend()
	{
		return t_Platform::Suspend();
	}

	// Resume the thread...
	inline xresult Resume()
	{
		return t_Platform::Resume();
	}

	// Terminate the thread...
	inline xresult Terminate(xcdword dwExitCode = xeTerminate)
	{
		return t_Platform::Terminate(dwExitCode);
	}

	// Get the thread ID...
	inline xdword GetThreadID() const
	{
		return t_Platform::GetThreadID();
	}

	// Get the thread exit code...
	inline xdword GetExitCode()
	{
		return t_Platform::GetExitCode();
	}

	// Check if the thread is running...
	inline xbool IsRunning()
	{
		return t_Platform::IsRunning();
	}

	// Wait until thread operation is finished...
	inline xdword WaitForCompletion(xcuint32 uiMilliseconds = _XInfinite)
	{
		return t_Platform::WaitForCompletion(uiMilliseconds);
	}

private:
// Internal functions...
	inline static xdword _xstdcall _ThreadProc(xvoid* pData)
	{
		IXFunctor* pFunctor = static_cast<IXFunctor*>(pData);

		_XAssert(pFunctor);
		if (!pFunctor)
			return xeInvalidArgument;

		xresult xr = pFunctor->Call();

		pFunctor->Release();

		return xr;
	}
};

// Platform type...
typedef XThreadBaseT<XThreadPlatform> XThread;



// #######################################################################################



// ***************************************************************************************
// XThreadT: Generic thread...
// ***************************************************************************************
template
<
	typename t_Type
>
class XThreadT : public XThreadBaseT<XThreadPlatform>
{
// Base class...
typedef XThreadBaseT<XThreadPlatform> _Base;

public:
// Construction/Destruction...
	inline explicit XThreadT()
	{
	}

	// Create the thread...
	inline xresult Create(t_Type* pType, xresult (t_Type::*pfn)(), xcbool bRun = true, xvoid* pSecurityAttributes = NULL)
	{
		_XAssert(pType && pfn);
		if (!pType || !pfn)
			return xeInvalidArgument;

		if (!m_pFunctor)
		{
			m_pFunctor = _xnew XFunctorT<t_Type>;
			if (!m_pFunctor)
				return xeOutOfMemory;
		}
		
		xresult xr = m_pFunctor->Set(pType, pfn);
		if (_XFailed(xr))
			return xr;

		return _Base::Create(m_pFunctor, bRun, pSecurityAttributes);
	}

private:
// Internal data...
	XIPtrT<XFunctorT<t_Type> > m_pFunctor;
};



// #######################################################################################



// ***************************************************************************************
// XArrayT: This object encapsulates an array...
// ***************************************************************************************
template
<
	typename t_Type,
	typename t_Argument = const t_Type&
>
class XArrayT
{
public:
// Construction/Destruction
	inline XArrayT()
	{
		m_pData = NULL;

		m_uiSize = m_uiSizeMax = m_uiGrow = 0;
	}

	inline XArrayT(const XArrayT& arr)
	{
		Copy(arr);
	}

	inline ~XArrayT()
	{
		if (m_pData)
		{
			for (UINT ui = 0; ui < m_uiSize; ui++)
			{
				(m_pData + ui)->~t_Type();
			}

			_xfree(m_pData);
		}
	}

public:
// Operations
	// Attributes...
	inline xuint32 GetSize() const
	{
		return m_uiSize;
	}

	inline xvoid SetSize(xcuint32 uiSize, xcint32 iGrow = -1)
	{
		if (iGrow >= 0)
		{
			m_uiGrow = (xuint32) iGrow;
		}

		if (uiSize == 0)
		{
			// Empty...
			if (m_pData)
			{
				for (xuint32 ui = 0; ui < m_uiSize; ui++)
				{
					(m_pData + ui)->~t_Type();
				}

				_xfree(m_pData);

				m_pData = NULL;
			}

			m_uiSize = m_uiSizeMax = 0;
		}
		else if (!m_pData)
		{
			// Create buffer big enough to hold number of requested elements...
			xcuint32 uiSizeAlloc = __max(uiSize, m_uiGrow);

			m_pData = (t_Type*) _xalloc(uiSizeAlloc * sizeof(t_Type));
			
			for (xuint32 ui = 0; ui < uiSize; ui++)
			{
#pragma push_macro("new")
#undef new
				// Call placement "new"...
				::new((xvoid*) (m_pData + ui)) t_Type;
#pragma pop_macro("new")
			}

			m_uiSize = uiSize;

			m_uiSizeMax = uiSizeAlloc;
		}
		else if (uiSize <= m_uiSizeMax)
		{
			// It fits...
			if (uiSize > m_uiSize)
			{
				// Initialize the new elements...
				_XMemoryZero(m_pData + m_uiSize, (uiSize - m_uiSize) * sizeof(t_Type));

				for (xuint32 ui = 0; ui < (uiSize - m_uiSize); ui++)
				{
#pragma push_macro("new")
#undef new
					// Call placement "new"...
					::new((xvoid*) (m_pData + m_uiSize + ui)) t_Type;
#pragma pop_macro("new")
				}
			}
			else if (m_uiSize > uiSize)
			{
				// Destroy the old elements
				for (xuint32 ui = 0; ui < (m_uiSize - uiSize); ui++)
				{
					(m_pData + uiSize + ui)->~t_Type();
				}
			}

			m_uiSize = uiSize;
		}
		else
		{
			// Grow array...
			xuint32 uiGrow = m_uiGrow;

			if (!uiGrow)
			{
				// Heuristically determine growth when "uiGrow == 0" (this avoids heap fragmentation in many situations)...
				uiGrow = m_uiSize / 8;
				uiGrow = (uiGrow < 4) ? 4 : ((uiGrow > 1024) ? 1024 : uiGrow);
			}

			xcuint32 uiSizeMaxNew = (uiSize < m_uiSizeMax + uiGrow) ? (m_uiSizeMax + uiGrow) : uiSize;

			_XAssert(uiSizeMaxNew >= m_uiSizeMax);
			if (uiSizeMaxNew  < m_uiSizeMax)
				return;

			t_Type* pData = (t_Type*) _xalloc(uiSizeMaxNew * sizeof(t_Type));

			// Copy new data from old...
			_XMemoryCopy(pData, m_pData, m_uiSize * sizeof(t_Type));

			// Construct remaining elements...
			_XAssert(uiSize > m_uiSize);
			
			_XMemoryZero(pData + m_uiSize, (uiSize - m_uiSize) * sizeof(t_Type));

			for (xuint32 ui = 0; ui < (uiSize - m_uiSize); ui++)
			{
#pragma push_macro("new")
#undef new
				// Call placement "new"...
				::new((xvoid*) (pData + m_uiSize + ui)) t_Type;
#pragma pop_macro("new")
			}

			// Get rid of old stuff...
			_xfree(m_pData);

			m_pData = pData;
			
			m_uiSize = uiSize;
			
			m_uiSizeMax = uiSizeMaxNew;
		}
	}

	inline xbool IsEmpty() const
	{
		return m_uiSize == 0;
	}

	// Elements access...
	inline const t_Type& GetAt(xcuint32 uiIndex) const
	{
		_XAssert(uiIndex >= 0 && uiIndex < m_uiSize);

		return m_pData[uiIndex];
	}

	inline t_Type& GetAt(xcuint32 uiIndex)
	{
		_XAssert(uiIndex >= 0 && uiIndex < m_uiSize);

		return m_pData[uiIndex];
	}

	inline const t_Type& operator[](xcuint32 uiIndex) const
	{
		_XAssert(uiIndex >= 0 && uiIndex < m_uiSize);

		return GetAt(uiIndex);
	}

	inline t_Type& operator[](xcuint32 uiIndex)
	{
		_XAssert(uiIndex >= 0 && uiIndex < m_uiSize);

		return GetAt(uiIndex);
	}

	// Direct access to the buffer (may return NULL)...
	inline const t_Type* GetData() const
	{
		return (const t_Type*) m_pData;
	}

	inline t_Type* GetData()
	{
		return (t_Type*) m_pData;
	}

	// Element setting/adding...
	inline xuint32 Add(t_Argument object)
	{
		xcuint32 uiIndex = m_uiSize;

		SetAtGrow(uiIndex, object);
		
		return uiIndex;
	}

	inline xvoid InsertAt(xcuint32 uiIndex, t_Argument object, xcuint32 uiCount = 1)
	{
		_XAssert(uiIndex >= 0 && uiCount > 0);
		if (uiIndex < 0 || uiCount <= 0)
			return;

		if (uiIndex >= m_uiSize)
		{
			// Adding after the end of the array...
			SetSize(uiIndex + uiCount, -1);
		}
		else
		{
			// Inserting in the middle of the array...
			xcuint32 uiSizeOld = m_uiSize;

			SetSize(m_uiSize + uiCount, -1);

			// Destroy intial data before copying over it...
			for (xuint32 ui = 0; ui < uiCount; ui++)
			{
				(m_pData + uiSizeOld + ui)->~t_Type();
			}

			// Shift old data up to fill gap...
			_XMemoryMove(m_pData + uiIndex + uiCount, m_pData + uiIndex, (uiSizeOld - uiIndex) * sizeof(t_Type));

			// Re-init slots we copied from...
			_XMemoryZero(m_pData + uiIndex, uiCount * sizeof(t_Type));

			for (xuint32 ui = 0; ui < uiCount; ui++)
			{
#pragma push_macro("new")
#undef new
				// Call placement "new"...
				::new((xvoid*) (m_pData + m_uiSize + ui)) t_Type;
#pragma pop_macro("new")
			}
		}

		// Insert new value in the gap...
		_XAssert(uiIndex + uiCount <= m_uiSize);

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			m_pData[uiIndex + ui] = object;
		}
	}

	inline xvoid InsertAt(xcuint32 uiIndex, XArrayT* pArray)
	{
		_XAssert(pArray);
		if (!pArray)
			return;

		if (pArray->GetSize())
		{
			InsertAt(uiIndex, pArray->GetAt(0), pArray->GetSize());

			for (xuint32 ui = 0; ui < pArray->GetSize(); ui++)
			{
				SetAt(uiIndex + ui, pArray->GetAt(ui));
			}
		}
	}

	inline xvoid SetAt(xcuint32 uiIndex, t_Argument object)
	{
		_XAssert(uiIndex >= 0 && uiIndex < m_uiSize);

		m_pData[uiIndex] = object; 
	}

	inline xvoid SetAtGrow(xcuint32 uiIndex, t_Argument object)
	{
		_XAssert(uiIndex >= 0);
		if (uiIndex < 0)
			return;

		if (uiIndex >= m_uiSize)
		{
			SetSize(uiIndex + 1, -1);
		}

		m_pData[uiIndex] = object;
	}

	// Array operations...
	inline xuint32 Append(const XArrayT& arr)
	{
		_XAssert(this != &arr);
		if (this == &arr)
			return -1;

		xcuint32 uiSizeOld = m_uiSize;

		SetSize(m_uiSize + arr.m_uiSize);

		CopyElements(m_pData + uiSizeOld, arr.m_pData, arr.m_uiSize);

		return uiSizeOld;
	}

	inline xvoid Copy(const XArrayT& arr)
	{
		_XAssert(this != &arr);
		if (this == &arr)
			return;

		SetSize(source.m_uiSize);

		CopyElements(m_pData, arr.m_pData, arr.m_uiSize);
	}

	// Deletion...
	inline xvoid RemoveAt(xcuint32 uiIndex, xcuint32 uiCount = 1)
	{
		_XAssert((uiIndex + uiCount) <= m_uiSize);
		if ((uiIndex + uiCount) > m_uiSize)
			return;

		// Just remove a range...
		xcuint32 uiMove = m_uiSize - (uiIndex + uiCount);

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			(m_pData + uiIndex + ui)->~t_Type();
		}

		if (uiMove)
		{
			_XMemoryMove(m_pData + uiIndex, m_pData + uiIndex + uiCount, uiMove * sizeof(t_Type));
		}

		m_uiSize -= uiCount;
	}

	inline xvoid RemoveAll()
	{
		SetSize(0, -1);
	}

	// Compacting...
	inline xvoid Compact()
	{
		if (m_uiSize != m_uiSizeMax)
		{
			// Shrink to desired size...
			t_Type* pData = NULL;

			if (m_uiSize)
			{
				pData = (t_Type*) _xalloc(m_uiSize * sizeof(t_Type));

				// Copy new data from old...
				_XMemoryCopy(pData, m_pData, m_uiSize * sizeof(t_Type));
			}

			// Get rid of old stuff...
			_xfree(m_pData);

			m_pData = pData;

			m_uiSizeMax = m_uiSize;
		}
	}

private:
// Internal functions...
	inline xvoid CopyElements(t_Type* pTarget, const t_Type* pSource, xcuint32 uiCount)
	{
		_XAssert(uiCount == 0 || pTarget);
		_XAssert(uiCount == 0 || pSource);

		// Default copy using assignment...
		xuint32 ui = uiCount;

		while (ui--)
		{
			*pTarget++ = *pSource++;
		}
	}

private:
// Internal data...
	t_Type* m_pData;
	xuint32 m_uiSize;
	xuint32 m_uiSizeMax;
	xuint32 m_uiGrow;
};



// #######################################################################################



// Shortcuts...
#define _XA2W(__XA)						_XConversionA2W(__XA)
#define _XW2A(__XW)						_XConversionW2A(__XW)
#define _XA2UTF8(__XA)					(xcchara*) __XA
#define _XW2UTF8(__XW)					_XConversionW2UTF8(__XW)
#define _XUTF82W(__XUTF8)				_XConversionUTF82W(__XUTF8)
#define _XUTF82A(__XUTF8)				(xcchara*) __XUTF8

#ifdef X_WINDOWS
	#define _XA2I(__XA)					_XA2W(__XA)
	#define _XW2I(__XW)					__XW
	#define _XUTF82I(__XUTF8)			_XConversionUTF82W(__XUTF8)
	#define _XI2A(__XI)					_XW2A(__XI)
	#define _XI2W(__XI)					__XI
	#define _XI2UTF8(__XI)				_XConversionW2UTF8(__XI)
#else // X_WINDOWS
	#error "Missing definition..."
#endif

#ifdef X_UNICODE
	#define _XA2T(__XA)					_XA2W(__XA)
	#define _XW2T(__XW)					(xcchart*) __XW
	#define _XT2A(__XT)					_XW2A(__XT)
	#define _XT2W(__XT)					(xccharw*) __XT
	#define _XT2UTF8(__XT)				_XW2UTF8(__XT)
	#define _XUTF82T(__XUTF8)			_XUTF82W(__XUTF8)
	#ifdef X_WINDOWS
		#define _XT2I(__XT)				(xcchari*) __XT
		#define _XI2T(__XI)				(xcchart*) __XI
	#else // X_WINDOWS
		#error "Missing definition..."
	#endif
#else // X_UNICODE
	#define _XA2T(__XA)					(xcchart*) __XA
	#define _XW2T(__XW)					_XW2A(__XW)
	#define _XT2A(__XT)					(xcchara*) __XT
	#define _XT2W(__XT)					_XA2W(__XT)
	#define _XUTF82T(__XUTF8)			(xcchart*) __XUTF8
	#define _XT2UTF8(__XT)				(xcchara*) __XT
	#ifdef X_WINDOWS
		#define _XT2I(__XT)				_XA2W(__XT)
		#define _XI2T(__XI)				_XW2A(__XI)
	#else // X_WINDOWS
		#error "Missing definition..."
	#endif
#endif

#define _XB2H(__XBin, __Size)			_XConversionB2H(__XBin, __Size)
#define _XH2B(__XHex, __Size)			_XConversionH2B(__XHex, __Size)

// Mappings...
#define _XConversion XConversion		_XConversionTemp
#define _XConversionA2W(__XA)			_XConversionTemp.Convert(__XA, CP_ACP)
#define _XConversionW2A(__XW)			_XConversionTemp.Convert(__XW, CP_ACP)
#define _XConversionUTF82W(__XUTF8)		_XConversionTemp.Convert(__XUTF8, CP_UTF8)
#define _XConversionW2UTF8(__XW)		_XConversionTemp.Convert(__XW, CP_UTF8)
#define _XConversionB2H(__XBin, __Size)	_XConversionTemp.ConvertB2H
#define _XConversionH2B(__XHex, __Size)	_XConversionTemp.ConvertH2B



// ***************************************************************************************
// XConversion: Various conversions...
// ***************************************************************************************
class XConversion
{
public:
// Construction/Destruction...
	inline XConversion()
	{
		m_pBuffer = NULL;
	}

	inline ~XConversion()
	{
		Clean();
	}

public:
// Operations...
	// Convert "ANSI" to "UNICODE"...
	inline xccharw* Convert(xcchara* pszA, xcuint32 uiCodePage)
	{
		_XAssert(pszA);
		if (!pszA)
			return &s_szBufferFallback;

		Clean();

		if (!(m_pBuffer = _Convert(pszA, uiCodePage)))
			return &s_szBufferFallback;

		return (xccharw*) m_pBuffer;
	}
	
	// Convert "UNICODE" to "ANSI"...
	inline xcchara* Convert(xccharw* pszW, xcuint32 uiCodePage)
	{
		_XAssert(pszW);
		if (!pszW)
			return (xcchara*) &s_szBufferFallback;

		Clean();

		if (!(m_pBuffer = _Convert(pszW, uiCodePage)))
			return (xcchara*) &s_szBufferFallback;

		return (xcchara*) m_pBuffer;
	}

	// Convert binary to hexadecimal...
	inline xcchart* XConversion::ConvertB2H(xcbyte* pBuffer, xcuint32 uiSizeBuffer)
	{
		_XAssert(pBuffer && uiSizeBuffer);
		if (!pBuffer || !uiSizeBuffer)
			return (xcchart*) s_szBufferFallback;

		xchart* pszHex = NULL;

		for (xuint32 ui = 0; ui < uiSizeBuffer; ui++)
		{
			xchart szTemp[3] = {__XT('\0')};

			_stprintf_s(szTemp, ui != (uiSizeBuffer - 1) ? __XT("%02x ") : __XT("%02x"), pBuffer[ui]);

			xchart* pszTemp = pszHex;

			xcuint32 uiSize = (xcuint32) (_xstrlent(pszHex) + _xstrlent(szTemp) + 1) * sizeof(xchart);

			pszHex = (xchart*) _xalloc(uiSize);
			if (!pszHex)
			{
				_xfree(pszTemp);

				return (xcchart*) s_szBufferFallback;
			}

			_xstrcatt(pszHex, uiSize / sizeof(xchart), pszTemp);

			_xfree(pszTemp);

			_xstrcatt(pszHex, uiSize / sizeof(xchart), szTemp);
		}

		Clean();

		xcuint32 uiSize = (xcuint32) _xstrlent(pszHex + 1) * sizeof(xcchart);

		m_pBuffer = _xalloc(uiSize);
		if (!m_pBuffer)
			return (xcchart*) &s_szBufferFallback;

		_xstrncpyt((xchart*) m_pBuffer, uiSize, pszHex, uiSize);

		_xfree(pszHex);

		return (xcchart*) m_pBuffer;
	}

	// Convert hexadecimal to binary...
	inline xcbyte* XConversion::ConvertH2B(xcchart* pszHex, xuint32* puiSize)
	{
		_XAssert(pszHex);
		if (!pszHex)
			return (xcbyte*) s_szBufferFallback;

		xcchart* psz = pszHex;

		for (/* nothing */; psz && *psz; psz++)
		{
			if ((*psz != __XT(' ')) || (*psz != __XT('\r')) || (*psz != __XT('\n')) || (*psz != __XT('\t')))
			{
				break;
			}
		}

		if (psz && (*psz == __XT('\0')) && ((*(psz + 1) == __XT('x')) || (*(psz + 1) == __XT('X'))))
		{
			psz += 2;
		}

		xbool bHigh = true;
		xbyte byteCurrent = 0;
		XArrayT<xbyte> arr;
		for (/* nothing */; psz && *psz; psz++)
		{
			if ((*psz >= __XT('0')) && (*psz <= __XT('9')))
			{
				byteCurrent |= (xbyte) (*psz - __XT('0'));
			}
			else if ((*psz >= __XT('a')) && (*psz <= __XT('f')))
			{
				byteCurrent |= (xbyte) (*psz - __XT('a') + 10);
			}
			else if ((*psz >= __XT('A')) && (*psz <= __XT('F')))
			{
				byteCurrent |= (xbyte) (*psz - __XT('A') + 10);
			}
			else if ((*psz == __XT(' ')) || (*psz == __XT('\r')) || (*psz == __XT('\n')) || (*psz == __XT('\t')))
			{
				continue;
			}
			else
			{
				(xcbyte*) &s_szBufferFallback;
			}

			if (bHigh)
			{
				byteCurrent <<= 4;
				bHigh = false;
			}
			else
			{
				arr.Add(byteCurrent);
				byteCurrent = 0;
				bHigh = true;
			}
		}

		Clean();

		xcuint32 uiSize = arr.GetSize();

		m_pBuffer = _xalloc(uiSize * sizeof(xbyte));
		if (!m_pBuffer)
			return (xcbyte*) &s_szBufferFallback;

		_XMemoryCopy(m_pBuffer, arr.GetData(), uiSize);

		if (puiSize)
		{
			*puiSize = uiSize;
		}

		return (xcbyte*) m_pBuffer;
	}

	// Convert "ANSI" to "UNICODE" (untracked allocation)...
	inline static xcharw* _Convert(xcchara* pszA, xcuint32 uiCodePage = CP_ACP)
	{
		xcint32 iSize = ::MultiByteToWideChar(uiCodePage, 0, pszA, -1, NULL, 0);
		if (!iSize)
			return NULL;

		xcharw* pszW = (xcharw*) _xalloc(iSize * sizeof(xcharw));
		if (!pszW)
			return NULL;

		return ::MultiByteToWideChar(uiCodePage, 0, pszA, -1, pszW, iSize) ? pszW : (_xfree(pszW), NULL);
	}

	// Convert "UNICODE" to "ANSI" (untracked allocation)...
	inline static xchara* _Convert(xccharw* pszW, xcuint32 uiCodePage = CP_ACP)
	{
		xcint32 iSize = ::WideCharToMultiByte(uiCodePage, 0, pszW, -1, NULL, 0, NULL, NULL);
		if (!iSize)
			return NULL;

		xchara* pszA = (xchara*) _xalloc(iSize * sizeof(xcharw));
		if (!pszW)
			return NULL;

		return ::WideCharToMultiByte(uiCodePage, 0, pszW, -1, pszA, iSize, NULL, NULL) ? pszA : (_xfree(pszA), NULL);
	}

private:
// Internal functions...
	inline xvoid Clean()
	{
		if (m_pBuffer)
		{
			_xfree(m_pBuffer);
			m_pBuffer = NULL;
		}
	}

private:
// Internal data...
	xvoid* m_pBuffer;
	static xccharw s_szBufferFallback;
};

_xselectany xccharw XConversion::s_szBufferFallback = __XW('\0');



// #######################################################################################



// ***************************************************************************************
// XString: This object encapsulates a string...
// ***************************************************************************************
class XString
{
private:
// Internal data...
	struct _Data
	{
		xlong lRefCount;
		xuint32 uiLengthData;
		xuint32 uiLengthAlloc;
		inline xchart* _GetData()
		{
			return (xchart*) (this + 1);
		}
	};

public:
// Construction/Destruction...
	inline XString()
	{
		Init();
	}

	inline XString(const XString& s)
	{
		_XAssert(s.GetData()->lRefCount != 0);

		if (s.GetData()->lRefCount >= 0)
		{
			_XAssert(s.GetData() != s_pDataTemp);

			m_pszData = s.m_pszData;
			
			::InterlockedIncrement(&GetData()->lRefCount);
		}
		else
		{
			Init();

			*this = s.m_pszData;
		}
	}

	inline XString(xcchart sz, xcuint32 uiRepeat = 1)
	{
		_XAssert(!_istlead(sz));

		Init();

		if (uiRepeat >= 1)
		{
			if (Alloc(uiRepeat))
			{
#ifdef X_UNICODE
				for (xuint32 ui = 0; ui < uiRepeat; ui++)
				{
					m_pszData[ui] = sz;
				}
#else // X_UNICODE
				_XMemorySet(m_pszData, sz, uiRepeat);
#endif
			}
		}
	}

	inline XString(xcchart* psz)
	{
		Init();

		xcuint32 uiLength = GetSafeLength(psz);
		if (uiLength)
		{
			if (Alloc(uiLength))
			{
				_XMemoryCopy(m_pszData, psz, uiLength * sizeof(xchart));
			}
		}
	}

	inline XString(xcchart* psz, xcuint32 uiLength)
	{
		Init();

		if (uiLength)
		{
			if (Alloc(uiLength))
			{
				_XMemoryCopy(m_pszData, psz, uiLength * sizeof(xchart));
			}
		}
	}

#ifdef X_UNICODE
	inline XString(xcchara* psz)
	{
		Init();

		if (psz && strlen(psz))
		{
			_XConversion;
			*this = _XA2W(psz);
		}
	}
#else // X_UNICODE
	inline XString(xccharw* psz)
	{
		Init();

		if (psz && wcslen(psz))
		{
			xchara* pszTemp = _XW2A(psz);

			*this = pszTemp;

			_xfree(pszTemp);
		}
	}
#endif

	inline XString(xcbyte* pBuffer)
	{
		Init();

		*this = (xcchara*) pBuffer;
	}

	inline ~XString()
	{
		if (GetData() != s_pDataTemp)
		{
			if (::InterlockedDecrement(&GetData()->lRefCount) <= 0)
			{
				_xfree(GetData());
			}
		}
	}

public:
// Operations...
	// Get length of the string...
	inline xuint32 GetLength() const
	{
		return GetData()->uiLengthData;
	}

	// Check if string is zero length...
	inline xbool IsEmpty() const
	{
		return !GetData()->uiLengthData;
	}

	// Set zero length...
	inline void Empty()
	{
		if (!GetData()->uiLengthData)
			return;

		if (GetData()->lRefCount >= 0)
		{
			Release();
		}
		else
		{
			*this = __XT("");
		}

		_XAssert(GetData()->uiLengthData == 0);

		_XAssert(GetData()->lRefCount < 0 || GetData()->uiLengthAlloc == 0);
	}

	// Get a character...
	inline xchart GetAt(xcuint32 uiIndex) const
	{
		_XAssert(uiIndex < GetData()->uiLengthData);
		if (uiIndex >= GetData()->uiLengthData)
			return __XT('\0');

		return m_pszData[uiIndex];
	}

	// Get a character...
	inline xchart operator[](xcuint32 uiIndex) const
	{
		return GetAt(uiIndex);
	}

	// Set a character...
	inline void SetAt(xcuint32 uiIndex, xcchart sz)
	{
		_XAssert(uiIndex < GetData()->uiLengthData);
		if (uiIndex >= GetData()->uiLengthData)
			return;

		Copy();

		m_pszData[uiIndex] = sz;
	}

	// Buffer access...
	inline operator xcchart*() const
	{
		return m_pszData;
	}

	// Assignment...
	inline XString& operator=(const XString& s)
	{
		if (m_pszData != s.m_pszData)
		{
			if ((GetData()->lRefCount < 0 && GetData() != s_pDataTemp) || s.GetData()->lRefCount < 0)
			{
				// Actual copy necessary since one of the strings is locked...
				AssignCopy(s.GetData()->uiLengthData, s.m_pszData);
			}
			else
			{
				// Can just copy references around...
				Release();

				_XAssert(s.GetData() != s_pDataTemp);

				m_pszData = s.m_pszData;

				::InterlockedIncrement(&GetData()->lRefCount);
			}
		}

		return *this;
	}

	// Assignment...
	inline XString& XString::operator=(xcchart sz)
	{
		_XAssert(!_istlead(sz));

		AssignCopy(1, &sz);
		
		return *this;
	}

#ifdef X_UNICODE
	// Assignment...
	inline XString& XString::operator=(xcchara sz)
	{
		*this = (xchart) sz;

		return *this;
	}
#endif // X_UNICODE

	// Assignment...
	inline XString& operator=(xcchart* psz)
	{
		_XAssert(!psz || IsValid(psz));

		AssignCopy(GetSafeLength(psz), psz);
		
		return *this;
	}

#ifdef X_UNICODE
	// Assignment...
	inline XString& operator=(xcchara* psz)
	{
		if (psz && strlen(psz))
		{
			_XConversion;
			*this = _XA2W(psz);
		}
		else
		{
			Empty();
		}

		return *this;
	}
#else // X_UNICODE
	// Assignment...
	inline XString& operator=(xccharw* psz)
	{
		if (psz && strlen(psz))
		{
			xchara* pszTemp = _XW2A(psz);

			*this = pszTemp;

			_xfree(pszTemp);
		}
		else
		{
			Empty();
		}

		return *this;
	}
#endif

	// Assignment...
	inline XString& XString::operator=(xcbyte* p)
	{
		*this = (xcchara*) p;

		return *this;
	}

	// Concatenation...
	inline XString& XString::operator+=(const XString& s)
	{
		ConcatInPlace(s.GetData()->uiLengthData, s.m_pszData);

		return *this;
	}

	// Concatenation...
	inline XString& XString::operator+=(xcchart sz)
	{
		ConcatInPlace(1, &sz);

		return *this;
	}

#ifdef X_UNICODE
	// Concatenation...
	inline XString& XString::operator+=(xcchara sz)
	{
		*this += (xchart) sz;

		return *this;
	}
#endif // X_UNICODE

	// Concatenation...
	inline XString& XString::operator+=(xcchart* psz)
	{
		_XAssert(!psz || IsValid(psz));

		ConcatInPlace(GetSafeLength(psz), psz);
		
		return *this;
	}

	// Concatenation...
	friend inline XString _xstdcall operator+(const XString& s1, const XString& s2)
	{
		XString s;

		s.ConcatCopy(s1.GetData()->uiLengthData, s1.m_pszData, s2.GetData()->uiLengthData, s2.m_pszData);
		
		return s;
	}

	// Concatenation...
	friend inline XString _xstdcall operator+(const XString& s, xcchart sz)
	{
		XString sTemp;

		sTemp.ConcatCopy(s.GetData()->uiLengthData, s.m_pszData, 1, &sz);
		
		return sTemp;
	}

	// Concatenation...
	friend inline XString _xstdcall operator+(xcchart sz, const XString& s)
	{
		XString sTemp;
		
		sTemp.ConcatCopy(1, &sz, s.GetData()->uiLengthData, s.m_pszData);
		
		return sTemp;
	}

#ifdef X_UNICODE
	// Concatenation...
	friend inline XString _xstdcall operator+(const XString& s, const char sz)
	{
		return s + (xchart) sz;
	}

	// Concatenation...
	friend inline XString _xstdcall operator+(const char sz, const XString& s)
	{
		return (xchart) sz + s;
	}
#endif // X_UNICODE

	// Concatenation...
	friend inline XString _xstdcall operator+(const XString& s, xcchart* psz)
	{
		_XAssert(!psz || IsValid(psz));

		XString sTemp;
		
		sTemp.ConcatCopy(s.GetData()->uiLengthData, s.m_pszData, GetSafeLength(psz), psz);
		
		return sTemp;
	}

	// Concatenation...
	friend inline XString _xstdcall operator+(xcchart* psz, const XString& s)
	{
		_XAssert(!psz || IsValid(psz));

		XString sTemp;
		
		sTemp.ConcatCopy(GetSafeLength(psz), psz, s.GetData()->uiLengthData, s.m_pszData);
		
		return sTemp;
	}

	// Compare two strings...
	inline xint32 Compare(xcchart* psz) const
	{
		return _xstrcmpt(m_pszData, psz);
	}

	// Compare two strings (case insensitive)...
	inline xint32 CompareNoCase(xcchart* psz) const
	{
		return _xstricmpt(m_pszData, psz);
	}

	// Compare two strings locale specific...
	inline xint32 Collate(xcchart* psz) const
	{
		return _xstrcollt(m_pszData, psz);
	}

	// Compare two strings locale specific (case insensitive)...
	inline xint32 CollateNoCase(xcchart* psz) const
	{
		return _xstricollt(m_pszData, psz);
	}

	// Substring extraction...
	inline XString Mid(xcuint32 uiFirst) const
	{
		return Mid(uiFirst, GetData()->uiLengthData - uiFirst);
	}

	// Substring extraction...
	inline XString Mid(xcuint32 uiFirst, xcuint32 uiCount) const
	{
		xuint32 uiCountCopy = uiCount;

		if (uiFirst + uiCount > GetData()->uiLengthData)
		{
			uiCountCopy = GetData()->uiLengthData - uiFirst;
		}

		if (uiFirst > GetData()->uiLengthData)
		{
			uiCountCopy = 0;
		}

		XString s;

		AllocCopy(s, uiCount, uiFirst, 0);
		
		return s;
	}

	// Substring extraction...
	inline XString Left(xcuint32 uiCount) const
	{
		xuint32 uiCountCopy = uiCount;

		if (uiCount > GetData()->uiLengthData)
		{
			uiCountCopy = GetData()->uiLengthData;
		}

		XString s;

		AllocCopy(s, uiCount, 0, 0);
		
		return s;
	}

	// Substring extraction...
	inline XString Right(xcuint32 uiCount) const
	{
		xuint32 uiCountCopy = uiCount;

		if (uiCount > GetData()->uiLengthData)
		{
			uiCountCopy = GetData()->uiLengthData;
		}

		XString s;

		AllocCopy(s, uiCount, GetData()->uiLengthData - uiCount, 0);
		
		return s;
	}

	// Find first not matching character...
	inline XString SpanIncluding(xcchart* pszCharSet) const
	{
		_XAssert(IsValid(pszCharSet));

		return Left((xuint32) _xstrspnt(m_pszData, pszCharSet));
	}

	// Find first matching character...
	inline XString SpanExcluding(xcchart* pszCharSet) const
	{
		_XAssert(IsValid(pszCharSet));

		return Left((xuint32) _xstrcspnt(m_pszData, pszCharSet));
	}

	// Upper...
	inline void XString::Upper()
	{
		Copy();

		_xstruprt(m_pszData, GetLength() + 1);
	}

	// Lower...
	inline void XString::Lower()
	{
		Copy();

		_xstrlwrt(m_pszData, GetLength() + 1);
	}

	// Reverse...
	inline void XString::Reverse()
	{
		Copy();

		_xstrrevt(m_pszData);
	}

	// Remove whitespaces starting from left...
	inline void TrimLeft()
	{
		Copy();

		// Find first non space character...
		xcchart* psz = m_pszData;
		while (_istspace(*psz))
		{
			psz = ::CharNext(psz);
		}

		// Fix up data and length...
		xcuint32 uiDataLength = GetData()->uiLengthData - (xuint32) (xulongptr) (psz - m_pszData);

		_XMemoryMove(m_pszData, psz, (uiDataLength + 1) * sizeof(xchart));
		
		GetData()->uiLengthData = uiDataLength;
	}

	// Remove whitespaces starting from right...
	inline void TrimRight()
	{
		Copy();

		// Find beginning of trailing spaces by starting at beginning (DBCS aware)...
		xchart* psz = m_pszData;
		xchart* pszLast = NULL;
		while (*psz != '\0')
		{
			if (_istspace(*psz))
			{
				if (!pszLast)
				{
					pszLast = psz;
				}
			}
			else
			{
				pszLast = NULL;
			}

			psz = ::CharNext(psz);
		}

		if (pszLast)
		{
			// Truncate at trailing space start...
			*pszLast = __XT('\0');

			GetData()->uiLengthData = (xint32) (xdwordptr) (pszLast - m_pszData);
		}
	}

	// Remove continuous occurrences of "sz" starting from left...
	inline void TrimLeft(xcchart sz)
	{
		// Find first non matching character...
		Copy();

		xcchart* psz = m_pszData;

		while (sz == *psz)
		{
			psz = ::CharNext(psz);
		}

		if (psz != m_pszData)
		{
			// Fix up data and length...
			xcuint32 uiDataLength = GetData()->uiLengthData - (xuint32) (xulongptr) (psz - m_pszData);

			_XMemoryMove(m_pszData, psz, (uiDataLength + 1) * sizeof(xchart));
			
			GetData()->uiLengthData = uiDataLength;
		}
	}

	// Remove continuous occcurrences of characters in "psz" starting from left...
	inline void TrimLeft(xcchart* psz)
	{
		// If we're not trimming anything, we're not doing any work...
		if (!GetSafeLength(psz))
			return;

		Copy();

		xcchart* pszTemp = m_pszData;

		while (*pszTemp != __XT('\0'))
		{
			xchart* pszNext = ::CharNext(pszTemp);

			if (pszNext > pszTemp + 1)
			{
				if (!_CharNext(psz, *pszTemp, *(pszTemp + 1)))
					break;
			}
			else
			{
				if (!_xstrchrt(psz, *pszTemp))
					break;
			}

			pszTemp = pszNext;
		}

		if (pszTemp != m_pszData)
		{
			// Fix up data and length...
			xcuint32 uiDataLength = GetData()->uiLengthData - (xuint32) (xulongptr) (pszTemp - m_pszData);

			_XMemoryMove(m_pszData, pszTemp, (uiDataLength + 1) * sizeof(xchart));
			
			GetData()->uiLengthData = uiDataLength;
		}
	}

	// Remove continuous occurrences of sz starting from right...
	inline void TrimRight(xcchart sz)
	{
		// Find beginning of trailing matches by starting at beginning (DBCS aware)...
		Copy();

		xchart* psz = m_pszData;
		xchart* pszLast = NULL;

		while (*psz != __XT('\0'))
		{
			if (*psz == sz)
			{
				if (!pszLast)
				{
					pszLast = psz;
				}
			}
			else
			{
				pszLast = NULL;
			}

			psz = ::CharNext(psz);
		}

		if (pszLast)
		{
			// Truncate at left most matching character...
			*pszLast = __XT('\0');

			GetData()->uiLengthData = (xint32) (xulongptr) (pszLast - m_pszData);
		}
	}

	// Remove continuous occurrences of characters in passed string starting from right...
	inline void TrimRight(xcchart* psz)
	{
		// Find beginning of trailing matches by starting at beginning (DBCS aware)...
		Copy();

		xchart* pszTemp = m_pszData;
		xchart* pszLast = NULL;

		while (*pszTemp != '\0')
		{
			xchart* pszNext = ::CharNext(pszTemp);

			if(pszNext > pszTemp + 1)
			{
				if (_CharNext(psz, *pszTemp, *(pszTemp + 1)))
				{
					if (!pszLast)
					{
						pszLast = pszTemp;
					}
				}
				else
				{
					pszLast = NULL;
				}
			}
			else
			{
				if (_xstrchrt(psz, *pszTemp))
				{
					if (!pszLast)
					{
						pszLast = pszTemp;
					}
				}
				else
				{
					pszLast = NULL;
				}
			}

			pszTemp = pszNext;
		}

		if (pszLast)
		{
			// Truncate at left-most matching character...
			*pszLast = __XT('\0');

			GetData()->uiLengthData = (xuint32) (xulongptr) (pszLast - m_pszData);
		}
	}

	// Replace occurrences of "szOld" with "szNew"...
	inline xuint32 Replace(xcchart szOld, xcchart szNew)
	{
		xuint32 uiCount = 0;

		// Short circuit the nop case...
		if (szOld != szNew)
		{
			// Otherwise modify each character that matches in the string...
			Copy();

			xchart* psz = m_pszData;
			xchart* pszEnd = psz + GetData()->uiLengthData;
			while (psz < pszEnd)
			{
				// Replace instances of the specified character only...
				if (*psz == szOld)
				{
					*psz = szNew;
					uiCount++;
				}

				psz = ::CharNext(psz);
			}
		}

		return uiCount;
	}

	// Replace occurrences of substring "pszOld" with "pszNew". Empty "pszNew" removes instances of "pszOld"...
	inline xint32 Replace(xcchart* pszOld, xcchart* pszNew)
	{
		// Can't have empty or "NULL" "pszOld"...
		xcuint32 uiLength = GetSafeLength(pszOld);
		if (!uiLength)
			return 0;

		xcuint32 uiLengthReplace = GetSafeLength(pszNew);

		// Loop once to figure out the size of the result string...
		xuint32 uiCount = 0;
		xchart* pszStart = m_pszData;
		xchart* pszEnd = m_pszData + GetData()->uiLengthData;
		xchart* pszTarget = NULL;
		while (pszStart < pszEnd)
		{
			while ((pszTarget = (xchart*) _xstrstrt(pszStart, pszOld)))
			{
				pszStart = pszTarget + uiLength;
				uiCount++;
			}

			pszStart += _xstrlent(pszStart) + 1;
		}

		// If any changes were made, make 'em...
		if (uiCount > 0)
		{
			Copy();

			// If the buffer is too small just allocate a new buffer (slow but sure)...
			xuint32 uiLengthOld = GetData()->uiLengthData;
			xcuint32 uiLengthNew = uiLengthOld + (uiLengthReplace - uiLength) * uiCount;
			if (GetData()->uiLengthAlloc < uiLengthNew || GetData()->lRefCount > 1)
			{
				_Data* pData = GetData();
				
				xchart* psz = m_pszData;

				if (!Alloc(uiLengthNew))
					return -1;

				_XMemoryCopy(m_pszData, psz, pData->uiLengthData * sizeof(xchart));
				
				Release(pData);
			}

			// Else we just do it in place...
			pszStart = m_pszData;
			pszEnd = m_pszData + GetData()->uiLengthData;

			// Loop again to actually do the work...
			while (pszStart < pszEnd)
			{
				while ((pszTarget = (xchart*) _xstrstrt(pszStart, pszOld)))
				{
					xcuint32 uiBalance = uiLengthOld - ((xuint32) (xulongptr) (pszTarget - m_pszData) + uiLength);

					_XMemoryMove(pszTarget + uiLengthReplace, pszTarget + uiLength, uiBalance * sizeof(xchart));
					
					_XMemoryCopy(pszTarget, pszNew, uiLengthReplace * sizeof(xchart));
					
					pszStart = pszTarget + uiLengthReplace;
					
					pszStart[uiBalance] = __XT('\0');
					
					uiLengthOld += (uiLengthReplace - uiLength);
				}

				pszStart += _xstrlent(pszStart) + 1;
			}

			_XAssert(m_pszData[uiLengthNew] == __XT('\0'));

			GetData()->uiLengthData = uiLengthNew;
		}

		return uiCount;
	}

	// Remove occurrences of "sz"...
	inline xint32 Remove(xcchart sz)
	{
		Copy();

		xchart* pszStart = m_pszData;
		xchart* psz = m_pszData;

		while (pszStart < (m_pszData + GetData()->uiLengthData))
		{
			if (*pszStart != sz)
			{
				*psz = *pszStart;

				psz = ::CharNext(psz);
			}

			pszStart = ::CharNext(pszStart);
		}

		*psz = __XT('\0');

		xcuint32 uiCount = (xuint32) (xulongptr) (pszStart - psz);
		
		GetData()->uiLengthData -= uiCount;

		return uiCount;
	}

	// Insert character at zero based index (concatenates if index is past end of string)...
	inline xint32 Insert(xcuint32 uiIndex, xcchart sz)
	{
		Copy();

		xuint32 uiLengthNew = GetData()->uiLengthData;
		xuint32 uiIndexInsert = __min(uiIndex, uiLengthNew);

		uiLengthNew++;

		if (GetData()->uiLengthAlloc < uiLengthNew)
		{
			_Data* pData = GetData();
			xchart* psz = m_pszData;

			if (!Alloc(uiLengthNew))
				return -1;

			_XMemoryCopy(m_pszData, psz, (pData->uiLengthData + 1) * sizeof(xchart));

			Release(pData);
		}

		// Move existing bytes down...
		_XMemoryMove(m_pszData + uiIndexInsert + 1, m_pszData + uiIndexInsert, (uiLengthNew - uiIndexInsert) * sizeof(xchart));

		m_pszData[uiIndexInsert] = sz;
		
		GetData()->uiLengthData = uiLengthNew;

		return uiLengthNew;
	}

	// Insert substring at zero based index (concatenates if index is past end of string)...
	inline xint32 Insert(xcuint32 uiIndex, xcchart* psz)
	{
		xuint32 uiLengthInsert = GetSafeLength(psz);
		xuint32 uiLengthNew = GetData()->uiLengthData;

		if (uiLengthInsert > 0)
		{
			Copy();

			xcuint32 uiIndexInsert = __min(uiIndex, uiLengthNew);

			uiLengthNew += uiLengthInsert;

			if (GetData()->uiLengthAlloc < uiLengthNew)
			{
				_Data* pData = GetData();

				xchart* psz = m_pszData;
				
				if (!Alloc(uiLengthNew))
					return -1;
				
				_XMemoryCopy(m_pszData, psz, (pData->uiLengthData + 1) * sizeof(xchart));

				Release(pData);
			}

			// Move existing bytes down...
			_XMemoryMove(m_pszData + uiIndexInsert + uiLengthInsert, m_pszData + uiIndexInsert, (uiLengthNew - uiIndexInsert - uiLengthInsert + 1) * sizeof(xchart));

			_XMemoryCopy(m_pszData + uiIndexInsert, psz, uiLengthInsert * sizeof(xchart));
			
			GetData()->uiLengthData = uiLengthNew;
		}

		return uiLengthNew;
	}

	// Delete "iCount" characters starting at zero based index...
	inline xuint32 Delete(xcuint32 uiIndex, xcuint32 uiCount = 1)
	{
		xuint32 uiLength = GetData()->uiLengthData;
		if (uiCount > 0 && uiIndex < uiLength)
		{
			xcuint32 uiCountDelete =  (uiIndex + uiCount) > uiLength ? uiLength - uiIndex : uiCount;

			Copy();

			xcuint32 uiBytesToCopy = uiLength - (uiIndex + uiCountDelete) + 1;

			_XMemoryMove(m_pszData + uiIndex, m_pszData + uiIndex + uiCountDelete, uiBytesToCopy * sizeof(xchart));

			uiLength -= uiCount;
			
			GetData()->uiLengthData = uiLength;
		}

		return uiLength;
	}

	// Find a character based on a starting position...
	inline xint32 Find(xcchart sz, xcuint32 uiStart) const
	{
		_XAssert(uiStart < GetData()->uiLengthData);
		if (uiStart >= GetData()->uiLengthData)
			return -1;

		xcchart* psz = _xstrchrt(m_pszData + uiStart, (xuchart) sz);

		return psz ? (xint32) (psz - m_pszData) : -1;
	}

	// Find a character (return starting index or -1 if not found)...
	inline xint32 Find(xcchart sz) const
	{
		return Find(sz, 0);
	}

	// Find a character beginning at the end...
	inline xint32 ReverseFind(xchart sz) const
	{
		xcchart* psz = _xstrrchrt(m_pszData, (xuchart) sz);

		return psz ? (xint32) (psz - m_pszData) : -1;
	}

	// Find a character in a set...
	inline xint32 FindOneOf(xcchart* pszCharSet) const
	{
		_XAssert(IsValid(pszCharSet));
		if (!IsValid(pszCharSet))
			return -1;

		xcchart* psz = _xstrpbrkt(m_pszData, pszCharSet);
		
		return psz ? (xint32) (psz - m_pszData) : -1;
	}

	// Find a specific substring based on a starting position...
	inline xint32 Find(xcchart* pszSub, xcuint32 uiStart) const
	{
		_XAssert(IsValid(pszSub));
		if (!IsValid(pszSub))
			return -1;

		if (uiStart >= GetData()->uiLengthData)
			return -1;

		xcchart* psz = _xstrstrt(m_pszData + uiStart, pszSub);

		return psz ? (xint32) (psz - m_pszData) : -1;
	}

	// Find a specific substring...
	inline xint32 Find(xcchart* pszSub) const
	{
		_XAssert(IsValid(pszSub));
		if (!IsValid(pszSub))
			return -1;

		return Find(pszSub, 0);
	}

	int ReverseFind(xcchart* pszSub, xint32 iStart = -1, xcbool bCompareNoCase = false) const
	{
		_XAssert(pszSub);
		if (!pszSub)
			return -1;

		xcint32 iLength = GetLength();
		xcint32 iLengthSub = _xstrlent(pszSub);

		if (0 < iLengthSub && 0 < iLength)
		{
			if (iStart == -1 || iStart >= iLength)
			{
				iStart = iLength - 1;
			}

			for (xchart* pszReverse = m_pszData + iStart; pszReverse >= m_pszData; pszReverse--)
			{
				if (bCompareNoCase ? _tcsnicmp(pszSub, pszReverse, iLengthSub) : _tcsncmp(pszSub, pszReverse, iLengthSub) == 0)
				{
					return (pszReverse - m_pszData);
				}
			}
		}
		return -1;
	}

	// Simple formatting...
	inline xbool _xcdecl Format(xcchart* pszFormat, ...)
	{
		_XAssert(IsValid(pszFormat));
		if (!IsValid(pszFormat))
			return false;

		va_list args;
		va_start(args, pszFormat);

		xbool bResult = FormatV(pszFormat, args);

		va_end(args);

		return bResult;
	}

	inline xbool FormatV(xcchart* pszFormat, va_list args)
	{
		_XAssert(IsValid(pszFormat));
		if (!IsValid(pszFormat))
			return false;

		enum _FormatModifiers
		{
			FORCE_ANSI =	0x10000,
			FORCE_UNICODE =	0x20000,
			FORCE_INT64 =	0x40000
		};

		va_list argsSave = args;

		// Make a guess at the maximum length of the resulting string...
		xuint32 uiLengthMax = 0;
		for (xcchart* psz = pszFormat; *psz != __XT('\0'); psz = ::CharNext(psz))
		{
			// Handle "%" character, but watch out for "%%"...
			if (*psz != __XT('%') || *(psz = ::CharNext(psz)) == __XT('%'))
			{
				// This is instead of "_xstrlent()"...
#if !defined(X_UNICODE) && defined(_MBCS)
				uiLengthMax += (xuint32) (::CharNext(psz) - psz);
#else
				uiLengthMax++;
#endif
				continue;
			}

			xuint32 uiLengthItem = 0;

			// Handle "%" character with format...
			xuint32 uiWidth = 0;
			for (/* nothing */; *psz != __XT('\0'); psz = ::CharNext(psz))
			{
				// Check for valid flags...
				if (*psz == __XT('#'))
				{
					uiLengthMax += 2;
				}
				else if (*psz == __XT('*'))
				{
					uiWidth = va_arg(args, xint32);
				}
				else if (*psz == __XT('-') || *psz == __XT('+') || *psz == __XT('0') || *psz == __XT(' '))
				{
					;
				}
				else
				{
					break;
				}
			}

			// Get width and skip it...
			if (!uiWidth)
			{
				// Width indicated by...
				uiWidth = _ttoi(psz);
				
				for (/* nothing */; *psz != __XT('\0') && _istdigit(*psz); psz = ::CharNext(psz));
			}

			_XAssert(uiWidth >= 0);

			xuint32 uiPrecision = 0;
			if (*psz == __XT('.'))
			{
				// Skip past "." separator (width.precision)...
				psz = ::CharNext(psz);

				// Get precision and skip it...
				if (*psz == __XT('*'))
				{
					uiPrecision = va_arg(args, xuint32);

					psz = ::CharNext(psz);
				}
				else
				{
					uiPrecision = _ttoi(psz);

					for (/* nothing */; *psz != __XT('\0') && _istdigit(*psz); psz = ::CharNext(psz));
				}
			}

			// Should be on type modifier or specifier...
			xuint32 uiModifier = 0;
			if(psz[0] == __XT('I') && psz[1] == __XT('6') && psz[2] == __XT('4'))
			{
				psz += 3;
				uiModifier = FORCE_INT64;
			}
			else
			{
				switch (*psz)
				{
					// Modifiers that affect size...
					case 'h':
						uiModifier = FORCE_ANSI;
						psz = ::CharNext(psz);
						break;
					case 'l':
						uiModifier = FORCE_UNICODE;
						psz = ::CharNext(psz);
						break;

					// Modifiers that do not affect size...
					case 'F':
					case 'N':
					case 'L':
						psz = ::CharNext(psz);
						break;
				}
			}

			// Now should be on specifier...
			switch (*psz | uiModifier)
			{
				// Single characters...
				case 'c':
				case 'C':
					uiLengthItem = 2;
					va_arg(args, xchart);
					break;
				case 'c' | FORCE_ANSI:
				case 'C' | FORCE_ANSI:
					uiLengthItem = 2;
					va_arg(args, xchara);
					break;
				case 'c' | FORCE_UNICODE:
				case 'C' | FORCE_UNICODE:
					uiLengthItem = 2;
					va_arg(args, xcharw);
					break;

				// Strings...
				case 's':
				{
					xcchart* pszArg = va_arg(args, xcchart*);
					if (!pszArg)
					{
						// "(null)"...
						uiLengthItem = 6;
					}
					else
					{
						uiLengthItem = (xint32) _xstrlent(pszArg);
						uiLengthItem = __max(1, uiLengthItem);
					}
					break;
				}
				case 'S':
				{
#ifndef X_UNICODE
					xcharw* pszArg = va_arg(args, xcharw*);
					if (!pszArg)
					{
						// "(null)"...
						uiLengthItem = 6;
					}
					else
					{
						uiLengthItem = (xuint32) wcslen(pszArg);
						uiLengthItem = __max(1, uiLengthItem);
					}
#else // X_UNICODE
					xcchara* pszArg = va_arg(args, xcchara*);
					if (!pszArg)
					{
						// "(null)"...
						uiLengthItem = 6;
					}
					else
					{
						uiLengthItem = (xuint32) strlen(pszArg);
						uiLengthItem = __max(1, uiLengthItem);
					}
#endif
					break;
				}
				case 's' | FORCE_ANSI:
				case 'S' | FORCE_ANSI:
				{
					xcchara* pszArg = va_arg(args, xcchara*);
					if (!pszArg)
					{
						// "(null)"...
						uiLengthItem = 6;
					}
					else
					{
						uiLengthItem = (xuint32) strlen(pszArg);
						uiLengthItem = __max(1, uiLengthItem);
					}
					break;
				}
				case 's' | FORCE_UNICODE:
				case 'S' | FORCE_UNICODE:
				{
					xcharw* pszArg = va_arg(args, xcharw*);
					if (!pszArg)
					{
						// "(null)"...
						uiLengthItem = 6;
					}
					else
					{
						uiLengthItem = (xuint32) wcslen(pszArg);
						uiLengthItem = __max(1, uiLengthItem);
					}
					break;
				}
			}

			// Adjust "uiLengthItem" for strings...
			if (uiLengthItem)
			{
				uiLengthItem = uiPrecision ? uiLengthItem = __min(uiLengthItem, uiPrecision) : __max(uiLengthItem, uiWidth);
			}
			else
			{
				switch (*psz)
				{
					// Integers...
					case 'd':
					case 'i':
					case 'u':
					case 'x':
					case 'X':
					case 'o':
						if (uiModifier & FORCE_INT64)
						{
							va_arg(args, xint64);
						}
						else
						{
							va_arg(args, xint32);
						}
						uiLengthItem = 32;
						uiLengthItem = __max(uiLengthItem, uiWidth + uiPrecision);
						break;
					case 'e':
					case 'E':
					case 'g':
					case 'G':
						va_arg(args, double);
						uiLengthItem = 128;
						uiLengthItem = __max(uiLengthItem, uiWidth + uiPrecision);
						break;
					case 'f':
					{
						// 312 == strlen("-1 + (309 zeroes).")...
						// 309 zeroes == max precision of a double...
						// 6 == adjustment in case precision is not specified which means that the precision defaults to 6...
						xcuint32 uiSize = __max(uiWidth, 312 + uiPrecision + 6);
						xchart* pszTemp = (xchart*) ::_alloca(uiSize);
						double d = va_arg(args, double);

						_stprintf_s(pszTemp, uiSize, __XT("%*.*f"), uiWidth, uiPrecision + 6, d);

						uiLengthItem = (xint32) _xstrlent(pszTemp);
						break;
					}
					case 'p':
						va_arg(args, xvoid*);
						uiLengthItem = 32;
						uiLengthItem = __max(uiLengthItem, uiWidth + uiPrecision);
						break;
					case 'n':
						// No output...
						va_arg(args, xint32*);
						break;

					default:
						// Unknown formatting option...
						_XAssert(false); 
				}
			}

			// Adjust "iLengthMax" for output "LengthItem"...
			uiLengthMax += uiLengthItem;
		}

		if (!GetBuffer(uiLengthMax))
			return false;

		xint32 iResult = _vstprintf_s(m_pszData, uiLengthMax + 1, pszFormat, argsSave);

		_XAssert(iResult <= GetAllocLength());
		
		ReleaseBuffer();
		
		va_end(argsSave);

		return true;
	}

#ifdef X_WINDOWS
	// Formatting for localization (uses "FormatMessage" API)...
	inline xbool _xcdecl FormatMessage(xcchart* pszFormat, ...)
	{
		// Format message into temporary buffer "pszTemp"...
		va_list args;
		va_start(args, pszFormat);

		xchart* pszTemp = NULL;
		xbool bResult = true;
		if (!::FormatMessage(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ALLOCATE_BUFFER, pszFormat, 0, 0, (xchart*) &pszTemp, 0, &args) || !pszTemp)
		{
			bResult = false;
		}

		// Assign "pszTemp" into the resulting string and free it...
		*this = pszTemp;

		::LocalFree(pszTemp);
		
		va_end(args);

		return bResult;
	}
#endif // X_WINDOWS

	inline xchart* Duplicate() const
	{
		xcuint32 uiLength = GetLength() + 1;

		xchart* psz = (xchart*) _xalloc(uiLength * sizeof(xchart));
		if (!psz)
			return NULL;

		_xstrcpyt(psz, uiLength, GetString());

		return psz;
	}

	inline xchari* DuplicateInterfaceString() const
	{
		xcuint32 uiLength = GetLength() + 1;

		xchari* psz = (xchari*) _xalloc(uiLength * sizeof(xchari));
		if (!psz)
			return NULL;

#ifdef X_UNICODE
		_xstrcpyt(psz, uiLength, GetString());
#else // X_UNICODE
		_XConversion;
		_xstrcpyt(psz, uiLength, _XA2W(GetString()));
#endif

		return psz;
	}

#ifdef X_WINDOWS
	inline BSTR DuplicateInterfaceStringCOM() const
	{
		_XConversion;
		return ::SysAllocString(_XT2W(GetString()));
	}
#endif // X_WINDOWS

	// Access to string implementation buffer...
	inline xchart* GetBuffer(xcuint32 uiLengthMin)
	{
		if (GetData()->lRefCount > 1 || uiLengthMin > GetData()->uiLengthAlloc)
		{
			// We have to grow the buffer...
			_Data* pData = GetData();

			xcuint32 uiLengthOld = GetData()->uiLengthData;
			
			if (!Alloc(__max(uiLengthMin, uiLengthOld)))
				return NULL;

			_XMemoryCopy(m_pszData, pData->_GetData(), (uiLengthOld + 1) * sizeof(xchart));
			
			GetData()->uiLengthData = uiLengthOld;

			Release(pData);
		}

		_XAssert(GetData()->lRefCount <= 1);

		_XAssert(m_pszData);

		return m_pszData;
	}

	// Return a pointer to the character storage for this string...
	inline xcchart* GetString() const
	{
		_XAssert(m_pszData);

		return m_pszData;
	}

	// Return a pointer to the character storage for this string...
	inline xchart* GetString()
	{
		_XAssert(m_pszData);

		return m_pszData;
	}

	inline void ReleaseBuffer(xint32 iLengthNew = -1)
	{
		// Just in case "GetBuffer()" was not called...
		Copy();
		
		xuint32 uiLengthNew = iLengthNew == -1 ? (xuint32) _xstrlent(m_pszData) : (xuint32) iLengthNew;

		_XAssert(uiLengthNew <= GetData()->uiLengthAlloc);

		GetData()->uiLengthData = uiLengthNew;

		m_pszData[uiLengthNew] = __XT('\0');
	}

	inline xchart* GetBufferSetLength(xcuint32 uiLengthNew)
	{
		if (!GetBuffer(uiLengthNew))
			return NULL;

		GetData()->uiLengthData = uiLengthNew;

		m_pszData[uiLengthNew] = __XT('\0');

		return m_pszData;
	}

	// Free extra allocated memory...
	inline void Compact()
	{
		_XAssert(GetData()->uiLengthData <= GetData()->uiLengthAlloc);

		if (GetData()->uiLengthData != GetData()->uiLengthAlloc)
		{
			_Data* pData = GetData();
			if (Alloc(GetData()->uiLengthData))
			{
				_XMemoryCopy(m_pszData, pData->_GetData(), pData->uiLengthData * sizeof(xchart));

				_XAssert(m_pszData[GetData()->uiLengthData] == __XT('\0'));
				
				Release(pData);
			}
		}

		_XAssert(GetData());
	}

	// Used to turn ref-counting off...
	inline xchart* LockBuffer()
	{
		xchart* psz = GetBuffer(0);
		if (psz)
		{
			GetData()->lRefCount = -1;
		}

		return psz;
	}

	// Used to turn ref-counting on...
	inline xvoid UnlockBuffer()
	{
		_XAssert(GetData()->lRefCount == -1);

		if (GetData() != s_pDataTemp)
		{
			GetData()->lRefCount = 1;
		}
	}

	// Get allocation length...
	inline xint32 GetAllocLength() const
	{
		return GetData()->uiLengthAlloc;
	}

	// Validity check...
	inline static xbool _xstdcall IsValid(xccharw* psz, xcint32 iLength = -1)
	{
		if (!psz)
			return false;

#ifndef _WIN32_WCE
		return !::IsBadStringPtrW(psz, iLength);
#else // !_WIN32_WCE
		iLength;
		return true;
#endif
	}

	inline static xbool _xstdcall IsValid(xcchara* psz, xcint32 iLength = -1)
	{
		if (!psz)
			return false;

#ifndef _WIN32_WCE
		return !::IsBadStringPtrA(psz, iLength);
#else // !_WIN32_WCE
		iLength;
		return true;
#endif
	}

private:
// Internal functions...
	inline XString::_Data* GetData() const
	{
		return ((_Data*) m_pszData) - 1;
	}

	inline void XString::Init()
	{
		m_pszData = GetEmptyString().m_pszData;
	}

	inline xbool Alloc(xcuint32 uiLength)
	{
		// Always allocate one extra character for '\0' termination. Assumes (optimistically) that data length will equal
		//  allocation length...
		_XAssert(uiLength <= INT_MAX - 1);

		if (!uiLength)
		{
			Init();
		}
		else
		{
			_Data* pData = NULL;
			pData = (_Data*) _xalloc(sizeof(_Data) + (uiLength + 1) * sizeof(xchart));
			if (!pData)
				return false;

			pData->lRefCount = 1;
			pData->_GetData()[uiLength] = __XT('\0');
			pData->uiLengthData = uiLength;
			pData->uiLengthAlloc = uiLength;
			m_pszData = pData->_GetData();
		}

		return true;
	}

	inline void AllocCopy(XString& s, xcint32 iLengthCopy, xcint32 iIndexCopy, xcint32 iLengthExtra) const
	{
		// Will clone the data attached to this string allocating "iLengthExtra" characters. Places results in uninitialized
		//  string "s". Will copy the part or all of original data to start of new string...
		xint32 iLengthNew = iLengthCopy + iLengthExtra;
		if (!iLengthNew)
		{
			s.Init();
		}
		else
		{
			if (s.Alloc(iLengthNew))
			{
				_XMemoryCopy(s.m_pszData, m_pszData + iIndexCopy, iLengthCopy * sizeof(xchart));
			}
		}
	}

	inline xbool AllocWrite(xcuint32 uiLength)
	{
		xbool bResult = true;

		if (GetData()->lRefCount > 1 || uiLength > GetData()->uiLengthAlloc)
		{
			Release();
			bResult = Alloc(uiLength);
		}

		_XAssert(GetData()->lRefCount <= 1);

		return bResult;
	}

	inline xvoid Copy()
	{
		if (GetData()->lRefCount > 1)
		{
			_Data* pData = GetData();

			Release();

			if (Alloc(pData->uiLengthData))
			{
				_XMemoryCopy(m_pszData, pData->_GetData(), (pData->uiLengthData + 1) * sizeof(xchart));
			}
		}

		_XAssert(GetData()->lRefCount <= 1);
	}

	inline void AssignCopy(xcint32 iLength, xcchart* psz)
	{
		if (AllocWrite(iLength))
		{
			_XMemoryCopy(m_pszData, psz, iLength * sizeof(xchart));
			GetData()->uiLengthData = iLength;
			m_pszData[iLength] = '\0';
		}
	}

	inline xbool ConcatCopy(xcuint32 uiLength1, xcchart* psz1, xcuint32 uiLength2, xcchart* psz2)
	{
		// Master concatenation routine. Concatenate two sources assume that "this" is a new "XString" object...
		xbool bResult = true;

		xcuint32 uiLengthNew = uiLength1 + uiLength2;

		if (uiLengthNew && (bResult = Alloc(uiLengthNew)))
		{
			_XMemoryCopy(m_pszData, psz1, uiLength1 * sizeof(xchart));
			_XMemoryCopy(m_pszData + uiLength1, psz2, uiLength2 * sizeof(xchart));
		}

		return bResult;
	}

	inline void ConcatInPlace(xcint32 iLength, xcchart* psz)
	{
		// The main routine for "+=" operators.
		if (!iLength)
			return;

		// If the buffer is too small or we have a width mismatch just allocate a new buffer (slow but sure)...
		if (GetData()->lRefCount > 1 || GetData()->uiLengthData + iLength > GetData()->uiLengthAlloc)
		{
			// we have to grow the buffer, use the ConcatCopy routine
			_Data* pData = GetData();
			if (ConcatCopy(GetData()->uiLengthData, m_pszData, iLength, psz))
			{
				_XAssert(pData);

				XString::Release(pData);
			}
		}
		else
		{
			// Fast concatenation when buffer big enough...
			_XMemoryCopy(m_pszData + GetData()->uiLengthData, psz, iLength * sizeof(xchart));
			GetData()->uiLengthData += iLength;

			_XAssert(GetData()->uiLengthData <= GetData()->uiLengthAlloc);

			m_pszData[GetData()->uiLengthData] = '\0';
		}
	}

	inline xvoid Release()
	{
		if (GetData() != s_pDataTemp)
		{
			_XAssert(GetData()->lRefCount != 0);

			if (::InterlockedDecrement(&GetData()->lRefCount) <= 0)
			{
				_xfree(GetData());
			}

			Init();
		}
	}

	inline static xuint32 _xstdcall XString::GetSafeLength(xcchart* psz)
	{
		return psz ? (xuint32) _xstrlent(psz) : 0;
	}

	inline static xvoid _xstdcall Release(_Data* pData)
	{
		if (pData != s_pDataTemp)
		{
			_XAssert(pData->lRefCount != 0);

			if (::InterlockedDecrement(&pData->lRefCount) <= 0)
			{
				_xfree(pData);
			}
		}
	}

	inline static const XString& _xstdcall GetEmptyString()
	{
		return *(XString*) &s_pszDataTemp;
	}

	inline static xcchart* _CharNext(xcchart* p, xcchart sz1, xcchart sz2)
	{
		_XAssert(IsValid(p));
		if (!IsValid(p))
			return NULL;

		xcchart* psz = NULL;
		while (*p != 0)
		{
			if (*p == sz1 && *(p + 1) == sz2)
			{
				psz = p;
				break;
			}

			p = ::CharNext(p);
		}

		return psz;
	}

protected:
// Internal data...
	// Pointer to ref-counted string data...
	xchart* m_pszData;

private:
// Internal data...
	static xint32 s_iDataInit[];
	static xcchart* s_pszDataTemp;
	static _Data* s_pDataTemp;
};

_xselectany xint32 XString::s_iDataInit[] = {-1, 0, 0, 0};
_xselectany xcchart* XString::s_pszDataTemp = (xcchart*) (((xbyte*) &s_iDataInit) + sizeof(XString::_Data));
_xselectany XString::_Data* XString::s_pDataTemp = (XString::_Data*) &s_iDataInit;



// Comparison operators... 
inline static xbool _xstdcall operator==(const XString& s1, const XString& s2)
{
	return s1.Compare(s2) == 0;
}

inline static xbool _xstdcall operator==(const XString& s1, xcchart* psz2)
{
	return s1.Compare(psz2) == 0;
}

inline static xbool _xstdcall operator==(xcchart* psz1, const XString& s2)
{
	return s2.Compare(psz1) == 0;
}

inline static xbool _xstdcall operator!=(const XString& s1, const XString& s2)
{
	return s1.Compare(s2) != 0;
}

inline static xbool _xstdcall operator!=(const XString& s1, xcchart* psz2)
{
	return s1.Compare(psz2) != 0;
}

inline static xbool _xstdcall operator!=(xcchart* psz1, const XString& s2)
{
	return s2.Compare(psz1) != 0;
}

inline static xbool _xstdcall operator<(const XString& s1, const XString& s2)
{
	return s1.Compare(s2) < 0;
}

inline static xbool _xstdcall operator<(const XString& s1, xcchart* psz2)
{
	return s1.Compare(psz2) < 0;
}

inline static xbool _xstdcall operator<(xcchart* psz1, const XString& s2)
{
	return s2.Compare(psz1) > 0;
}

inline static xbool _xstdcall operator>(const XString& s1, const XString& s2)
{
	return s1.Compare(s2) > 0;
}

inline static xbool _xstdcall operator>(const XString& s1, xcchart* psz2)
{
	return s1.Compare(psz2) > 0;
}

inline static xbool _xstdcall operator>(xcchart* psz1, const XString& s2)
{
	return s2.Compare(psz1) < 0;
}

inline static xbool _xstdcall operator<=(const XString& s1, const XString& s2)
{
	return s1.Compare(s2) <= 0;
}

inline static xbool _xstdcall operator<=(const XString& s1, xcchart* psz2)
{
	return s1.Compare(psz2) <= 0;
}

inline static xbool _xstdcall operator<=(xcchart* psz1, const XString& s2)
{
	return s2.Compare(psz1) >= 0;
}

inline static xbool _xstdcall operator>=(const XString& s1, const XString& s2)
{
	return s1.Compare(s2) >= 0;
}

inline static xbool _xstdcall operator>=(const XString& s1, xcchart* psz2)
{
	return s1.Compare(psz2) >= 0;
}

inline static xbool _xstdcall operator>=(xcchart* psz1, const XString& s2)
{
	return s2.Compare(psz1) <= 0;
}



// #######################################################################################



// ***************************************************************************************
// Append a path...
// ***************************************************************************************
inline static xresult __XPathAppend(xcchart* psz, xcchart* pszAppend, xchart** pszResult)
{
	_XAssert(psz && pszAppend && pszResult);
	if (!psz || !pszAppend || !pszResult)
		return xeInvalidArgument;

	XString s(psz);
	if (s.ReverseFind(_XPathSeparator) != (s.GetLength() - 1))
	{
		s += _XPathSeparator;
	}

	s += pszAppend;

	return (*pszResult = s.Duplicate()) ? xsOK : xeOutOfMemory;
}

#define _XPathAppend __XPathAppend



// #######################################################################################



// ***************************************************************************************
// Remove file name...
// ***************************************************************************************
inline static xresult __XPathRemoveFileName(xcchart* psz, xchart** pszResult)
{
	_XAssert(psz && pszResult);
	if (!psz || !pszResult)
		return xeInvalidArgument;

	XString s(psz);
	xcint32 iPos = s.ReverseFind(_XPathSeparator);
	if (iPos != -1)
	{
		s.SetAt(iPos, __XT('\0'));
	}

	return (*pszResult = s.Duplicate()) ? xsOK : xeOutOfMemory;
}

#define _XPathRemoveFileName __XPathRemoveFileName



// #######################################################################################



// ***************************************************************************************
// Remove extension...
// ***************************************************************************************
inline static xresult __XPathRemoveExtension(xcchart* psz, xchart** pszResult)
{
	_XAssert(psz && pszResult);
	if (!psz || !pszResult)
		return xeInvalidArgument;

	XString s(psz);
	xcint32 iPos = s.ReverseFind(__XT('.'));
	if (iPos != -1)
	{
		s.SetAt(iPos, __XT('\0'));
	}

	return (*pszResult = s.Duplicate()) ? xsOK : xeOutOfMemory;
}

#define _XPathRemoveExtension __XPathRemoveExtension



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// Check if exists...
// ***************************************************************************************
inline static xbool __XPathExists(xcchart* psz)
{
	return psz && ::GetFileAttributes(psz) != INVALID_FILE_ATTRIBUTES;
}

#else
	#error "Missing definition..."
#endif

#define _XPathExists __XPathExists



// #######################################################################################



// Prototypes of trace functions...
typedef void (_xstdcall *XTraceProcA)(xcchara* pszTrace);
typedef void (_xstdcall *XTraceProcW)(xccharw* pszTrace);

// Mappings...
#if ((defined _DEBUG && !defined _XTraceNone) || defined _XTraceRelease)
	#define _XTrace				XTrace::GetTrace().Trace
	#define _XTraceSetConsole	XTrace::GetTrace().SetConsole()
	#define _XTraceSetCustom	XTrace::GetTrace().SetCustom()
	#define _XTraceSetDebug		XTrace::GetTrace().SetDebug()
	#define _XTraceSetFile		XTrace::GetTrace().SetFile
	#define _XTraceReplaceProc	XTrace::GetTrace().ReplaceProc
	#define _XTraceFunction		XTrace::GetTrace().Trace(__FUNCTION__)
#else // ((defined _DEBUG && !defined _XTraceNone) || defined _XTraceRelease)	
	#define _XTrace				_xnooperation
	#define _XTraceSetConsole	_xnooperation
	#define _XTraceSetCustom	_xnooperation
	#define _XTraceSetDebug		_xnooperation
	#define _XTraceSetFile		_xnooperation
	#define _XTraceReplaceProc	_xnooperation
	#define _XTraceFunction		_xnooperation
#endif



// ***************************************************************************************
// XTrace: Uniform way to trace data...
// ***************************************************************************************
#if ((defined _DEBUG && !defined _XTraceNone) || defined _XTraceRelease)

class XTrace : public XCriticalSectionCrossProcess
{	
private:
// Construction...
	inline XTrace() : XCriticalSectionCrossProcess(__XT("XTrace-{5D13C98E-7C71-4fa2-A048-293D7804B291}-skulk"))
	{
		XCriticalSectionCrossProcessLock lock(this);

		m_pfnTraceProcA = DefTraceProcDebugA;
		m_pfnTraceProcW = DefTraceProcDebugW;
	}

public:
// Destruction...
	inline ~XTrace()
	{
	}

public:
// Operations...
	// Singleton access...
	inline static XTrace& GetTrace()
	{
		static XTrace s_Trace;

		return s_Trace;
	}

	// Trace function...
	inline xvoid Trace(xcchara* pszFormat, ...)
	{
		XCriticalSectionCrossProcessLock lock(this);

		if (!m_pfnTraceProcA)
			return;

		// Format process/thread ID...
		xchara szIDs[17] = {'\0'};
		sprintf_s(szIDs, "[P:%04d T:%04d] ", ::GetCurrentProcessId(), ::GetCurrentThreadId());

		// Get arguments...
		va_list args;
		va_start(args, pszFormat);

		// Format arguments...
		xuint32 uiSize = _vscprintf(pszFormat, args) + 1;
		xchara* pszArguments = (xchara*) _xalloc(uiSize * sizeof(xchara));
		if (!pszArguments)
		{
			va_end(args);
			return;
		}

		if (vsprintf_s(pszArguments, uiSize, pszFormat, args) == -1)
			return;

		va_end(args);

		// Format message...
		uiSize = (xuint32) (strlen(szIDs) + strlen(pszArguments) + 3);
		xchara* pszTrace = (xchara*) _xalloc(uiSize * sizeof(xchara));
		if (!pszTrace)
		{
			_xfree(pszArguments);
			return;
		}

		sprintf_s(pszTrace, uiSize, "%s%s\r\n", szIDs, pszArguments);

		_xfree(pszArguments);

		// Trace out...
		m_pfnTraceProcA(pszTrace);

		_xfree(pszTrace);
	}

	// Trace function...
	inline xvoid Trace(xccharw* pszFormat, ...)
	{
		XCriticalSectionCrossProcessLock lock(this);

		if (!m_pfnTraceProcW)
			return;

		// Format process/thread ID...
		xcharw szIDs[17] = {'\0'};
		swprintf_s(szIDs, L"[P:%04d T:%04d] ", ::GetCurrentProcessId(), ::GetCurrentThreadId());

		// Get arguments...
		va_list args;
		va_start(args, pszFormat);

		// Format arguments...
		xuint32 uiSize = _vscwprintf(pszFormat, args) + 1;
		xcharw* pszArguments = (xcharw*) _xalloc(uiSize * sizeof(xcharw));
		if (!pszArguments)
		{	
			va_end(args);
			return;
		}

		if (vswprintf_s(pszArguments, uiSize, pszFormat, args) == -1)
			return;
	
		va_end(args);

		// Format message...
		uiSize = (xuint32) (wcslen(szIDs) + wcslen(pszArguments) + 3);
		xcharw* pszTrace = (xcharw*) _xalloc(uiSize * sizeof(xcharw));
		if (!pszTrace)
		{
			_xfree(pszArguments);
			return;
		}

		swprintf_s(pszTrace, uiSize, L"%s%s\r\n", szIDs, pszArguments);

		_xfree(pszArguments);

		// Trace out...
		m_pfnTraceProcW(pszTrace);

		_xfree(pszTrace);
	}

	// Set console tracing...
	inline xresult SetConsole()
	{
		XCriticalSectionCrossProcessLock lock(this);

		return ReplaceProc(DefTraceProcConsoleA, DefTraceProcConsoleW);
	}

	// Set custom tracing...
	inline xresult SetCustom()
	{
		XCriticalSectionCrossProcessLock lock(this);

		return ReplaceProc(DefTraceProcA, DefTraceProcW);
	}

	// Set debug tracing...
	inline xresult SetDebug()
	{
		XCriticalSectionCrossProcessLock lock(this);

		return ReplaceProc(DefTraceProcDebugA, DefTraceProcDebugW);
	}

	// Set file tracing...
	inline xresult SetFile()
	{
		XCriticalSectionCrossProcessLock lock(this);

		return ReplaceProc(DefTraceProcFileA, DefTraceProcFileW);
	}

	// Replace trace functions...
	inline xresult ReplaceProc(XTraceProcA pfnA, XTraceProcW pfnW)
	{
		_XAssert(pfnA && pfnW);
		if (!pfnA || !pfnW)
			return xeInvalidArgument;

		XCriticalSectionCrossProcessLock lock(this);

		m_pfnTraceProcA = pfnA;
		m_pfnTraceProcW = pfnW;

		return xsOK;
	}

private:
// Internal functions..
	// Trace output to the "XTrace" tracing utility (ANSI)...
	inline static xvoid _xstdcall DefTraceProcA(xcchara* pszTrace)
	{
		if (!pszTrace)
			return;

		_XConversion;
		DefTraceProcW(_XA2W(pszTrace));
	}

	// Trace output to the "XTrace" tracing utility...
	inline static xvoid _xstdcall DefTraceProcW(xccharw* pszTrace)
	{
		if (!pszTrace)
			return;

		// Get the window...
		HWND hXTrace = ::FindWindow(__XT("XTrace-{D69B5245-5EE9-48AF-8024-FED55C3C43A3}-skulk"), NULL);
		if (!hXTrace)
			return;

		// Trace it...
		COPYDATASTRUCT cds;
		_XMemoryZero(&cds, sizeof(COPYDATASTRUCT));
		cds.dwData = 0xc0cac01a;
		cds.cbData = (xuint32) ((wcslen(pszTrace) + 1) * sizeof(xccharw));
		cds.lpData = const_cast<xcharw*>(pszTrace);

		DWORD dwResult = 0;
		::SendMessageTimeout(hXTrace, WM_COPYDATA, NULL, (LPARAM) &cds, SMTO_ABORTIFHUNG | SMTO_NORMAL, 500, &dwResult);
	}

	// Trace output to a file (ANSI)...
	inline static xvoid _xstdcall DefTraceProcFileA(xcchara* pszTrace)
	{
		if (!pszTrace)
			return;

		_XConversion;
		DefTraceProcFileW(_XA2W(pszTrace));
	}

	// Trace output to a file (UNICODE)...
	inline static xvoid _xstdcall DefTraceProcFileW(xccharw* pszTrace)
	{
		if (!pszTrace)
			return;

		HANDLE hFile = ::CreateFile(__XT("C:\\XTraceLog.txt"), GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile == INVALID_HANDLE_VALUE)
			return;

		if (!::GetFileSize(hFile, NULL))
		{
			// Write "UNICODE" BOM...
			xword wBOM = _XUnicodeBOM;
			xdword dwWritten = 0;
			if (!::WriteFile(hFile, &wBOM, sizeof(xword), &dwWritten, NULL))
			{
				::CloseHandle(hFile);
				return;
			}
		}

		::SetFilePointer(hFile, 0, 0, FILE_END);

		xdword dwWritten = 0;
		::WriteFile(hFile, pszTrace, (xulong) (wcslen(pszTrace) * sizeof(xcharw)), &dwWritten, NULL);

		::CloseHandle(hFile);
	}

	// Trace output to standard debugger (ANSI)...
	inline static xvoid _xstdcall DefTraceProcDebugA(xcchara* pszTrace)
	{
		::OutputDebugStringA(pszTrace);
	}

	// Trace output to standard debugger (UNICODE)...
	inline static xvoid _xstdcall DefTraceProcDebugW(xccharw* pszTrace)
	{
		::OutputDebugStringW(pszTrace);
	}

	// Trace output to console (ANSI)...
	inline static xvoid _xstdcall DefTraceProcConsoleA(xcchara* pszTrace)
	{
		::printf(pszTrace);
	}

	// Trace output to console (UNICODE)...
	inline static xvoid _xstdcall DefTraceProcConsoleW(xccharw* pszTrace)
	{
		::wprintf(pszTrace);
	}

private:
// Internal data...
	XTraceProcA m_pfnTraceProcA;
	XTraceProcW m_pfnTraceProcW;
};

#endif // ((defined _DEBUG && !defined _XTraceNone) || defined _XTraceRelease)



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XCoInitialize: Initialize COM libraries...
// ***************************************************************************************
class XCoInitialize
{
private:
// Construction/Destruction...
	inline XCoInitialize()
	{
	}

	inline ~XCoInitialize()
	{
#ifdef _DEBUG
		_XAssert(s_lRefCount == 0);
#endif // _DEBUG
	}

public:
// Operations...
	// Singleton access...
	inline static XCoInitialize& GetCoInitialize()
	{
		static XCoInitialize s_CoInitialize;

		return s_CoInitialize;
	}

	// Initialize COM libraries for calling thread...
	inline xresult Init(xcbool bAppartmentThreaded, xcbool bFallback = true)
	{
		_Data& d = GetData();

		if (d.xrCoInitialize == ~0u)
		{
#if (_WIN32_WINNT >= 0x0400 ) || defined(_WIN32_DCOM)
			if (_XSucceeded(d.xrCoInitialize = ::CoInitializeEx(NULL, bAppartmentThreaded ? COINIT_APARTMENTTHREADED : COINIT_MULTITHREADED)))
			{
				s_iAppartmentThreaded = bAppartmentThreaded ? 1 : 0;
			}
			else if (bFallback && _XSucceeded(d.xrCoInitialize = ::CoInitialize(NULL)))
			{
				s_iAppartmentThreaded = 1;
			}
#else // #if (_WIN32_WINNT >= 0x0400 ) || defined(_WIN32_DCOM)
			if (bFallback && _XSucceeded(d.xrCoInitialize = ::CoInitialize(NULL)))
			{
				s_iAppartmentThreaded = 1;
			}
#endif
			else 
			{
				_XAssert(_XSucceeded(d.xrCoInitialize));
			}
#ifdef _DEBUG
			if (_XSucceeded(d.xrCoInitialize))
			{
				::InterlockedIncrement(&s_lRefCount);
			}
#endif // _DEBUG
			return d.xrCoInitialize;
		}
		else if (_XSucceeded(d.xrCoInitialize))
		{
#ifdef _DEBUG
			::InterlockedIncrement(&s_lRefCount);
#endif // _DEBUG
			return xsFalse;
		}
		else
		{
			_XAssert(_XSucceeded(d.xrCoInitialize));

			return d.xrCoInitialize;
		}
	}

	inline xvoid Shutdown()
	{
		if (_XSucceeded(GetData().xrCoInitialize))
		{
			::CoUninitialize();
#ifdef _DEBUG
			::InterlockedDecrement(&s_lRefCount);
#endif // _DEBUG
		}
	}

	inline xbool IsAppartmentThreaded() const
	{
		_XAssert(s_iAppartmentThreaded != -1);

		return s_iAppartmentThreaded == 1;
	}

private:
// Internal data...
	struct _Data
	{
		_Data()
		{
			dwThreadID = ::GetCurrentThreadId();
			xrCoInitialize = ~0u;
		}

		xdword dwThreadID;
		xresult xrCoInitialize;
	};

private:
// Internal functions...
	inline _Data& GetData()
	{
		for (xuint32 ui = 0; ui < m_arrData.GetSize(); ui++)
		{
			if (m_arrData[ui].dwThreadID == ::GetCurrentThreadId())
			{
				return m_arrData[ui];
			}
		}

		_Data d;
		return m_arrData[m_arrData.Add(d)];
	}

private:
// Internal data...
	static xint32 s_iAppartmentThreaded;

	// Compiler driven TLS does not work if linked into a DLL -> we need to do this crap here...
	XArrayT<_Data> m_arrData;

#ifdef _DEBUG
	static xlong s_lRefCount;
#endif // _DEBUG
};

// Initial threading Model...
_xselectany xint32 XCoInitialize::s_iAppartmentThreaded = -1;

// Debug checking...
#ifdef _DEBUG
	_xselectany xlong XCoInitialize::s_lRefCount = 0;
#endif // _DEBUG

#endif // X_WINDOWS



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XCoInitializeThreadT: Initialize COM libraries for a specific thread...
// ***************************************************************************************
template
<
	typename t_Callee
>
class XCoInitializeThreadT
{
public:
// Construction/Destruction...
	inline XCoInitializeThreadT(xcbool bAppartmentThreaded)
	{
		XCoInitialize::GetCoInitialize().Init(bAppartmentThreaded);
	}

	inline ~XCoInitializeThreadT()
	{
		XCoInitialize::GetCoInitialize().Shutdown();
	}

public:
// Operations...
	inline static LPTHREAD_START_ROUTINE GetThreadProc()
	{
		return _ThreadProc;
	}

private:
// Callbacks...
	inline static xdword _xstdcall _ThreadProc(xvoid* pData)
	{
		_XAssert(t_Callee);
		if (!t_Callee)
			return xePointer;

		XCoInitializeThreadT coinit(XCoInitialize::GetCoInitialize().IsAppartmentThreaded());

		return (t_Callee)(pData);
	}
};

#endif // X_WINDOWS



// #######################################################################################



// ***************************************************************************************
// XCommandLineData: Command line argument struct...
// ***************************************************************************************
class XCommandLineData
{
public:
// Shared data...
	XString sArgument;
	XString sArgumentOption;
	XArrayT<XString> arrArgumentOption;
};



// #######################################################################################



// ***************************************************************************************
// XCommandLineTraits: Command line traits...
// ***************************************************************************************
class XCommandLineTraits
{
public:
// Operations...
	// Win32 style command line...
	struct _Win32
	{
		// Get the argument token...
		inline static xcchart* GetTokenArgument()
		{
			return __XT(" /");
		}

		// Get the option token...
		inline static xcchart* GetTokenOption()
		{
			return __XT(" ");
		}

		// Get the option token separator...
		inline static xcchart* GetTokenOptionSeparator()
		{
			return __XT(" ");
		}

		// Get the quote token...
		inline static xcchart* GetTokenQuote()
		{
			return __XT("\"");
		}
	};
	
	// Unix style command line...
	struct _Unix
	{
		// Get the argument token...
		inline static xcchart* GetTokenArgument()
		{
			return __XT(" --");
		}

		// Get the option token...
		inline static xcchart* GetTokenOption()
		{
			return __XT(" ");
		}

		// Get the option token separator...
		inline static xcchart* GetTokenOptionSeparator()
		{
			return __XT(" ");
		}

		// Get the quote token...
		inline static xcchart* GetTokenQuote()
		{
			return __XT("\"");
		}
	};
};



// #######################################################################################



// ***************************************************************************************
// XCommandLineT: Command line parser...
// ***************************************************************************************
template
<
	typename t_Traits
>
class XCommandLineT
{
public:
// Construction/Destruction...
	inline XCommandLineT()
	{
	}

	inline ~XCommandLineT()
	{
	}

public:
// Operations...	
	// Parse the command line...
	inline xresult Parse(xcchart* pszCommandLine)
	{
		_XAssert(pszCommandLine);
		if (!pszCommandLine)
			return xeInvalidArgument;

		XString sCommandLine(pszCommandLine);

		// Collect positions of argument tokens...
		XArrayT<xint32> arrPositionToken;
		for (xint32 iPositionToken = sCommandLine.Find(t_Traits::GetTokenArgument(), 0); iPositionToken != -1; iPositionToken = sCommandLine.Find(t_Traits::GetTokenArgument(), iPositionToken + 1))
		{
			arrPositionToken.Add(iPositionToken);
		}

		// Collect positions of quotes...
		XArrayT<xint32> arrPositionQuote;
		for (xint32 iPositionQuote = sCommandLine.Find(t_Traits::GetTokenQuote(), 0); iPositionQuote != -1; iPositionQuote = sCommandLine.Find(t_Traits::GetTokenQuote(), iPositionQuote + 1))
		{
			arrPositionQuote.Add(iPositionQuote);
		}

		// Check for valid quotes...
		_XAssert(arrPositionQuote.GetSize() % 2 == 0);
		if (arrPositionQuote.GetSize() % 2 != 0)
			return xeUnexpected;

		// Check if a token is inside quotes -> if so skip it...
		for (xuint32 uiToken = 0; uiToken < arrPositionToken.GetSize(); uiToken++)
		{
			for (xuint32 uiQuote = 0; uiQuote < arrPositionQuote.GetSize(); uiQuote += 2)
			{
				if (arrPositionToken[uiToken] > arrPositionQuote[uiQuote] &&
					arrPositionToken[uiToken] < arrPositionQuote[uiQuote + 1])
				{
					arrPositionToken.RemoveAt(uiToken);
					uiToken--;
					break;
				}
			}
		}

		// Collect parts...
		XArrayT<XString> arrPart;
		for (xuint32 ui = 0; ui < arrPositionToken.GetSize(); ui++)
		{
			arrPart.Add(sCommandLine.Mid(arrPositionToken[ui], (ui < (arrPositionToken.GetSize() - 1) ? arrPositionToken[ui + 1] : sCommandLine.GetLength()) - arrPositionToken[ui]));
		}

		// Normalize parts...
		for (xuint32 ui = 0; ui < arrPart.GetSize(); ui++)
		{
			arrPart[ui].TrimLeft();
			arrPart[ui].TrimRight();
		}

		for (xuint32 ui = 0; ui < arrPart.GetSize(); ui++)
		{
			XString sArgument;
			XString sArgumentOption;
			xcint32 iPositionOption = arrPart[ui].Find(t_Traits::GetTokenOption(), 0);
			if (iPositionOption != -1)
			{
				// Normalize argument option...
				sArgumentOption = arrPart[ui].Mid(iPositionOption, arrPart[ui].GetLength() - iPositionOption);
				sArgumentOption.TrimLeft();
				sArgumentOption.TrimRight();
			}

			// Normalize argument...
			sArgument = arrPart[ui].Left(iPositionOption == -1 ? arrPart[ui].GetLength() : iPositionOption);
			sArgument.TrimLeft();
			sArgument.TrimLeft(t_Traits::GetTokenArgument());
			sArgument.TrimRight();

			XCommandLineData d;
			d.sArgument = sArgument;
			d.sArgumentOption = sArgumentOption;

			m_arrData.Add(d);
		}

		// Check if argument options shall be parsed too...
		if (t_Traits::GetTokenOptionSeparator())
		{
			for (xuint32 ui = 0; ui < m_arrData.GetSize(); ui++)
			{
				// Check if we've something to do...
				if (!m_arrData[ui].sArgumentOption.GetLength())
					continue;

				// Concept: Argument options are separated by "t_Traits::GetTokenOptionSeparator()". If argument options are quoted (e.g. " ") and
				//  "t_Traits::GetTokenOptionSeparator()" is found between quotes the argument option is NOT separated...
				xresult xr = ParseArgumentOption(m_arrData[ui]);
				if (_XFailed(xr))
					return xr;
			}
		}
		else
		{
			// Normalize argument option (remove quotes if present)...
			for (xuint32 ui = 0; ui < m_arrData.GetSize(); ui++)
			{
				m_arrData[ui].sArgumentOption.TrimLeft();
				m_arrData[ui].sArgumentOption.TrimLeft(t_Traits::GetTokenQuote());
				m_arrData[ui].sArgumentOption.TrimLeft();

				m_arrData[ui].sArgumentOption.TrimRight();
				m_arrData[ui].sArgumentOption.TrimRight(t_Traits::GetTokenQuote());
				m_arrData[ui].sArgumentOption.TrimRight();
			}
		}

		return xsOK;
	}
		
	// Get count of arguments...
	inline xuint32 GetArgumentCount() const
	{
		return m_arrData.GetSize();
	}

	// Get a specific argument by index...
	inline const XCommandLineData* GetArgument(xcuint32 uiIndex)
	{
		_XAssert(uiIndex < m_arrData.GetSize());
		if (uiIndex >= m_arrData.GetSize())
			return NULL;

		return &m_arrData[uiIndex];
	}

	// Get a specific argument by argument...
	inline const XCommandLineData* GetArgument(xcchart* pszArgument)
	{
		_XAssert(pszArgument);
		if (!pszArgument)
			return NULL;

		for (xuint32 ui = 0; ui < m_arrData.GetSize(); ui++)
		{
			if (m_arrData[ui].sArgument.CompareNoCase(pszArgument) == 0)
			{
				return &m_arrData[ui];
			}
		}

		return NULL;
	}

	// Check if a specific argument is present...
	inline xbool HasArgument(xcchart* pszArgument) const
	{
		_XAssert(pszArgument);
		if (!pszArgument)
			return false;

		for (xuint32 ui = 0; ui < m_arrData.GetSize(); ui++)
		{
			if (m_arrData[ui].sArgument.CompareNoCase(pszArgument) == 0)
				return true;
		}

		return false;
	}

private:
// Internal functions...
	inline xresult ParseArgumentOption(XCommandLineData& d)
	{
		// Collect quote positions...
		XArrayT<xint32> arrPositionQuote;
		for (xint32 iPosition = d.sArgumentOption.Find(t_Traits::GetTokenQuote(), 0); iPosition != -1; iPosition = d.sArgumentOption.Find(t_Traits::GetTokenQuote(), iPosition + 1))
		{
			arrPositionQuote.Add(iPosition);
		}

		// Check for valid quotes...
		_XAssert(arrPositionQuote.GetSize() % 2 == 0);
		if (arrPositionQuote.GetSize() % 2 != 0)
			return xeUnexpected;

		// Collect separator positions...
		XArrayT<xint32> arrPositionSeparator;
		for (xint32 iPosition = d.sArgumentOption.Find(t_Traits::GetTokenOptionSeparator(), 0); iPosition != -1; iPosition = d.sArgumentOption.Find(t_Traits::GetTokenOptionSeparator(), iPosition + 1))
		{
			xbool bSeparator = true;
			for (xuint32 ui = 0; ui < arrPositionQuote.GetSize(); ui += 2)
			{
				// Check if separator is between quotes...
				if (iPosition > arrPositionQuote[ui] && iPosition < arrPositionQuote[ui + 1])
				{
					bSeparator = false;
					break;
				}
			}

			if (bSeparator)
			{
				arrPositionSeparator.Add(iPosition);
			}
		}

		xuint32 uiCount = arrPositionSeparator.GetSize();
		if (uiCount)
		{
			for (xuint32 ui = 0; ui <= uiCount; ui++)
			{
				xcint32 iFirst = ui == 0 ? 0 : arrPositionSeparator[ui - 1];
				xcint32 iCount = (ui < uiCount ? arrPositionSeparator[ui] : d.sArgumentOption.GetLength()) - iFirst;

				XString sArgumentOption = d.sArgumentOption.Mid(iFirst, iCount);
				sArgumentOption.TrimLeft();
				sArgumentOption.TrimLeft(t_Traits::GetTokenQuote());
				sArgumentOption.TrimLeft();

				sArgumentOption.TrimRight();
				sArgumentOption.TrimRight(t_Traits::GetTokenQuote());
				sArgumentOption.TrimRight();

				d.arrArgumentOption.Add(sArgumentOption);
			}
		}
		else
		{
			// Normalize argument option (remove quotes if present)...
			d.sArgumentOption.TrimLeft();
			d.sArgumentOption.TrimLeft(t_Traits::GetTokenQuote());
			d.sArgumentOption.TrimLeft();

			d.sArgumentOption.TrimRight();
			d.sArgumentOption.TrimRight(t_Traits::GetTokenQuote());
			d.sArgumentOption.TrimRight();

			d.arrArgumentOption.Add(d.sArgumentOption);
		}

		return xsOK;
	}

private:
// Internal data...
	XArrayT<XCommandLineData> m_arrData;
};

#ifdef X_WINDOWS
	typedef XCommandLineT<XCommandLineTraits::_Win32> XCommandLine;
#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XOperatingSystem: Singleton handling operating sytem information...
// ***************************************************************************************
class XOperatingSystem
{
private:
// Construction...
	inline XOperatingSystem()
	{
		_XMemoryZero(&m_osvi, sizeof(OSVERSIONINFO));
	}

public:
// Destruction...
	inline ~XOperatingSystem()
	{
	}

public:
// Operations...
	// Singleton access...
	inline static XOperatingSystem& GetOperatingSystem()
	{
		static XOperatingSystem s_OperatingSystem;

		return s_OperatingSystem;
	}

	// Get the operating system...
	inline xresult Init()
	{
		_XMemoryZero(&m_osvi, sizeof(OSVERSIONINFO));
		m_osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

		if (!::GetVersionEx(&m_osvi))
			return _XResultLastError;
		
		return xsOK;
	}

	// Get the operating system...
	inline xuint32 GetOperatingSystemInfo() const
	{
		xuint32 uiMask = 0;
		if (m_osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
		{
			uiMask |= XOperatingSystem_Platform_NT;

			if (m_osvi.dwMajorVersion == 3 && m_osvi.dwMinorVersion == 51)
				return uiMask |= XOperatingSystem_System_WinNT351;
			else if (m_osvi.dwMajorVersion == 4 && m_osvi.dwMinorVersion == 0)
				return uiMask |= XOperatingSystem_System_WinNT4;
			else if (m_osvi.dwMajorVersion == 5 && m_osvi.dwMinorVersion == 0)
				return uiMask |= XOperatingSystem_System_Win2K;
			else if (m_osvi.dwMajorVersion == 5 && m_osvi.dwMinorVersion == 1)
				return uiMask |= XOperatingSystem_System_WinXP;
			else if (m_osvi.dwMajorVersion == 5 && m_osvi.dwMinorVersion == 2)
				return uiMask |= XOperatingSystem_System_Win2K3;
		}
		else if (m_osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
		{
			uiMask |= XOperatingSystem_Platform_9x;

			if (m_osvi.dwMajorVersion == 4 && m_osvi.dwMinorVersion == 0)
				return uiMask |= XOperatingSystem_System_Win95;
			else if (m_osvi.dwMajorVersion == 4 && m_osvi.dwMinorVersion == 10)
				return uiMask |= XOperatingSystem_System_Win98;
			else if (m_osvi.dwMajorVersion == 4 && m_osvi.dwMinorVersion == 90)
				return uiMask |= XOperatingSystem_System_WinME;
		}

		return uiMask;
	}
	
	// Get the build number...
	inline xuint32 GetBuildNumber()
	{
		return (xuint32) m_osvi.dwBuildNumber;
	}

	// Get the operating system as string...
	inline xcchart* GetOperatingSystemString()
	{
		if (m_osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
		{
			if (m_osvi.dwMajorVersion == 3 && m_osvi.dwMinorVersion == 51)
				return __XT("Microsoft Windows NT 3.51");
			else if (m_osvi.dwMajorVersion == 4 && m_osvi.dwMinorVersion == 0)
				return __XT("Microsoft Windows NT 4.0");
			else if (m_osvi.dwMajorVersion == 5 && m_osvi.dwMinorVersion == 0)
				return __XT("Microsoft Windows 2000");
			else if (m_osvi.dwMajorVersion == 5 && m_osvi.dwMinorVersion == 1)
				return __XT("Microsoft Windows XP");
			else if (m_osvi.dwMajorVersion == 5 && m_osvi.dwMinorVersion == 2)
				return __XT("Microsoft Windows Server 2003");
		}
		else if (m_osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
		{
			if (m_osvi.dwMajorVersion == 4 && m_osvi.dwMinorVersion == 0)
				return __XT("Microsoft Windows 95");
			else if (m_osvi.dwMajorVersion == 4 && m_osvi.dwMinorVersion == 10)
				return __XT("Microsoft Windows 98");
			else if (m_osvi.dwMajorVersion == 4 && m_osvi.dwMinorVersion == 90)
				return __XT("Microsoft Windows Me");
		}

		return __XT("Unknown");
	}

	// Get the build number as string...
	inline XString GetBuildNumberString()
	{
		XString sBuildNumber;
		sBuildNumber.Format(__XT("%d"), m_osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS ? LOWORD(m_osvi.dwBuildNumber) : m_osvi.dwBuildNumber);

		return sBuildNumber;
	}

	// Get the service pack as string...
	inline XString GetServicePackString()
	{
		return m_osvi.szCSDVersion;
	}

	// Check if the current operating system is equal or newer than the one passed.
	//  We assume the following order (based on release date) here:
	//  - Windows NT 3.51
	//  - Windows 95
	//  - Windows NT 4.0
	//  - Windows 98
	//  - Windows 2K
	//  - Windows ME
	//  - Windows XP
	//  - Windows 2K3
	//  - Windows Vista
	inline xbool IsGreaterOrEqual(xcuint32 uiOperatingSystem)
	{
		xdword dwPlatform = 0;
		xdword dwMajor = 0;
		xdword dwMinor = 0;

		switch (uiOperatingSystem)
		{
			case XOperatingSystem_System_WinNT351:
				dwPlatform = VER_PLATFORM_WIN32_NT;
				dwMajor = 3;
				dwMinor = 51;
				break;
			case XOperatingSystem_System_WinNT4:
				dwPlatform = VER_PLATFORM_WIN32_NT;
				dwMajor = 4;
				dwMinor = 0;
				break;
			case XOperatingSystem_System_Win2K:
				dwPlatform = VER_PLATFORM_WIN32_NT;
				dwMajor = 5;
				dwMinor = 0;
				break;
			case XOperatingSystem_System_WinXP:
				dwPlatform = VER_PLATFORM_WIN32_NT;
				dwMajor = 5;
				dwMinor = 1;
				break;
			case XOperatingSystem_System_Win2K3:
				dwPlatform = VER_PLATFORM_WIN32_NT;
				dwMajor = 5;
				dwMinor = 2;
				break;
			case XOperatingSystem_System_Win95:
				dwPlatform = VER_PLATFORM_WIN32_WINDOWS;
				dwMajor = 4;
				dwMinor = 0;
				break;
			case XOperatingSystem_System_Win98:
				dwPlatform = VER_PLATFORM_WIN32_WINDOWS;
				dwMajor = 4;
				dwMinor = 10;
				break;
			case XOperatingSystem_System_WinME:
				dwPlatform = VER_PLATFORM_WIN32_WINDOWS;
				dwMajor = 4;
				dwMinor = 90;
				break;

			default:
				_XAssert(false);
				return false;
		}

		if (m_osvi.dwPlatformId == dwPlatform)
		{
			// Equal platform...
			if (m_osvi.dwMajorVersion > dwMajor)
			{
				return true;
			}
			else if (m_osvi.dwMajorVersion == dwMajor)
			{
				return m_osvi.dwMinorVersion >= dwMinor;
			}
			else
			{
				return false;
			}
		}
		else if (dwPlatform == VER_PLATFORM_WIN32_WINDOWS && m_osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
		{
			// This is a NT based system compared to a 9x based system...
			return (m_osvi.dwMajorVersion >= 4 && dwMinor <= 0) ||
				   (m_osvi.dwMajorVersion >= 5 && dwMinor <= 10) ||
				   (m_osvi.dwMajorVersion >= 5 && m_osvi.dwMinorVersion >= 1 && dwMinor <= 90);
		}
		else if (dwPlatform == VER_PLATFORM_WIN32_NT && m_osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
		{
			// This is a 9x based system compared to a NT based system...
			return (m_osvi.dwMinorVersion >= 0 && dwMajor <= 3) ||
				   (m_osvi.dwMajorVersion >= 10 && dwMajor <= 4) ||
				   (m_osvi.dwMajorVersion >= 90 && dwMajor <= 5 && dwMinor <= 0);
		}
		else
		{
			// Unknown...
			_XAssert(false);
		}

		return false;
	}

	// Check if we're running on the specified platform...
	inline xbool IsPlatform(xcuint32 uiPlatform)
	{
		return !!(GetOperatingSystemInfo() & uiPlatform);
	}

public:
// Shared Data...
	// Operating system...
	enum XOperatingSystem_System
	{
		XOperatingSystem_System_Win95		= 0x00000001,
		XOperatingSystem_System_Win98		= 0x00000002,
		XOperatingSystem_System_WinME		= 0x00000004,
		XOperatingSystem_System_WinNT351	= 0x00000008,
		XOperatingSystem_System_WinNT4		= 0x00000010,
		XOperatingSystem_System_Win2K		= 0x00000020,
		XOperatingSystem_System_WinXP		= 0x00000040,
		XOperatingSystem_System_Win2K3		= 0x00000080,
		XOperatingSystem_System_Vista		= 0x00000100
	};

	// Platform identifier...
	enum XOperatingSystem_Platform
	{
		XOperatingSystem_Platform_NT		= 0x10000000,
		XOperatingSystem_Platform_9x		= 0x20000000
	};

private:
// Internal Data...
	OSVERSIONINFO m_osvi;
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XModulePlatform: Platform traits for "XModuleT<>"
// ***************************************************************************************
class XModulePlatform : public XHandleModule
{
public:
// Operations...
	// Load...
	inline xresult Load(xcchart* pszFileName)
	{
		_XAssert(pszFileName);
		if (!pszFileName)
			return xeInvalidArgument;

		HMODULE hModule = ::LoadLibrary(pszFileName);

		if (hModule)
		{
			Attach(hModule);
		}

		return IsValid() ? xsOK : _XResultLastError;
	}

	// Free...
	inline xresult Free()
	{
		Release();

		return xsOK;
	}

	// Get file name...
	inline xresult GetFileName(xchart** pszFileName)
	{
		_XAssert(pszFileName);
		if (!pszFileName)
			return xeInvalidArgument;

		xcuint32 uiSize = _MAX_PATH * sizeof(xchart);

		xchart* psz = (xchart*) _xalloc(uiSize);
		if (!psz)
			return xeOutOfMemory;

		if (::GetModuleFileName(GetSafeType(), psz, uiSize / sizeof(xchart)))
		{
			*pszFileName = psz;

			return xsOK;
		}
		else
		{
			_xfree(psz);

			return _XResultLastError;
		}
	}

	// Get function pointer...
	inline xvoid* GetFunction(xchart* pszName)
	{
		_XAssert(pszName);
		if (!pszName)
			return NULL;

		_XConversion;
		return ::GetProcAddress(GetSafeType(), _XT2A(pszName));
	}
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XModuleT: Generic module...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XModuleT : public t_Platform
{
public:
// Construction/Destruction...
	inline XModuleT()
	{
	}

	inline ~XModuleT()
	{
		Free();
	}

public:
// Operations...
	// Load...
	inline xresult Load(xcchart* pszFileName)
	{
		_XAssert(pszFileName);
		if (!pszFileName)
			return xeInvalidArgument;

		return t_Platform::Load(pszFileName);
	}

	// Load...
	inline xresult Free()
	{
		return IsValid() ? t_Platform::Free() : xsFalse;
	}

	// Get file name...
	inline xresult GetFileName(xchart** pszFileName)
	{
		_XAssert(pszFileName);
		if (!pszFileName)
			return xeInvalidArgument;

		return t_Platform::GetFileName(pszFileName);
	}

	// Get function pointer...
	inline xvoid* GetFunction(xchart* pszName)
	{
		_XAssert(pszName);
		if (!pszName)
			return NULL;

		return t_Platform::GetFunction(pszName);
	}
};

// Platform type...
typedef XModuleT<XModulePlatform> XModule;



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XFileFindPlatform: Platform traits for "XFileFindT<>"
// ***************************************************************************************
class XFileFindPlatform
{
public:
// Construction...
	inline XFileFindPlatform()
	{
		_XMemoryZero(&m_fd, sizeof(WIN32_FIND_DATA));

		m_hFindFile = INVALID_HANDLE_VALUE;
	}

	inline ~XFileFindPlatform()
	{
		Clean();
	}

public:
// Operations...	
	// Start traversal...
	inline xresult First(xcchart* pszFileName)
	{
		Clean();

		_XAssert(pszFileName);
		if (!pszFileName)
			return xeInvalidArgument;		

		m_hFindFile = ::FindFirstFile(pszFileName, (LPWIN32_FIND_DATA) &m_fd);
		if (m_hFindFile == INVALID_HANDLE_VALUE)
			return _XResultLastError;

		return xsOK;
	}

	// Continue traversal...
	inline xresult Next()
	{
		return ::FindNextFile(m_hFindFile, &m_fd) ? xsOK : (::GetLastError() == ERROR_NO_MORE_FILES ? xeFalse : _XResultLastError);
	}

	inline xuint64 GetSize() const
	{
		ULARGE_INTEGER uliSize = {0, 0};
		uliSize.LowPart = m_fd.nFileSizeLow;
		uliSize.HighPart = m_fd.nFileSizeHigh;

		return uliSize.QuadPart;
	}

	inline XString GetName() const
	{
		return m_fd.cFileName;
	}

	inline XString GetTitle() const
	{
		XString s;

		_tsplitpath_s(GetName(),
					  NULL,
					  0,
					  NULL,
					  0,
					  s.GetBuffer(_MAX_PATH),
					  _MAX_PATH,
					  NULL,
					  0);

		s.ReleaseBuffer();
		
		return s;
	}

	inline xbool Match(xcuint32 uiFlags) const
	{
		return !!(m_fd.dwFileAttributes & uiFlags);
	}

private:
// Internal functions...
	xvoid Clean()
	{
		_XMemoryZero(&m_fd, sizeof(WIN32_FIND_DATA));

		if (m_hFindFile != INVALID_HANDLE_VALUE)
		{
			::FindClose(m_hFindFile);

			m_hFindFile = NULL;

		}
	}

private:
// Internal data...
	HANDLE m_hFindFile;
	WIN32_FIND_DATA m_fd;
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XFileFind: Find files...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XFileFindT : public t_Platform
{
public:
// Construction/Destruction...
	inline XFileFindT()
	{
	}

	inline ~XFileFindT()
	{
	}

public:
// Operations...
	// Get file name(s) and directory name(s) array based...
	xresult Find(xcchart* pszFileName, IXEnumString* pEnumFiles, IXEnumString* pEnumDirectories = NULL, xcuint32 uiFlags = ~0u, xcbool bRecursive = true)
	{
		_XAssert(pszFileName && (pEnumFiles || pEnumDirectories));
		if (!pszFileName || (!pEnumFiles && !pEnumDirectories))
			return xeInvalidArgument;

		// Get base file name...
		xchart szDrive[_MAX_DRIVE] = {'\0'};
		xchart szPath[_MAX_PATH] = {'\0'};
		xchart szFName[_MAX_FNAME] = {'\0'};
		xchart szExt[_MAX_EXT] = {'\0'};
		_tsplitpath_s(pszFileName,
					  szDrive,
					  _XCountCharT(szDrive), 
					  szPath,
					  _XCountCharT(szPath),
					  szFName,
					  _XCountCharT(szFName),
					  szExt,
					  _XCountCharT(szExt));

		XString sBase(szDrive);
		sBase += szPath;

		for (xresult xr = First(pszFileName); _XSucceeded(xr); xr = Next())
		{
			if (IsDirectory())
			{
				if (IsDots())
					continue;

				// Check if item matches mask...
				if (Match(uiFlags))
				{
					if (pEnumDirectories)
					{
						XString sDirectory(sBase);

						sDirectory += GetName();

						pEnumDirectories->Add(sDirectory);
					}
				}

				// Check if recursive traversal is requested...
				if (bRecursive)
				{
					XString sTemp;
					sTemp.Format(__XT("%s%s\\%s%s"), sBase, GetName(), szFName, szExt);

					XFileFind ff;
					xresult xr = ff.Find(sTemp, pEnumFiles, pEnumDirectories, uiFlags, bRecursive);
					if (_XFailed(xr))
						return xr;
				}
			}
			else
			{
				// Check if item matches mask...
				if (Match(uiFlags))
				{
					XString sTemp(sBase);
					sTemp += GetName();

					if (pEnumFiles)
					{
						pEnumFiles->Add(sTemp);
					}
				}
			}
		}

		return xsOK;
	}

	// Do your own traversal...
	inline xresult First(xcchart* pszFileName)
	{
		return t_Platform::First(pszFileName);
	}

	// Continue your own traversal...
	inline xresult Next()
	{
		return t_Platform::Next();
	}

public:
// Attributes...
	// Get the size...
	inline xuint64 GetSize() const
	{
		t_Platform::GetSize();
	}

	// Get complete file name...
	inline XString GetName() const
	{
		return t_Platform::GetName();
	}

	// Get file name without extension...
	inline XString GetTitle() const
	{
		return t_Platform::GetTitle();
	}
/*
	inline bool GetLastWriteTime(FILETIME& ft) const
	{
		_XAssert(m_hFileFind != INVALID_HANDLE_VALUE);
		return m_pWin32FindData ? (ft = ((LPWIN32_FIND_DATA) m_pWin32FindData)->ftLastWriteTime, true) : false;
	}

	inline bool GetLastAccessTime(FILETIME& ft) const
	{
		_XAssert(m_hFileFind != INVALID_HANDLE_VALUE);
		return m_pWin32FindData ? (ft = ((LPWIN32_FIND_DATA) m_pWin32FindData)->ftLastAccessTime, true) : false;
	}

	inline bool GetCreationTime(FILETIME& ft) const
	{
		_XAssert(m_hFileFind != INVALID_HANDLE_VALUE);
		return m_pWin32FindData ? (ft = ((LPWIN32_FIND_DATA) m_pWin32FindData)->ftCreationTime, true) : false;
	}
*/
	inline xbool Match(xcuint32 uiFlags) const
	{
		return t_Platform::Match(uiFlags);
	}

	inline xbool IsArchived() const
	{
		return Match(XFileAttributeArchive);
	}

	inline xbool IsCompressed() const
	{
		return Match(XFileAttributeCompressed);
	}

	inline xbool IsDirectory() const
	{
		return Match(XFileAttributeDirectory);
	}

	inline xbool IsHidden() const
	{
		return Match(XFileAttributeHidden);
	}

	inline xbool IsNormal() const
	{
		return Match(XFileAttributeNormal);
	}

	inline xbool IsReadOnly() const
	{
		return Matches(XFileAttributeReadOnly);
	}
		
	inline xbool IsSystem() const
	{
		return Match(XFileAttributeSystem);
	}
		
	inline xbool IsTemporary() const
	{
		return Match(XFileAttributeTemporary);
	}
		
	inline xbool IsDots() const
	{
		if (IsDirectory())
		{
			XString sName = GetName();

			return (sName.GetAt(0) == __XT('.') && sName.GetAt(1) == __XT('\0')) || (sName.GetAt(0) == __XT('.') && sName.GetAt(1) == __XT('.') && sName.GetAt(2) == __XT('\0'));
		}

		return false;
	}
};

// Platform type...
typedef XFileFindT<XFileFindPlatform> XFileFind;



// #######################################################################################



/*
#define XEventLog_RegPath __XT("System\\CurrentControlSet\\Services\\Eventlog\\Application\\")

// ***************************************************************************************
// XEventLog: Object encapsulating windows (NT) event logging...
// ***************************************************************************************
class XEventLog : public XRefCountTypeT<XEventLog::_Traits::_RefCount>
{
public:
// Construction/Destruction...
	inline XEventLog()
	{
	}

	inline ~XEventLog()
	{
	}

public:
// Operations...
	// Init event logging (register dll, etc.)...
	inline xresult Init(xcchart* pszApplication, xcchart* pszMessageDll)
	{
		// Close event log if already open...
		Release();

		// Register source...
		xresult xr = RegisterSource(pszApplication, pszMessageDll);
		if (_XFailed(xr))
			return xr;

		HANDLE h = ::RegisterEventSource(NULL, pszApplication);
		_XAssert(h);
		if (!h)
			return _XResultLastError;

		Attach(h);

		return xsOK;
	}

	// Write data to the event log...
	inline xresult Write(xcword wType, xcchart* pszMessage, xcdword dwEventID)
	{
		_XAssert(IsValid());
		if (!IsValid())
			return E_HANDLE;

		_XAssert(pszMessage);
		if (!pszMessage)
			return xeInvalidArgument;

		// Get our user name information...
		PSID pSID = NULL;
		xresult xr = GetUserSID(&pSID);
		if (_XFailed(xr))
			return xr;

		// Report the event...
		xbool bResult = (xbool) ::ReportEvent(m_hEventLog, wType, 0, dwEventID, pSID, 1, 0, &pszMessage, NULL);

		_xfree(pSID);

		return bResult ? xsOK : _XResultLastError;
	}

public:
// Shared data...
	// Traits for this object...
	struct _Traits
	{
		// Reference counting...
		struct _RefCount
		{
			typedef HANDLE Type;

			inline static Type InvalidValue()
			{
				return NULL;
			}

			inline static void AddRef(Type type)
			{
				// Nothing...
			}

			inline static void Release(Type type)
			{
				::DeregisterEventSource(type);
			}
		};
	};
private:
// Internal functions...
	inline xresult GetUserSID(PSID* pSID)
	{
		_XAssert(pSID);
		if (!pSID)
			return xeInvalidArgument;

		// Get calling thread's access token...
		XRefCountObject<XHandle::_Traits::_RefCount> hToken;
		if (!::OpenThreadToken(::GetCurrentThread(), TOKEN_QUERY, true, &hToken)) 
		{
			xcdword dwError = ::GetLastError();
			if (dwError != ERROR_NO_TOKEN)
				return _XResultFromWin32(dwError);

			// Retry for process token if no thread token exists...
			if (!::OpenProcessToken(::GetCurrentProcess(), TOKEN_QUERY, &hToken))
				return _XResultLastError;
		}

		// Get size of the user information in the token...
		xdword dwSize = 0;
		if (::GetTokenInformation(hToken, TokenUser, NULL, 0, &dwSize)) 
		{
			// Call should have failed due to zero-length buffer...
			return _XResultLastError;
		} 
		else 
		{
			// Call should have failed due to zero-length buffer...
			xcdword dwError = ::GetLastError();
			if (dwError != ERROR_INSUFFICIENT_BUFFER)
				return _XResultFromWin32(dwError);
		}

		// Allocate buffer for user information in the token...
		PTOKEN_USER pTokenUser = (PTOKEN_USER) _xalloc(dwSize);
		if (!pTokenUser)
			return xeOutOfMemory;

		// Retrieve the user information from the token...
		if (!::GetTokenInformation(hToken, TokenUser, pTokenUser, dwSize, &dwSize))
		{
			_xfree(pTokenUser);
			return _XResultLastError;
		}

		xcdword dwLength = ::GetLengthSid(pTokenUser->User.Sid);

		// Allocate buffer for SID...
		*pSID = (PSID) _xalloc(dwLength);
		if (!*pSID)
		{
			_xfree(pTokenUser);
			return xeOutOfMemory;
		}

		if (!::CopySid(dwLength, *pSID, pTokenUser->User.Sid))
		{
			_xfree(pTokenUser);
			_xfree(*pSID);
			*pSID = NULL;
			return xeOutOfMemory;
		}

		_xfree(pTokenUser);

		return xsOK;
	}

	inline xresult RegisterSource(xcchart* pszApplication, xcchart* pszMessageDll = NULL)
	{
		_XAssert(pszApplication && pszMessageDll);
		if (!pszApplication || !pszMessageDll)
			return xeInvalidArgument;

		XString sKey;
		sKey.Format(__XT("%s%s"), XEventLog_RegPath, pszApplication);

		// Open the registry event source key...
		HKEY hKey = NULL;
		xdword dwResult = 0;
		if (::RegCreateKeyEx(HKEY_LOCAL_MACHINE, sKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, &dwResult) != ERROR_SUCCESS)
			return _XResultLastError;

		// Write "EventMessageFile" key...
		if (::RegSetValueEx(hKey,
							__XT("EventMessageFile"),
							0,
							REG_SZ,
							(xcbyte*) pszMessageDll,
							(_xstrlent(pszMessageDll) + 1) * sizeof(xchart)) != ERROR_SUCCESS)
			return _XResultLastError;

		// Write "TypesSupported" key. We assume that message dll supports all types...
		DWORD dwTypesSupported = EVENTLOG_ERROR_TYPE |
								 EVENTLOG_WARNING_TYPE |
								 EVENTLOG_INFORMATION_TYPE |
								 EVENTLOG_AUDIT_SUCCESS |
								 EVENTLOG_AUDIT_FAILURE;

		if (::RegSetValueEx(hKey, __XT("TypesSupported"), 0, REG_DWORD, (xcbyte*) &dwTypesSupported, sizeof(xdword)) != ERROR_SUCCESS)
			return _XResultLastError;

		::RegCloseKey(hKey);

		return xsOK;
	}
};
*/



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XMemoryMapPlatform: Platform traits for "XMemoryMapT<>"
// ***************************************************************************************
class XMemoryMapPlatform : public XHandleFile
{
public:
// Operations...
	inline XMemoryMapPlatform()
	{
		m_bExist = false;
	}

	// Create...
	inline xresult Create(xcchart* pszName, xcuint32 uiSize)
	{
		_XAssert(pszName && uiSize);
		if (!pszName || !uiSize)
			return xeInvalidArgument;

		Destroy();

		Attach(::CreateFileMapping(INVALID_HANDLE_VALUE, 0, PAGE_READWRITE, 0, uiSize, pszName));

		xcbool bValid = IsValid();

		_XAssert(bValid);

		if (!bValid)
			return _XResultLastError;

		m_bExist = ::GetLastError() == ERROR_ALREADY_EXISTS;

		m_pBuffer = ::MapViewOfFile(GetSafeType(), FILE_MAP_ALL_ACCESS, 0, 0, uiSize);
		if (!m_pBuffer)
			return _XResultLastError;

		m_uiSize = uiSize;
	}

	// Check if exists...
	inline xbool Exist() const
	{
		return m_bExist;
	}

	// Destroy...
	inline xresult Destroy()
	{
		if (m_pBuffer)
		{
			::UnmapViewOfFile(m_pBuffer);

			m_pBuffer = NULL;
		}

		m_uiSize = 0;
	}

	// Get buffer...
	inline xvoid* GetBuffer()
	{
		_XAssert(m_pBuffer);

		return m_pBuffer;
	}

	// Get buffer...
	inline xcvoid* GetBuffer() const
	{
		_XAssert(m_pBuffer);
	
		return m_pBuffer;
	}

	// Get size...
	inline xuint32 GetSize() const
	{
		return m_uiSize;
	}

private:
// Internal data...
	xbool m_bExist;
	xvoid* m_pBuffer;
	xuint32 m_uiSize;
};

#else // X_WINDOWS
	#error "Missing definition..."
#endif



// #######################################################################################



// ***************************************************************************************
// XMemoryMapT: Generic memory mapping...
// ***************************************************************************************
template
<
	typename t_Platform
>
class XMemoryMapT : t_Platform
{
public:
// Construction/Destruction...
	inline XMemoryMapT()
	{
	}

	inline ~XMemoryMapT()
	{
	}

public:
// Operations...
	// Create...
	inline xresult Create(xcchart* pszName, xcuint32 uiSize)
	{
		return t_Platform::Create(pszName, uiSize);
	}

	// Check if exists...
	inline xbool Exist() const
	{
		return t_Platform::Exist();
	}

	// Destroy...
	inline xvoid Destroy()
	{
		return t_Platform::Destroy();
	}

public:
// Attributes...
	// Get buffer...
	inline xvoid* GetBuffer()
	{
		return t_Platform::GetBuffer();
	}

	// Get buffer...
	inline xcvoid* GetBuffer() const
	{
		return t_Platform::GetBuffer();
	}

	// Get size...
	inline xcuint32 GetSize() const
	{
		return t_Platform::GetSize();
	}
};

// Platform type...
typedef XMemoryMapT<XMemoryMapPlatform> XMemoryMap;



// #######################################################################################



// ***************************************************************************************
// XEnumInterfaceImpl: This object encapsulates an interface enumeration...
// ***************************************************************************************
class XEnumInterfaceImpl : public XRefCountT<IXEnumInterface>,
						   public XArrayT<XIPtrT<IXInterface> >
{
// Base class...
typedef XArrayT<XIPtrT<IXInterface> > _Base;

// Interface map...
XInterfaceMapBegin
	XInterfaceMapEntry(IXEnumInterface)
XInterfaceMapEnd

public:
	// Construction/Destruction...
	inline XEnumInterfaceImpl()
	{
	}

	inline ~XEnumInterfaceImpl()
	{
	}

public:
// Overridables...
	// *** IXEnum ***
	// Get count of strings...
	inline _XInterfaceMethod(xuint32, GetCount)()
	{
		return _Base::GetSize();
	}

	// Delete a string...
	inline _XInterfaceMethod(xresult, Delete)(xcuint32 uiIndex)
	{
		xcuint32 uiCount = GetCount();

		_XAssert(uiIndex < uiCount);
		if (uiIndex >= uiCount)
			return xeInvalidArgument;

		_Base::RemoveAt(uiIndex);

		return xsOK;
	}

	// Delete all strings...
	inline _XInterfaceMethod(xresult, DeleteAll)()
	{
		_Base::RemoveAll();

		return xsOK;
	}

	// *** IXEnumInterface ***
	// Get an interface...
	_XInterfaceMethod(xresult, Get)(xcuint32 uiIndex, IXInterface** pInterface)
	{
		xcuint32 uiCount = GetCount();

		_XAssert(uiIndex < uiCount && pInterface);
		if (uiIndex >= uiCount || !pInterface)
			return xeInvalidArgument;

		(*pInterface = (*this)[uiIndex])->AddRef();

		return xsOK;
	}

	// Get an interface...
	_XInterfaceMethod(xresult, Get)(xcuint32 uiIndex, xcguid& iid, IXInterface** pInterface)
	{
		xcuint32 uiCount = GetCount();

		_XAssert(uiIndex < uiCount && pInterface);
		if (uiIndex >= uiCount || !pInterface)
			return xeInvalidArgument;

		XIPtrT<IXInterface> p;
		xresult xr = _Base::GetAt(uiIndex)->QueryInterface(iid, (xvoid**) &p);
		if (_XFailed(xr))
			return xr;

		*pInterface = p.Detach();

		return xsOK;
	}

	// Add an interface...
	_XInterfaceMethod(xresult, Add)(IXInterface* pInterface)
	{
		_XAssert(pInterface);
		if (!pInterface)
			return xeInvalidArgument;
	
		_Base::Add(pInterface);

		return xsOK;
	}

	// Add an enum interface...
	_XInterfaceMethod(xresult, Add)(IXEnumInterface* pEnum)
	{
		_XAssert(pEnum);
		if (!pEnum)
			return xeInvalidArgument;

		xcuint32 uiCount = pEnum->GetCount();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			XIPtrT<IXInterface> p;
			xresult xr = pEnum->Get(ui, &p);
			if (_XFailed(xr) ||
				_XFailed(xr = Add(p)))
				return xr;
		}

		return xsOK;
	}

	// Insert an interface...
	inline _XInterfaceMethod(xresult, Insert)(xcuint32 uiIndex, IXInterface* pInterface)
	{
		_XAssert(pInterface);
		if (!pInterface)
			return xeInvalidArgument;

		// If insertion is beyond our scope we do an add...
		xcuint32 uiCount = GetCount();

		if (uiIndex >= uiCount)
		{
			return Add(pInterface);
		}
		else
		{
			_Base::InsertAt(uiIndex, pInterface);
		}

		return xsOK;
	}

	// Insert an enum interface...
	_XInterfaceMethod(xresult, Insert)(xcuint32 uiIndex, IXEnumInterface* pEnum)
	{
		_XAssert(pEnum);
		if (!pEnum)
			return xeInvalidArgument;

		xcuint32 uiCount = pEnum->GetCount();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			XIPtrT<IXInterface> p;
			xresult xr = pEnum->Get(ui, &p);
			if (_XFailed(xr) ||
				_XFailed(xr = Insert(uiIndex + ui, p)))
				return xr;
		}

		return xsOK;

	}

	// Delete an interface...
	inline _XInterfaceMethod(xresult, Delete)(IXInterface* pInterface)
	{
		_XAssert(pInterface);
		if (!pInterface)
			return xeInvalidArgument;

		xcuint32 uiCount = GetCount();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			if (_Base::GetAt(ui) == pInterface)
				return Delete(ui);
		}

		return xsFalse;
	}
};



// #######################################################################################



// ***************************************************************************************
// XEnumStringImpl: This object encapsulates a string enumeration...
// ***************************************************************************************
class XEnumStringImpl : public XRefCountT<IXEnumString>,
						public XArrayT<XString>
{
// Base class...
typedef XArrayT<XString> _Base;

// Interface map...
XInterfaceMapBegin
	XInterfaceMapEntry(IXEnumString)
XInterfaceMapEnd

public:
	// Construction/Destruction...
	inline XEnumStringImpl()
	{
	}

	inline XEnumStringImpl(xcchara* psz)
	{
		*this = psz;
	}

	inline XEnumStringImpl(xccharw* psz)
	{
		*this = psz;
	}

	inline ~XEnumStringImpl()
	{
	}

public:
// Overridables...
	// *** IXEnum ***
	// Get count of strings...
	inline _XInterfaceMethod(xuint32, GetCount)()
	{
		return _Base::GetSize();
	}

	// Delete a string...
	inline _XInterfaceMethod(xresult, Delete)(xcuint32 uiIndex)
	{
		xcuint32 uiCount = GetCount();

		_XAssert(uiIndex < uiCount);
		if (uiIndex >= uiCount)
			return xeInvalidArgument;

		_Base::RemoveAt(uiIndex);

		return xsOK;
	}

	// Delete all strings...
	inline _XInterfaceMethod(xresult, DeleteAll)()
	{
		_Base::RemoveAll();

		return xsOK;
	}

	// *** IXEnumString ***
	// Get a string...
	inline _XInterfaceMethod(xresult, Get)(xcuint32 uiIndex, xchari** psz)
	{
		xcuint32 uiCount = GetCount();

		_XAssert(uiIndex < uiCount && psz);
		if (uiIndex >= uiCount || !psz)
			return xeInvalidArgument;

		return (*psz = (*this)[uiIndex].DuplicateInterfaceString()) ? xsOK : xeOutOfMemory;
	}

	// Add a string...
	inline _XInterfaceMethod(xresult, Add)(xcchari* psz, xcbool bSplit)
	{
		_XAssert(psz);
		if (!psz)
			return xeInvalidArgument;

		XArrayT<xcchari*> arr;

		_XConversion;

		if (bSplit)
		{
			Split(psz, arr);
		}
		else
		{
			arr.Add(_XI2T(psz));
		}

		for (xuint32 ui = 0; ui < arr.GetSize(); ui++)
		{
			_Base::Add(arr[ui]);
		}

		return xsOK;
	}

	// Add an enum string...
	inline _XInterfaceMethod(xresult, Add)(IXEnumString* pEnum)
	{
		_XAssert(pEnum);
		if (!pEnum)
			return xeInvalidArgument;

		xcuint32 uiCount = pEnum->GetCount();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			XPtrT<xchari> psz;
			xresult xr = pEnum->Get(ui, &psz);
			if (_XFailed(xr) ||
				_XFailed(xr = Add(psz, false)))
				return xr;
		}

		return xsOK;
	}

	// Insert a string...
	inline _XInterfaceMethod(xresult, Insert)(xcuint32 uiIndex, xcchari* psz, xcbool bSplit)
	{
		_XAssert(psz);
		if (!psz)
			return xeInvalidArgument;

		// If insertion is beyond our scope we do an add...
		if (uiIndex >= GetSize())
		{
			return Add(psz, bSplit);
		}

		XArrayT<xcchari*> arr;

		_XConversion;

		if (bSplit)
		{
			Split(psz, arr);
		}
		else
		{
			arr.Add(_XI2T(psz));
		}

		for (xuint32 ui = 0; ui < arr.GetSize(); ui++)
		{
			_Base::InsertAt(uiIndex + ui, arr[ui]);
		}
	
		return xsOK;
	}

	// Insert an enum string...
	_XInterfaceMethod(xresult, Insert)(xcuint32 uiIndex, IXEnumString* pEnum)
	{
		_XAssert(pEnum);
		if (!pEnum)
			return xeInvalidArgument;

		xcuint32 uiCount = pEnum->GetCount();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			XPtrT<xchari> psz;
			xresult xr = pEnum->Get(ui, &psz);
			if (_XFailed(xr) ||
				_XFailed(xr = Insert(uiIndex + ui, psz, false)))
				return xr;
		}

		return xsOK;

	}

	// Delete a string...
	inline _XInterfaceMethod(xresult, Delete)(xcchari* psz)
	{
		_XAssert(psz);
		if (!psz)
			return xeInvalidArgument;

		xcuint32 uiCount = GetCount();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			if (_Base::GetAt(ui) == psz)
				return Delete(ui);
		}

		return xsFalse;
	}

	// Convert to a multi string...
	inline _XInterfaceMethod(xresult, Convert)(xchari** psz, xuint32* puiSize = NULL)
	{
		_XAssert(psz);
		if (!psz)
			return xeInvalidArgument;

		xcuint32 uiSize = GetLength() * sizeof(xchari);

		xchari* pszTemp = (xchari*) _xalloc(uiSize);
		if (!pszTemp)
			return xeOutOfMemory;

		xuint32 uiPos = 0;
		for (xuint32 ui = 0; ui < GetSize(); ui++)
		{
			xcuint32 uiTemp = (*this)[ui].GetLength();

			_XConversion;

			_XMemoryCopy(pszTemp + uiPos, _XT2I((xcchart*) (*this)[ui]), uiTemp * sizeof(xchari));

			uiPos += uiTemp + 1;
		}

		*psz = pszTemp;

		if (puiSize)
		{
			*puiSize = uiSize;
		}

		return xsOK;
	}

public:
// Operations...
	// Assignment...
	inline XEnumStringImpl& operator=(xcchara* psz)
	{
		_XAssert(psz);
		if (!psz)
			return *this;

		_Base::RemoveAll();

		XArrayT<xcchara*> arr;

		Split(psz, arr);

		for (UINT ui = 0; ui = arr.GetSize(); ui++)
		{
			_Base::Add(arr[ui]);
		}

		return *this;
	}

	inline XEnumStringImpl& operator=(xccharw* psz)
	{
		_XAssert(psz);
		if (!psz)
			return *this;

		_Base::RemoveAll();

		XArrayT<xccharw*> arr;

		Split(psz, arr);
		for (UINT ui = 0; ui < arr.GetSize(); ui++)
		{
			_Base::Add(arr[ui]);
		}

		return *this;
	}

	// Retrieve the size including all '\0' characters...
	inline xuint32 GetLength() const
	{
		xuint32 uiSize = 0;
		for (xuint32 ui = 0; ui < GetSize(); ui++)
		{
			uiSize += (*this)[ui].GetLength();
		}

		return (uiSize + GetSize() + 1);
	}

private:
// Internal functions...
	inline xuint32 Split(xcchara* psz, XArrayT<xcchara*>& arr) const
	{
		_XAssert(psz);
		if (!psz)
			return 0;

		xcchara* pszIterator = NULL;
		xuint32 uiCount = 0;
		for (pszIterator = psz; *pszIterator; pszIterator += strlen(pszIterator) + 1, uiCount++)
		{
			arr.Add(pszIterator);
		}

		return uiCount;
	}

	inline xuint32 Split(xccharw* psz, XArrayT<xccharw*>& arr) const
	{
		_XAssert(psz);
		if (!psz)
			return 0;

		xccharw* pszIterator = NULL;
		xuint32 uiCount = 0;
		for (pszIterator = psz; *pszIterator; pszIterator += wcslen(pszIterator) + 1, uiCount++)
		{
			arr.Add(pszIterator);
		}

		return uiCount;
	}
};



// #######################################################################################



// ***************************************************************************************
// XPropertyT: This object encapsulates a generic propery implementation...
// ***************************************************************************************
template
<
	typename t_Base
>
class XPropertyT : public t_Base
{
public:
// Construction/Destruction...
	inline XPropertyT()
	{

	}

	inline ~XPropertyT()
	{
		_Clean();
	}

protected:
// Operations...
	// Set string...
	inline xresult _SetString(xcuint32 uiID, xcchari* psz)
	{
		xcbool bValid = _XPropertyType(uiID) == XPropertyType_String;

		_XAssert(bValid && psz);
		if (!bValid || !psz)
			return xeInvalidArgument;

		return _SetData(uiID, psz, (xcuint32) (_xstrleni(psz) + 1) * sizeof(xchari));
	}

	// Get string...
	inline xresult _GetString(xcuint32 uiID, xchari** psz)
	{
		xcbool bValid = _XPropertyType(uiID) == XPropertyType_String;

		_XAssert(bValid && psz);
		if (!bValid || !psz)
			return xeInvalidArgument;

		xuint32 uiSize = 0;

		return _GetData(uiID, (xvoid**) psz, &uiSize);
	}

	// Set binary...
	inline xresult _SetBinary(xcuint32 uiID, xcvoid* pData, xcuint32 uiSize)
	{
		xcbool bValid = _XPropertyType(uiID) == XPropertyType_Binary;

		_XAssert(bValid && pData);
		if (!bValid || !pData)
			return xeInvalidArgument;

		return _SetData(uiID, pData, uiSize);
	}

	// Get binary...
	inline xresult _GetBinary(xcuint32 uiID, xvoid** pData, xuint32* puiSize)
	{
		xcbool bValid = _XPropertyType(uiID) == XPropertyType_Binary;

		_XAssert(bValid && pData && puiSize);
		if (!bValid || !pData || !puiSize)
			return xeInvalidArgument;

		return _GetData(uiID, pData, puiSize);
	}

	// Set interface...
	inline xresult _SetInterface(xcuint32 uiID, IXInterface* pInterface)
	{
		xcbool bValid = _XPropertyType(uiID) == XPropertyType_Interface;

		_XAssert(bValid && pInterface);
		if (!bValid || !pInterface)
			return xeInvalidArgument;

		return _SetData(uiID, pInterface, sizeof(IXInterface*));
	}

	// Get interface...
	inline xresult _GetInterface(xcuint32 uiID, IXInterface** pInterface)
	{
		xbool bValid = _XPropertyType(uiID) == XPropertyType_Interface;

		_XAssert(bValid && pInterface);
		if (!bValid || !pInterface)
			return xeInvalidArgument;

		xuint32 uiSize = 0;
		xresult xr = _GetData(uiID, (xvoid**) pInterface, &uiSize);
		if (_XFailed(xr))
			return xr;

		bValid = uiSize == sizeof(IXInterface*);

		_XAssert(bValid);

		return bValid ? xr : xeInternal;
	}

	// Set type...
	template <typename t_Type> inline xresult _SetType(xcuint32 uiID, const t_Type* pType)
	{
		xcbool bValid = _XPropertyType(uiID) == XPropertyType_Type;

		_XAssert(bValid && pType);
		if (!bValid || !pType)
			return xeInvalidArgument;

		return _SetData(uiID, pType, sizeof(t_Type));
	}

	// Set type...
	template <typename t_Type> inline xresult _SetType(xcuint32 uiID, const t_Type type)
	{
		xcbool bValid = _XPropertyType(uiID) == XPropertyType_Type;

		_XAssert(bValid);
		if (!bValid)
			return xeInvalidArgument;

		return _SetData(uiID, &type, sizeof(t_Type));
	}

	// Get type...
	template <typename t_Type> inline xresult _GetType(xcuint32 uiID, t_Type* pType)
	{
		xbool bValid = _XPropertyType(uiID) == XPropertyType_Type;

		_XAssert(bValid && pType);
		if (!bValid || !pType)
			return xeInvalidArgument;

		return _GetData(uiID, pType);
	}
	
private:
// Internal functions...
	inline xresult _SetData(xcuint32 uiID, xcvoid* pData, xcuint32 uiSize)
	{
		_XAssert(pData);
		if (!pData)
			return xeInvalidArgument;

		xvoid* p = NULL;

		if (_XPropertyType(uiID) == XPropertyType_Interface)
		{
			p = const_cast<xvoid*>(pData);

			static_cast<IXInterface*>(p)->AddRef();
		}
		else
		{
			p = _xalloc(uiSize);
			if (!p)
				return xeOutOfMemory;

			_XMemoryCopy(p, pData, uiSize);
		}

		_Data d = {uiID, (xbyte*) p, uiSize};

		xcuint32 uiCount = m_arr.GetSize();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			if (m_arr[ui].uiID == uiID)
			{
				_XAssert(m_arr[ui].pBuffer);

				if (_XPropertyType(m_arr[ui].uiID) == XPropertyType_Interface)
				{
					reinterpret_cast<IXInterface*>(m_arr[ui].pBuffer)->Release();
				}
				else
				{
					_xfree(m_arr[ui].pBuffer);
				}

				m_arr.RemoveAt(ui);

				m_arr.InsertAt(ui, d);

				return xsOK;
			}
		}

		m_arr.Add(d);

		return xsOK;
	}

	inline xresult _GetData(xcuint32 uiID, xvoid** pData, xuint32* puiSize)
	{
		_XAssert(pData && puiSize);
		if (!pData || !puiSize)
			return xeInvalidArgument;

		xcuint32 uiCount = m_arr.GetSize();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			if (m_arr[ui].uiID == uiID)
			{
				if (_XPropertyType(uiID) == XPropertyType_Interface)
				{
					reinterpret_cast<IXInterface*>(m_arr[ui].pBuffer)->AddRef();

					*pData = m_arr[ui].pBuffer;
				}
				else
				{
					xvoid* p = _xalloc(m_arr[ui].uiSize);
					if (!p)
						return false;

					_XMemoryCopy(p, m_arr[ui].pBuffer, m_arr[ui].uiSize);

					*pData = p;
				}

				*puiSize = m_arr[ui].uiSize;

				return xsOK;
			}
		}

		return xeFail;
	}

	template <typename t_Type> inline xresult _GetData(xcuint32 uiID, t_Type* pType)
	{
		xcbool bValid = _XPropertyType(uiID) == XPropertyType_Type;

		_XAssert(bValid && pType);
		if (!bValid || !pType)
			return xeInvalidArgument;

		XPtrT<xvoid> p;
		xuint32 uiSize = 0;

		xresult xr = _GetData(uiID, &p, &uiSize);
		if (_XFailed(xr))
			return xr;

		_XAssert(uiSize == sizeof(t_Type));
		if (uiSize != sizeof(t_Type))
			return xeInternal;

		_XMemoryCopy(pType, p, uiSize);

		return xsOK;
	}

	xvoid _Clean()
	{
		xcuint32 uiCount = m_arr.GetSize();

		for (xuint32 ui = 0; ui < uiCount; ui++)
		{
			_XAssert(m_arr[ui].pBuffer);

			if (m_arr[ui].pBuffer)
			{
				if (_XPropertyType(m_arr[ui].uiID) == XPropertyType_Interface)
				{
					reinterpret_cast<IXInterface*>(m_arr[ui].pBuffer)->Release();
				}
				else
				{			
					_xfree(m_arr[ui].pBuffer);
				}
			}
		}

		m_arr.RemoveAll();
	}

private:
// Internal data...
	struct _Data
	{
		xuint32 uiID;
		xbyte* pBuffer;
		xuint32 uiSize;
	};

	XArrayT<_Data> m_arr;
};



// #######################################################################################



// ***************************************************************************************
// Property map and entry definition macros for "IXProperty"...
// ***************************************************************************************
#define XPropertyMapBegin

#define XPropertyMapEntrySetString(__Name, __ID) \
public: \
	inline _XInterfaceMethod(xresult, __Name)(xcchari* psz) \
	{ \
		return _SetString(__ID, psz); \
	}

#define XPropertyMapEntryGetString(__Name, __ID) \
public: \
	inline _XInterfaceMethod(xresult, __Name)(xchari** psz) \
	{ \
		return _GetString(__ID, psz); \
	}

#define XPropertyMapEntrySetGetString(__NameSet, __NameGet, __ID) \
	XPropertyMapEntrySetString(__NameSet, __ID) \
	XPropertyMapEntryGetString(__NameGet, __ID)

#define XPropertyMapEntrySetBinary(__Name, __ID) \
public: \
	inline _XInterfaceMethod(xresult, __Name)(xcvoid* p, xcuint32 uiSize) \
	{ \
		return _SetBinary(__ID, p, uiSize); \
	}

#define XPropertyMapEntryGetBinary(__Name, __ID) \
public: \
	inline _XInterfaceMethod(xresult, __Name)(xvoid** p, xuint32* puiSize) \
	{ \
		return _GetBinary(__ID, p, puiSize); \
	}

#define XPropertyMapEntrySetGetBinary(__NameSet, __NameGet, __ID) \
	XPropertyMapEntrySetBinary(__NameSet, __ID) \
	XPropertyMapEntryGetBinary(__NameGet, __ID)


#define XPropertyMapEntrySetInterface(__Name, __Interface, __ID) \
public: \
	inline _XInterfaceMethod(xresult, __Name)(__Interface* p) \
	{ \
		return _SetInterface(__ID, p); \
	}

#define XPropertyMapEntryGetInterface(__Name, __Interface, __ID) \
public: \
	inline _XInterfaceMethod(xresult, __Name)(__Interface** p) \
	{ \
		return _GetInterface(__ID, (IXInterface**) p); \
	}

#define XPropertyMapEntrySetGetInterface(__NameSet, __NameGet, __Interface, __ID) \
	XPropertyMapEntrySetInterface(__NameSet, __Interface, __ID) \
	XPropertyMapEntryGetInterface(__NameGet, __Interface, __ID) \

#define XPropertyMapEntrySetType(__Name, __Type, __ID) \
public: \
	inline _XInterfaceMethod(xresult, __Name)(const __Type* pType) \
	{ \
		return _SetType(__ID, pType); \
	} \
	inline _XInterfaceMethod(xresult, __Name)(const __Type type) \
	{ \
		return _SetType(__ID, type); \
	}

#define XPropertyMapEntryGetType(__Name, __Type, __Default, __ID) \
public: \
	inline _XInterfaceMethod(xresult, __Name)(__Type* pType) \
	{ \
		return _GetType(__ID, pType); \
	} \
	inline _XInterfaceMethod(__Type, __Name)() \
	{ \
		__Type type = __Default; \
		return _XSucceeded(_GetType(__ID, &type)) ? type : __Default; \
	}
	
#define XPropertyMapEntrySetGetType(__NameSet, __NameGet, __Type, __Default, __ID) \
	XPropertyMapEntrySetType(__NameSet, __Type, __ID) \
	XPropertyMapEntryGetType(__NameGet, __Type, __Default, __ID)

#define XPropertyMapEnd



// #######################################################################################



#ifdef X_WINDOWS

// ***************************************************************************************
// XRegistry: Object encapsulating the registry.
// ***************************************************************************************
class XRegistry
{
public:
// Construction/Destruction...
	inline XRegistry()
	{
		m_hKey = NULL;
	}

	inline ~XRegistry()
	{
		Close();
	}
	
public:
// Operations...
	// Open a specified key..
	inline xresult Open(HKEY hKey, xcchart* pszKey, xcdword dwMask = KEY_READ)
	{
		if (m_hKey)
		{
			Close();
		}

		return _XResultFromWin32(::RegOpenKeyEx(hKey, pszKey, NULL, dwMask, &m_hKey));
	}
	// Close this object
	inline xvoid Close()
	{
		if (m_hKey)
		{
			::RegCloseKey(m_hKey);
			m_hKey = NULL;
		}
	}

	// Create a key...
	inline xresult CreateKey(HKEY hKey, xcchart* pszKey, xcdword dwOptions = REG_OPTION_NON_VOLATILE, xcdword dwMask = KEY_WRITE)
	{
		_XAssert(hKey && pszKey);
		if (!hKey || !pszKey)
			return xeInvalidArgument;

		HKEY hKeyCreate = NULL;
		xdword dwDisp = 0;
		xresult xr = _XResultFromWin32(::RegCreateKeyEx(hKey, pszKey, NULL, NULL, dwOptions, dwMask, NULL, &hKeyCreate, &dwDisp));
		if (_XFailed(xr))
			return xr;

		Close();

		m_hKey = hKeyCreate;

		return xr;
	}
	
	// Delete a single (and empty) key...
	inline xresult DeleteKey(xcchart* pszKey)
	{
		_XAssert(m_hKey);
		if (!m_hKey)
			return E_HANDLE;

		_XAssert(pszKey);
		if (!pszKey)
			return xeInvalidArgument;

		return _XResultFromWin32(::RegDeleteKey(m_hKey, pszKey));
	}

	// Delete a single value...
	inline xresult DeleteValue(xcchart* pszValue)
	{
		_XAssert(m_hKey);
		if (!m_hKey)
			return xePointer;

		_XAssert(pszValue);
		if (!pszValue)
			return xeInvalidArgument;

		return _XResultFromWin32(::RegDeleteValue(m_hKey, pszValue));
	}

	// Reading DWORDs...
	inline xresult ReadDword(xcchart* pszKey, xdword* pdwValue)
	{
		xbyte* pBuffer = NULL;
		xdword dwType = REG_DWORD;
		xdword dwSize = 0;
		xresult xr = ReadValue(pszKey, &pBuffer, &dwType, &dwSize);
		if (_XFailed(xr))
			return xr;

		_XMemoryCopy(pdwValue, pBuffer, dwSize);

		_xfree(pBuffer);

		return xr;
	}

	// Reading strings...
	inline xresult ReadString(xcchart* pszKey, xchart** pszValue)
	{
		_XAssert(pszValue);
		if (!pszValue)
			return xeInvalidArgument;

		xbyte* pBuffer = NULL;
		xdword dwType = REG_SZ;
		xdword dwSize = 0;
		xresult xr = ReadValue(pszKey, &pBuffer, &dwType, &dwSize);
		if (_XFailed(xr))
			return xr;

		XString s;
		_XMemoryCopy(s.GetBuffer(dwSize), pBuffer, dwSize);
		
		s.ReleaseBuffer();

		*pszValue = s.Duplicate();

		_xfree(pBuffer);

		return (*pszValue) ? xr : xeOutOfMemory;
	}

	// Reading multi strings...
	inline xresult ReadEnumString(xcchart* pszKey, IXEnumString* pEnumString)
	{
		_XAssert(pszKey && pEnumString);
		if (!pszKey || !pEnumString)
			return xeInvalidArgument;

		xbyte* pBuffer = NULL;
		xdword dwType = REG_MULTI_SZ;
		xdword dwSize = 0;
		xresult xr = ReadValue(pszKey, &pBuffer, &dwType, &dwSize);
		if (_XFailed(xr))
			return xr;

		xr = pEnumString->Add((xcchari*) pBuffer, true);

		_xfree(pBuffer);

		return xr;
	}

	inline xresult ReadEnumString(xcchart* pszKey, IXEnumString** pEnumString)
	{
		_XAssert(pszKey && pEnumString);
		if (!pszKey || !pEnumString)
			return xeInvalidArgument;

		XIPtrT<IXEnumString> p(_xnew XEnumStringImpl);
		if (!p)
			return xeOutOfMemory;

		xresult xr = ReadEnumString(pszKey, p);
		if (_XFailed(xr))
			return xr;

		*pEnumString = p.Detach();

		return xr;
	}

	// Reading binary stuff...
	inline xresult ReadBinary(xcchart* pszKey, xbyte** pValue, xdword* pdwSize = NULL)
	{
		xbyte* pBuffer = NULL;
		xdword dwType = REG_BINARY;
		xdword dwSize = 0;
		xresult xr = ReadValue(pszKey, &pBuffer, &dwType, &dwSize);
		if (_XFailed(xr))
			return xr;

		*pValue = pBuffer;

		if (pdwSize)
		{
			*pdwSize = dwSize;
		}

		return xr;
	}

	// Writing DWORDs...
	inline xresult WriteDword(xcchart* pszKey, xcdword dwValue)
	{
		return WriteValue(pszKey, (xcbyte*) &dwValue, REG_DWORD, sizeof(xdword));
	}

	// Writing strings...
	inline xresult WriteString(xcchart* pszKey, xcchart* pszValue)
	{
		return WriteValue(pszKey, (xcbyte*) pszValue, REG_SZ, (xdword) (_xstrlent(pszValue) + 1) * sizeof(xchart));
	}

	// Writing multi strings...
	inline xresult WriteEnumString(xcchart* pszKey, IXEnumString* pEnumString)
	{
		_XAssert(pszKey && pEnumString);
		if (!pszKey || !pEnumString)
			return xeInvalidArgument;

		XPtrT<xchari> pszValue;
		xuint32 uiSize = 0;
		xresult xr = pEnumString->Convert(&pszValue, &uiSize);
		if (_XFailed(xr))
			return xr;

		return WriteValue(pszKey, (xcbyte*) (xcchari*) pszValue, REG_MULTI_SZ, uiSize);
	}

	// Writing binary stuff...
	inline xresult WriteBinary(xcchart* pszKey, xcbyte* pValue, xcdword dwSize)
	{
		return WriteValue(pszKey, pValue, REG_BINARY, dwSize);
	}
	
	inline operator HKEY()
	{
		_XAssert(m_hKey);

		return m_hKey;
	}

	inline xbool HasKey(xcchart* pszKey)
	{
		_XAssert(m_hKey);
		if (!m_hKey)
			return false;

		_XAssert(pszKey);
		if (!pszKey)
			return false;

		HKEY hKey = NULL;
		xresult xr = _XResultFromWin32(::RegOpenKeyEx(m_hKey, pszKey, 0, KEY_READ, &hKey));
		if (_XFailed(xr))
			return false;

		::RegCloseKey(hKey);

		return true;
	}

	inline xbool HasValue(xcchart* pszKey, xdword* pdwType = NULL)
	{
		_XAssert(m_hKey);
		if (!m_hKey)
			return false;

		_XAssert(pszKey);
		if (!pszKey)
			return false;

		xdword dwType = 0;
		xdword dwSize = 0;
		if (_XFailed(_XResultFromWin32(::RegQueryValueEx(m_hKey, pszKey, NULL, &dwType, NULL, &dwSize))))
			return false;

		// Check type...
		return (pdwType && *pdwType != REG_NONE) ? *pdwType == dwType : true;
	}

	// Get number of subkeys...
	inline xdword GetCountSubKeys()
	{
		_XAssert(m_hKey);
		if (!m_hKey)
			return 0;

		xdword dwCount = 0;
		return _XSucceeded(_XResultFromWin32(::RegQueryInfoKey(m_hKey, NULL, NULL, NULL, &dwCount, NULL, NULL, NULL, NULL, NULL, NULL, NULL))) ? dwCount : 0;
	}

	inline xresult EnumKeys(IXEnumString** pEnumString)
	{
		_XAssert(pEnumString);
		if (!pEnumString)
			return xeInvalidArgument;

		_XAssert(m_hKey);
		if (!m_hKey)
			return xePointer;

		xdword dwLengthMax = 0;
		xdword dwCount = 0;
		xresult xr = _XResultFromWin32(::RegQueryInfoKey(m_hKey, NULL, NULL, NULL, &dwCount, &dwLengthMax, NULL, NULL, NULL, NULL, NULL, NULL));
		if (_XFailed(xr))
			return xr;

		XPtrT<xchart> psz((xchart*) _xalloc((dwLengthMax + 1) * sizeof(xchart)));
		if (!psz)
			return xeOutOfMemory;

		XIPtrT<IXEnumString> p(_xnew(XEnumStringImpl));
		if (!p)
			return xeOutOfMemory;

		xuint32 ui = 0;
		xlong lResult = 0;
		for (xuint32 ui = 0; ui < dwCount; ui++)
		{
			if (_XSucceeded(xr = _XResultFromWin32(lResult = ::RegEnumKey(m_hKey, ui, psz, dwLengthMax + 1))))
			{
				_XConversion;
				if (_XFailed(xr = p->Add(_XT2I(psz))))
					return xr;
			}
			else if (lResult == ERROR_NO_MORE_ITEMS)
			{
				break;
			}
			else
				return xr;
		}

		*pEnumString = p.Detach();

		return xsOK;
	}

private:
// Internal functions...
	inline xresult ReadValue(xcchart* pszKey, xbyte** pValue, xdword* pdwType = NULL, xdword* pdwSize = NULL)
	{
		_XAssert(m_hKey);
		if (!m_hKey)
			return xePointer;

		_XAssert(pszKey && pValue);
		if (!pszKey || !pValue)
			return xeInvalidArgument;

		xdword dwType = 0;
		xdword dwSize = 0;
		xresult xr = _XResultFromWin32(::RegQueryValueEx(m_hKey, pszKey, NULL, &dwType, NULL, &dwSize));
		if (_XFailed(xr))
			return xr;

		// Check type...
		if (pdwType && *pdwType != REG_NONE && *pdwType != dwType)
			return xeFail;

		xbyte* pBuffer = (xbyte*) _xalloc(dwSize);
		if (!pBuffer)
			return xeOutOfMemory;

		if (_XFailed(xr = _XResultFromWin32(::RegQueryValueEx(m_hKey, pszKey, NULL, &dwType, (xbyte*) pBuffer, &dwSize))))
		{
			_xfree(pBuffer);
		}
		else if (pdwSize)
		{
			*pdwSize = dwSize;
		}

		*pValue = pBuffer;

		return xr;
	}

	inline xresult WriteValue(xcchart* pszKey, xcbyte* pValue, xcdword dwType, xcdword dwSize)
	{
		_XAssert(m_hKey);
		if (!m_hKey)
			return E_HANDLE;

		_XAssert(pszKey && pValue);
		if (!pszKey || !pValue)
			return xeInvalidArgument;

		return _XResultFromWin32(::RegSetValueEx(m_hKey, pszKey, 0, dwType, pValue, dwSize));
	}

private:
// Internal data...
	HKEY m_hKey;
};

#endif // X_WINDOWS



// #######################################################################################



#ifdef X_WINDOWS

#define XShellExecute_AppPaths __XT("Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\")

// ***************************************************************************************
// XShellExecute: Object encapsulating the "ShellExecute()" shell API. This object does
//  not call "ShellExecute()" 'cause this call is slow (shell hooks are called, etc).
//  Instead the registry lookup is done manually and "CreateProcess()" API is used. This
//  limits this object to run processes at the moment...
// ***************************************************************************************
class XShellExecute
{
public:
// Operations...
	// Execute an application...
	inline static xresult Execute(xcchart* pszFileName, xcchart* pszCommandLine = NULL, xcbool bWaitFinish = false)
	{
		_XAssert(pszFileName);
		if (!pszFileName)
			return xeInvalidArgument;

		XString sFileName;
		if (_xstrrchrt(pszFileName, _XPathSeparator))
		{
			// Path information included -> we're fine here...
			sFileName = pszFileName;
		}
		else
		{
			// Format query...
			XString sKey(XShellExecute_AppPaths);
			sKey += pszFileName;

			// Read default value...
			XRegistry reg;
			XPtrT<xchart> pszFileName;
			xresult xr = reg.Open(HKEY_LOCAL_MACHINE, sKey);
			if (_XFailed(xr) ||
				_XFailed(xr = reg.ReadString(__XT(""), &pszFileName)))
				return xr;

			sFileName = pszFileName;
		}

		XString sExecute(sFileName);
		if (pszCommandLine)
		{
			sExecute += __XT(" ");
			sExecute += pszCommandLine;
		}

		// Launch process...
		STARTUPINFO si;
		_XMemoryZero(&si, sizeof(STARTUPINFO));
		si.cb = sizeof(STARTUPINFO);

		PROCESS_INFORMATION pi;
		_XMemoryZero(&pi, sizeof(PROCESS_INFORMATION));

		if (!::CreateProcess(NULL, sExecute.GetBuffer(sExecute.GetLength()), NULL, NULL, false, 0, NULL, NULL, &si, &pi))
			return _XResultLastError;

		_XAssert(pi.hProcess);

		if (pi.hProcess)
		{
			// Check if we shall wait for process completion...
			if (bWaitFinish)
			{
				::WaitForSingleObject(pi.hProcess, INFINITE);
			}

			::CloseHandle(pi.hProcess);
		}

		return xsOK;
	}
};

#endif // X_WINDOWS
